#ifndef BatchLab_Cloud_h
#define BatchLab_Cloud_h

// Inheritance :
#include <BatchLab/Core/BaseObject.h>
#include <Slash/Store/IStorable.h>
#include <Slash/Meta/IObject.h>

#include <AIDA/ICloud1D.h>
#include <AIDA/ICloud2D.h>
#include <AIDA/ICloud3D.h>

namespace AIDA {class IHistogram;}

#include <BatchLab/Core/Annotation.h>

namespace BatchLab {

class Histogram1D;
class Histogram2D;
class Histogram3D;

class BaseCloud : public BaseObject {
public:
  virtual void* cast(const std::string&) const;
protected:
  BaseCloud(Slash::Core::IWriter&,const std::string&,int);
  BaseCloud(const BaseCloud&);
public:
  virtual ~BaseCloud();
  bool isValid() const;
  Annotation& getAnnotation();
  const Annotation& getAnnotation() const;
  void setAnnotation(const Annotation&);
  std::vector<double>& ws(); //For IO (HDF5).
private:
  virtual void clear() = 0;
protected:
  Annotation fAnnotation;
  std::string fTitle;
  int fLimit;
  double fSw;
  std::vector<double> fWs;
};

class Cloud1D 
:public BaseCloud
,public virtual AIDA::ICloud1D 
,public virtual Slash::Store::IStorable
,public virtual Slash::Meta::IObject
{
public: //AIDA/IManagedObject
  virtual void* cast(const std::string&) const;
  virtual std::string type() const; //AIDA-v3r3p0
public: //AIDA/IBaseHistogram
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual bool reset();
  virtual int entries() const;
  virtual int nanEntries() const;
public: //AIDA/ICloud
  virtual double sumOfWeights() const;
  virtual bool convertToHistogram();
  virtual bool isConverted() const;
  virtual int maxEntries() const;
  virtual bool scale(double);
public: //AIDA/ICloud1D
  virtual bool fill(double,double = 1);
  virtual double lowerEdge() const;
  virtual double upperEdge() const;
  virtual double value(int) const;
  virtual double weight(int) const;
  virtual double mean() const;
  virtual double rms() const;
  virtual bool convert(int,double,double);
  virtual bool convert(const std::vector<double>&);
  virtual const AIDA::IHistogram1D& histogram() const;
  virtual bool fillHistogram(AIDA::IHistogram1D&) const;
  virtual bool setConversionParameters(int,double,double); //AIDA-v3r3p0
public: //Slash::Store::IObject
  virtual Slash::Store::IObject* copy() const;
  virtual std::string storeClassName() const;
  virtual void setName(const std::string&);
public: //IVisited
  virtual bool visit(Slash::Store::IConstVisitor&) const;
  virtual bool read(Slash::Store::IVisitor&);
public: //BatchLab::BaseCloud
  virtual void clear();
  bool setHistogram(Histogram1D*);
public:
  Cloud1D(Slash::Core::IWriter&,const std::string& = "");
  Cloud1D(Slash::Core::IWriter&,
          const std::string&,const std::string&,int=-1);
  Cloud1D(const Cloud1D&);
  virtual ~Cloud1D();
  std::vector<double>& xs(); //For IO (HDF5).
  void update(); //IO
  SLASH_STORE_HEADER
private:
  std::vector<double> fXs;
  double fLowerX;
  double fUpperX;
  double fSxw;
  double fSx2w;
  //
  int fCnvXnumber;
  double fCnvXmin;
  double fCnvXmax;
  Histogram1D* fHistogram;
};

class Cloud2D 
:public BaseCloud
,public virtual AIDA::ICloud2D
,public virtual Slash::Store::IStorable
,public virtual Slash::Meta::IObject
{
public: //AIDA/IManagedObject
  virtual void* cast(const std::string&) const;
  virtual std::string type() const; //AIDA-v3r3p0
public: //AIDA/IBaseHistogram
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual bool reset();
  virtual int entries() const;
  virtual int nanEntries() const;
public: //AIDA/ICloud
  virtual double sumOfWeights() const;
  virtual bool convertToHistogram();
  virtual bool isConverted() const;
  virtual int maxEntries() const;
  virtual bool scale(double);
public: //ICloud2D
  virtual bool fill(double,double,double = 1);
  virtual double lowerEdgeX() const;
  virtual double upperEdgeX() const;
  virtual double lowerEdgeY() const;
  virtual double upperEdgeY() const;
  virtual double valueX(int) const;
  virtual double valueY(int) const;
  virtual double weight(int) const;
  virtual double meanX() const;
  virtual double meanY() const;
  virtual double rmsX() const;
  virtual double rmsY() const;
  virtual bool convert(int,double,double,int,double,double);
  virtual bool convert(const std::vector<double>&,const std::vector<double>&);
  virtual const AIDA::IHistogram2D& histogram() const;
  virtual bool fillHistogram(AIDA::IHistogram2D&) const;
  virtual bool setConversionParameters(int,double,double,  
                                       int,double,double); //AIDA-v3r3p0
public: //Slash::Store::IObject
  virtual Slash::Store::IObject* copy() const;
  virtual std::string storeClassName() const;
  virtual void setName(const std::string&);
public: //IVisited
  virtual bool visit(Slash::Store::IConstVisitor&) const;
  virtual bool read(Slash::Store::IVisitor&);
public: //BatchLab::BaseCloud
  virtual void clear();
  bool setHistogram(Histogram2D*);
public:
  Cloud2D(Slash::Core::IWriter&,const std::string& = "");
  Cloud2D(Slash::Core::IWriter&,
          const std::string&,const std::string&,int=-1);
  Cloud2D(const Cloud2D&);
  virtual ~Cloud2D();
  std::vector<double>& xs(); //For IO (HDF5).
  std::vector<double>& ys(); //For IO (HDF5).
  void update(); //IO
  SLASH_STORE_HEADER
private:
  std::vector<double> fXs;
  std::vector<double> fYs;
  double fLowerX;
  double fUpperX;
  double fLowerY;
  double fUpperY;
  double fSxw;
  double fSx2w;
  double fSyw;
  double fSy2w;
  //
  int fCnvXnumber;
  double fCnvXmin;
  double fCnvXmax;
  int fCnvYnumber;
  double fCnvYmin;
  double fCnvYmax;
  Histogram2D* fHistogram;
};

class Cloud3D
:public BaseCloud
,public virtual AIDA::ICloud3D
,public virtual Slash::Store::IStorable
,public virtual Slash::Meta::IObject
{
public: //AIDA/IManagedObject
  virtual void* cast(const std::string&) const;
  virtual std::string type() const; //AIDA-v3r3p0
public: //AIDA/IBaseHistogram
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual bool reset();
  virtual int entries() const;
  virtual int nanEntries() const;
public: //AIDA/ICloud
  virtual double sumOfWeights() const;
  virtual bool convertToHistogram();
  virtual bool isConverted() const;
  virtual int maxEntries() const;
  virtual bool scale(double);
public: //AIDA/ICloud3D
  virtual bool fill(double,double,double,double = 1);
  virtual double lowerEdgeX() const;
  virtual double upperEdgeX() const;
  virtual double lowerEdgeY() const;
  virtual double upperEdgeY() const;
  virtual double lowerEdgeZ() const;
  virtual double upperEdgeZ() const;
  virtual double valueX(int) const;
  virtual double valueY(int) const;
  virtual double valueZ(int) const;
  virtual double weight(int) const;
  virtual double meanX() const;
  virtual double meanY() const;
  virtual double meanZ() const;
  virtual double rmsX() const;
  virtual double rmsY() const;
  virtual double rmsZ() const;
  virtual bool convert(int,double,double,int,double,double,int,double,double);
  virtual bool convert(const std::vector<double>&,
                       const std::vector<double>&,
                       const std::vector<double>&);
  virtual const AIDA::IHistogram3D& histogram() const;
  virtual bool fillHistogram(AIDA::IHistogram3D&) const;
  virtual bool setConversionParameters(int,double,double,
                                       int,double,double,
                                       int,double,double); //AIDA-v3r3p0
public: //Slash::Store::IObject
  virtual Slash::Store::IObject* copy() const;
  virtual std::string storeClassName() const;
  virtual void setName(const std::string&);
public: //IVisited
  virtual bool visit(Slash::Store::IConstVisitor&) const;
  virtual bool read(Slash::Store::IVisitor&);
public: //BatchLab::BaseCloud
  virtual void clear();
  bool setHistogram(Histogram3D*);
public:
  Cloud3D(Slash::Core::IWriter&,const std::string& = "");
  Cloud3D(Slash::Core::IWriter&,
          const std::string&,const std::string&,int=-1);
  Cloud3D(const Cloud3D&);
  virtual ~Cloud3D();
  std::vector<double>& xs(); //For IO (HDF5).
  std::vector<double>& ys(); //For IO (HDF5).
  std::vector<double>& zs(); //For IO (HDF5).
  void update(); //IO
  SLASH_STORE_HEADER
private:
  std::vector<double> fXs;
  std::vector<double> fYs;
  std::vector<double> fZs;
  double fLowerX;
  double fUpperX;
  double fLowerY;
  double fUpperY;
  double fLowerZ;
  double fUpperZ;
  double fSxw;
  double fSx2w;
  double fSyw;
  double fSy2w;
  double fSzw;
  double fSz2w;
  //
  int fCnvXnumber;
  double fCnvXmin;
  double fCnvXmax;
  int fCnvYnumber;
  double fCnvYmin;
  double fCnvYmax;
  int fCnvZnumber;
  double fCnvZmin;
  double fCnvZmax;
  Histogram3D* fHistogram;
};

}

#endif
