
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Arguments.h>
#include <Lib/Debug.h>

#include <Lib/ItemML.h>
#include <Lib/BaseML.h>

#define ACTION_COUNT "count"
#define ID_COUNT 1

class AIDA_ML : public Lib::BaseML {
public:
  AIDA_ML(Lib::ItemMLFactory& aFactory,
          int aVerbose,
          const std::string& aAction,
          Slash::Core::IWriter& aPrinter)
  :Lib::BaseML(aFactory,aVerbose)
  ,fPrinter(aPrinter),fAction(0)
  //
  ,f_objects(0)
  ,f_histogram1d(0)
  ,f_histogram2d(0)
  ,f_histogram3d(0)
  ,f_profile1d(0)
  ,f_profile2d(0)
  ,f_cloud1d(0)
  ,f_cloud2d(0)
  ,f_cloud3d(0)
  ,f_tuple(0)
  ,f_dataPointSet(0)
  ,f_function(0)
  {
    if(aAction==ACTION_COUNT) fAction = ID_COUNT;
  }
public:
  virtual bool visitEndElement(Lib::ItemML& aItemML,bool& aKeep) {
    if(fAction==ID_COUNT) {
      const std::string& tagName = aItemML.tagName();
      if(tagName=="aida") {
        aKeep = false;
      } else if(tagName=="histogram1d") {
        f_histogram1d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="histogram2d") {
        f_histogram2d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="histogram3d") {
        f_histogram3d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="profile1d") {
        f_profile1d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="profile2d") {
        f_profile2d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="cloud1d") {
        f_cloud1d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="cloud2d") {
        f_cloud2d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="cloud3d") {
        f_cloud3d++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="tuple") {
        f_tuple++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="dataPointSet") {
        f_dataPointSet++;
        f_objects++;
        aKeep = false;
      } else if(tagName=="function") {
        f_function++;
        f_objects++;
        aKeep = false;
      } else {
        aKeep = true;
      }
    } else {
      aKeep = false;
    }
    return true;
  }
  void dump(Slash::Core::IWriter& aPrinter) const { 
    if(fAction==ID_COUNT) {
      Lib::Out out(aPrinter);
      out << " AIDA objects " << f_objects << Lib::endl;
      out << " histogram1d = " << f_histogram1d << Lib::endl;
      out << " histogram2d = " << f_histogram2d << Lib::endl;
      out << " histogram3d = " << f_histogram3d << Lib::endl;
      out << " profile1d = " << f_profile1d << Lib::endl;
      out << " profile2d = " << f_profile2d << Lib::endl;
      out << " cloud1d = " << f_cloud1d << Lib::endl;
      out << " cloud2d = " << f_cloud2d << Lib::endl;
      out << " cloud3d = " << f_cloud3d << Lib::endl;
      out << " tuple = " << f_tuple << Lib::endl;
      out << " dataPointSet = " << f_dataPointSet << Lib::endl;
      out << " function = " << f_function << Lib::endl;
    }
  }
private:
  Slash::Core::IWriter& fPrinter;
  int fAction;
  //
  int f_objects;
  int f_histogram1d;
  int f_histogram2d;
  int f_histogram3d;
  int f_profile1d;
  int f_profile2d;
  int f_cloud1d;
  int f_cloud2d;
  int f_cloud3d;
  int f_tuple;
  int f_dataPointSet;
  int f_function;
};


//////////////////////////////////////////////////////////////////////////////
int main(
 int aArgc
,char** aArgv
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;{

  Lib::Arguments args(aArgc,aArgv);

  Lib::Out out(printer);

  if(args.isAnArgument("-h")) {
    out << "OS> Lib_xml.exe [options] <file>" << Lib::endl;
    out << " -h : help" << Lib::endl;
    out << " -action=<int> : action" << Lib::endl;
    return 0;
  }

  int verboseLevel = 0;
  std::string sverbose;
  args.find("-verbose",verboseLevel);

  std::string file;
  if(!args.file(file)) {
    out << "no XML aida file given." << Lib::endl;
    return 0;
  }

  std::string action;
  if(!args.find("-action",action)) action = "count";

  Lib::ItemMLFactory factory;
  AIDA_ML aidaML(factory,verboseLevel,action,printer);
  std::vector<std::string> tags;
  tags.push_back("aida");
  tags.push_back("histogram1d");
  tags.push_back("histogram2d");
  tags.push_back("histogram3d");
  tags.push_back("profile1d");
  tags.push_back("profile2d");
  tags.push_back("axis");
  tags.push_back("statistics");
  tags.push_back("data1d");
  tags.push_back("data2d");
  tags.push_back("data3d");
  tags.push_back("tuple");
  tags.push_back("columns");
  tags.push_back("rows");
  tags.push_back("row");
  tags.push_back("entryTuple");
  tags.push_back("cloud1d");
  tags.push_back("cloud2d");
  tags.push_back("cloud3d");
  tags.push_back("entries1d");
  tags.push_back("entries2d");
  tags.push_back("entries3d");
  tags.push_back("dataPointSet");
  tags.push_back("dataPoint");
  tags.push_back("function");
  tags.push_back("arguments");
  tags.push_back("argument");
  tags.push_back("parameters");
  tags.push_back("annotation");
  aidaML.setTags(tags);
  if(!aidaML.loadFile(file,false,printer)) {
    out << "Lib_xml : " 
        << " problem when loading file " << Lib::sout(file) << Lib::endl;
  } else { 
    aidaML.dump(printer);
  }


 }Lib::Debug::balance(printer);

  return 0;
}
