// this :
#include <BatchLab/Core/AIDA_Session.h>

#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Session.h>
#include <Lib/LibraryManager.h>
#include <Lib/System.h>
#include <Lib/smanip.h>
#include <Lib/Arguments.h>
#include <Lib/Debug.h>

#include <BatchLab/Core/AnalysisFactory.h>

AIDA::ITreeFactory* BatchLab::AIDA_Session::createTreeFactory() {
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTreeFactory();
}
AIDA::IHistogramFactory* BatchLab::AIDA_Session::createHistogramFactory(AIDA::ITree& aTree) {
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createHistogramFactory(aTree);
}
AIDA::ITupleFactory* BatchLab::AIDA_Session::createTupleFactory(AIDA::ITree& aTree) {
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTupleFactory(aTree);
}
AIDA::IFunctionFactory* BatchLab::AIDA_Session::createFunctionFactory(AIDA::ITree& aTree) {
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFunctionFactory(aTree);
}
AIDA::IPlotterFactory* BatchLab::AIDA_Session::createPlotterFactory(int aArgc,char** aArgv,const std::string& aImp,const std::string& aLib) {
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createPlotterFactory(aArgc,aArgv,aImp,aLib);
}
AIDA::IDataPointSetFactory* BatchLab::AIDA_Session::createDataPointSetFactory(AIDA::ITree& aTree) {
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createDataPointSetFactory(aTree);
}
AIDA::IFitFactory* BatchLab::AIDA_Session::createFitFactory() {
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFitFactory();
}
BatchLab::AIDA_Session::AIDA_Session()
:fPrinter(0)
,fSession(0)
,fAnalysisFactory(0)
,fVerboseLevel(0)
,fBalance(false)
{
    std::string sargs = Lib::System::getenv("BATCHLABARGS");

    Lib::Arguments args(sargs);

    std::string sverbose;
    if(args.find("-verbose",sverbose)) {
      if(sverbose=="") fVerboseLevel = 1;
    }

    if(args.isAnArgument("-balance")) {
      fBalance = true;
    }

    if(fVerboseLevel) {    
      Lib::Printer printer;  
      Lib::Out out(printer);
      out << "BatchLab::AIDA_Session::AIDA_Session :"
          << " " << this << Lib::endl;
    }

    if(args.isAnArgument("-debug_check_class")) {
      Lib::Debug::checkByClass(true);
    }

    Lib::Debug::increment("BatchLab::AIDA_Session");

    fPrinter = new Lib::Printer;

    fSession = new Lib::Session(*fPrinter);

    if(!fSession->findManager("LibraryManager")) {
      Lib::LibraryManager* libraryManager = 
        new Lib::LibraryManager(*fSession);
      fSession->addManager(libraryManager);
    }
 
    fAnalysisFactory = new BatchLab::AnalysisFactory(*fSession);
  }
  virtual ~AIDA_Session() {

    delete fAnalysisFactory;
    delete fSession;
    delete fPrinter;

    Lib::Debug::decrement("BatchLab::AIDA_Session");

    if(fBalance) {
      Lib::Printer printer;
      Lib::Debug::balance(printer);
    }

    if(fVerboseLevel) {
      Lib::Printer printer;  
      Lib::Out out(printer);
      out << "BatchLab::AIDA_Session::~AIDA_Session :"
          << " " << this << Lib::endl;
    }
}
bool BatchLab::AIDA_Session::isValid() const { return (fAnalysisFactory?true:false);}
