// this :
#include <BatchLab/Core/AnalysisFactory.h>

#include <Lib/Session.h>
#include <Lib/Debug.h>
#include <Lib/Manager.h>
#include <Lib/FunctionManager.h>
#include <Lib/Cast.h>
#include <Lib/Out.h>
#include <Lib/Dictionary.h>

#include <Slash/Core/ILibraryManager.h>
#include <Slash/Core/ILibrary.h>
#include <Slash/Store/ITree.h>

#include <BatchLab/Core/TreeFactory.h>
#include <BatchLab/Core/TupleFactory.h>
#include <BatchLab/Core/HistogramFactory.h>
#include <BatchLab/Core/FunctionFactory.h>
#include <BatchLab/Core/DataPointSetFactory.h>
#include <BatchLab/Core/FitFactory.h>
#include <BatchLab/Core/HistogramAllocator.h>

#include <AIDA/ITree.h>

//FIXME : is it the right place ?
#include <BatchLab/Core/Histogram.h> //FIXME
#include <BatchLab/Core/Cloud.h>
#include <BatchLab/Core/DataPointSet.h>
#include <BatchLab/Core/Function.h>
#include <BatchLab/Core/MemoryTuple.h>

namespace BatchLab {
  template <class T>
  Slash::Meta::IObject* allocate(Slash::Core::IWriter& aWriter){
    return new T(aWriter);
  }
}

#define ADD_ALLOCATOR(aDataClass) \
 {typedef Slash::Meta::IObject* (*F)(Slash::Core::IWriter&);\
  F f = BatchLab::allocate<aDataClass>;\
  manager->add(new Lib::Class(fPrinter,#aDataClass,1,f));}

//////////////////////////////////////////////////////////////////////////////
BatchLab::AnalysisFactory::AnalysisFactory(
 Slash::Core::ISession& aSession
)
:fSession(aSession)
,fPrinter(aSession.printer())
,fName("AnalysisFactory")
,fFunctionFactory(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::AnalysisFactory");
  if(!fSession.findManager("FunctionManager")) {
    fSession.addManager(new Lib::FunctionManager(aSession));
  }
  if(!fSession.findManager("BatchLab::HistogramAllocator")) {
    fSession.addManager(new BatchLab::HistogramAllocator(fPrinter));
  }
  // For the IO :
  if(!fSession.findManager("Lib::Dictionary")) {
    Lib::Dictionary* manager = new Lib::Dictionary(fPrinter);
    //FIXME : is it the right place ?
    ADD_ALLOCATOR(BatchLab::Histogram1D);
    ADD_ALLOCATOR(BatchLab::Histogram2D);
    ADD_ALLOCATOR(BatchLab::Histogram3D);
    ADD_ALLOCATOR(BatchLab::Profile1D);
    ADD_ALLOCATOR(BatchLab::Profile2D);
    ADD_ALLOCATOR(BatchLab::Cloud1D);
    ADD_ALLOCATOR(BatchLab::Cloud2D);
    ADD_ALLOCATOR(BatchLab::Cloud3D);
    ADD_ALLOCATOR(BatchLab::DataPointSet);

    ADD_ALLOCATOR(BatchLab::MemoryTuple);

    ADD_ALLOCATOR(BatchLab::Function::Gaussian);
    ADD_ALLOCATOR(BatchLab::Function::BreitWigner);
    ADD_ALLOCATOR(BatchLab::Function::ExpHBOOK);
    ADD_ALLOCATOR(BatchLab::Function::ExpAIDA);
    ADD_ALLOCATOR(BatchLab::Function::Polynomial);
    ADD_ALLOCATOR(BatchLab::Function::Scripted);
    ADD_ALLOCATOR(BatchLab::Function::Composite);
    ADD_ALLOCATOR(BatchLab::Function::Array);
    ADD_ALLOCATOR(BatchLab::Function::Array2D);
    ADD_ALLOCATOR(BatchLab::Function::ArrayComplex);
    ADD_ALLOCATOR(BatchLab::Function::Compiled);

    fSession.addManager(manager);
  }
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::AnalysisFactory::~AnalysisFactory(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fFunctionFactory;
  Lib::Debug::decrement("BatchLab::AnalysisFactory");
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::AnalysisFactory::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::AnalysisFactory::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IAnalysisFactory)
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
AIDA::ITreeFactory* BatchLab::AnalysisFactory::createTreeFactory(
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
 const std::string& //aOptions
#endif
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new TreeFactory(fSession);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITupleFactory* BatchLab::AnalysisFactory::createTupleFactory(
 AIDA::ITree& aTree
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
,const std::string& //aOptions
#endif
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::ITree* baseTree = Lib_Cast(aTree,Slash::Store::ITree);
  if(!baseTree) {
    Lib::Out out(fPrinter);
    out << "BatchLab::AnalysisFactory::createTupleFactory :"
        << " received tree is not a Slash::Store::ITree." << Lib::endl;
    return 0;
  }
  return new BatchLab::TupleFactory(fSession,*baseTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IHistogramFactory* BatchLab::AnalysisFactory::createHistogramFactory(
 AIDA::ITree& aTree
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
,const std::string& //aOptions
#endif
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::ITree* baseTree = Lib_Cast(aTree,Slash::Store::ITree);
  if(!baseTree) {
    Lib::Out out(fPrinter);
    out << "BatchLab::AnalysisFactory::createHistogramFactory :"
        << " received tree is not a Slash::Store::ITree." << Lib::endl;
    return 0;
  }
  return new BatchLab::HistogramFactory(fSession,*baseTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFunctionFactory* BatchLab::AnalysisFactory::createFunctionFactory(
 AIDA::ITree& aTree
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
,const std::string& //aOptions
#endif
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::ITree* baseTree = Lib_Cast(aTree,Slash::Store::ITree);
  if(!baseTree) {
    Lib::Out out(fPrinter);
    out << "BatchLab::AnalysisFactory::createFunctionFactory :"
        << " received tree is not a Slash::Store::ITree." << Lib::endl;
    return 0;
  }
  return new BatchLab::FunctionFactory(fSession,*baseTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterFactory* BatchLab::AnalysisFactory::createPlotterFactory(
 int aArgc
,char** aArgv
,const std::string& aImplementation
,const std::string& aLibrary
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
,const std::string& //aOptions
#endif
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( (aImplementation!="") && (aLibrary!="") ) {
    // Get a plotter factory from another implementation :
    std::string entry = aImplementation + "_createPlotterFactory";

    Slash::Core::ILibraryManager* libraryManager = 
      Lib_findManager(fSession,"LibraryManager",Slash::Core::ILibraryManager);
    if(!libraryManager) {
      Lib::Out out(fPrinter);
      out << "BatchLab::AnalysisFactory::createPlotterFactory :"
          << " LibraryManager not found." << Lib::endl;
      return 0;
    }
    Slash::Core::ILibrary* dll = libraryManager->addLibrary(aLibrary,aLibrary,"");
    if(!dll) {
      Lib::Out out(fPrinter);
      out << "BatchLab::AnalysisFactory::createPlotterFactory :"
          << " unable to add a DLL" << Lib::endl;
      return 0;
    }
    typedef AIDA::IPlotterFactory*(*GetFactoryProc)(int,char**);
    GetFactoryProc p = (GetFactoryProc)dll->findProcedure(entry);
    if(!p) {
      Lib::Out out(fPrinter);
      out << "BatchLab::AnalysisFactory::createPlotterFactory :"
          << " cannot get the entry point \"" << entry << "\"" 
          << "in \"" << aLibrary << "\"." << Lib::endl;
      return 0;
    }
    return p(aArgc,aArgv);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IDataPointSetFactory* BatchLab::AnalysisFactory::createDataPointSetFactory(
 AIDA::ITree& aTree
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
,const std::string& //aOptions
#endif
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::ITree* baseTree = Lib_Cast(aTree,Slash::Store::ITree);
  if(!baseTree) {
    Lib::Out out(fPrinter);
    out << "BatchLab::AnalysisFactory::createDataPointSetFactory :"
        << " received tree is not a Slash::Store::ITree." << Lib::endl;
    return 0;
  }
  return new BatchLab::DataPointSetFactory(fPrinter,*baseTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFitFactory* BatchLab::AnalysisFactory::createFitFactory(
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
 const std::string& //aOptions
#endif
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::ITree* memoryTree = 
    Lib_findManager(fSession,"MemoryTree",Slash::Store::ITree);
  if(!memoryTree) {
    Lib::Out out(fPrinter);
    out << "BatchLab::AnalysisFactory::createFitFactory :"
        << " MemoryTree not found." 
        << Lib::endl;
    return 0;
  }
  if(!fFunctionFactory) {
    fFunctionFactory = 
      new BatchLab::FunctionFactory(fSession,*memoryTree);
  }
  if(!fFunctionFactory) {
    Lib::Out out(fPrinter);
    out << "BatchLab::AnalysisFactory::createFitFactory :"
        << " can't get a FunctionFactory." 
        << Lib::endl;
    return 0;
  }
  return new BatchLab::FitFactory(fPrinter,fFunctionFactory);
}
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
AIDA::IGenericFactory* BatchLab::AnalysisFactory::createGenericFactory(
 const std::string& //aFactoryType
,const std::string& //aOptions
)
{
  return 0;
}
AIDA::IGenericFactory* BatchLab::AnalysisFactory::createManagedObjectGenericFactory(
 const std::string& //factoryType
,AIDA::ITree& //tree
,const std::string& //options
)
{
  return 0;
}
#endif
