// this :
#include <BatchLab/Core/ChainedTuple.h>

#include <Lib/Out.h>
#include <Lib/Cast.h>

#include <BatchLab/Core/amanip.h>

#include <math.h>

typedef unsigned char byte;

#define NotFound (-1)
#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
BatchLab::ChainedTuple::ChainedTuple(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<AIDA::ITuple*>& aTuples
)
:BaseTuple(aPrinter,aName,aTitle)
,fTuples(aTuples)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //Check that tuples are compatibles :
  unsigned int tuplen = aTuples.size();
  if(!tuplen) return;
  AIDA::ITuple* ref = aTuples[0];
  for(unsigned int tuplei=1;tuplei<tuplen;tuplei++) {
    AIDA::ITuple* tuple = aTuples[tuplei];
    if(!amanip::compatible_tuples(printer(),*ref,*tuple)) {
      Lib::Out out(printer());
      out << "BatchLab::ChainedTuple::ChainedTuple :"
          << " booking mismatch for tuple " << tuplei << "." 
          << Lib::endl;
      fTuples.clear();
      return;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::ChainedTuple::ChainedTuple(
 const ChainedTuple& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::ITuple(aFrom)
,AIDA::ITupleEntry(aFrom)
,BaseTuple(aFrom)
,fTuples(aFrom.fTuples)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::ChainedTuple::~ChainedTuple(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::ChainedTuple::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::ChainedTuple(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::ChainedTuple::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::ChainedTuple";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::ChainedTuple::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::ChainedTuple)
  else return BaseTuple::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::ChainedTuple::rows(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int rown = 0;
  unsigned int tuplen = fTuples.size();
  for(unsigned int tuplei=0;tuplei<tuplen;tuplei++) 
    rown += fTuples[tuplei]->rows();
  return rown;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::addRow(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return false;
  AIDA::ITuple* tuple = fTuples[tuplen-1];
  tuple->addRow();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::ITuple* BatchLab::ChainedTuple::findTuple(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  return tuple->getTuple(aColumn);
#else
  return tuple->findTuple(aColumn);
#endif
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITuple* BatchLab::ChainedTuple::findTuple(
 int aColumn
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  return tuple->getTuple(aColumn);
#else
  return tuple->findTuple(aColumn);
#endif
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::ITuple* BatchLab::ChainedTuple::getTuple(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return findTuple(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITuple* BatchLab::ChainedTuple::getTuple(
 int aColumn
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return findTuple(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::ITupleEntry* BatchLab::ChainedTuple::getObject(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  return tuple->getObject(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::ChainedTuple::findColumn(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return NotFound;
  AIDA::ITuple* ref = fTuples[0];
  return ref->findColumn(aName);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,float /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,double /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,bool /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,char /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,short /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,int /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,long /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
#else
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,AIDA::int64 /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,byte /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
#endif
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,const std::string& /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 int /*aColumn*/
,const AIDA::ITupleEntry& /*aValue*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 const std::vector<double>& /*aRow*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::fill(
 const std::vector<float>& /*aRow*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
double BatchLab::ChainedTuple::getDouble(
 int aColumn
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getDouble(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
float BatchLab::ChainedTuple::getFloat(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getFloat(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::ChainedTuple::getInt(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getInt(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
short BatchLab::ChainedTuple::getShort(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getShort(aColumn);
}
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
//////////////////////////////////////////////////////////////////////////////
long BatchLab::ChainedTuple::getLong(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getLong(aColumn);
}
#else
//////////////////////////////////////////////////////////////////////////////
AIDA::int64 BatchLab::ChainedTuple::getLong(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getLong(aColumn);
}
#endif
//////////////////////////////////////////////////////////////////////////////
char BatchLab::ChainedTuple::getChar(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getChar(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
unsigned char BatchLab::ChainedTuple::getByte(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  BaseTuple* baseTuple = Lib_Cast(*tuple,BatchLab::BaseTuple);
  if(!baseTuple) {
    Lib::Out out(printer());
    out << "BatchLab::ChainedTuple::getByte : "
        << " tuple not a BatchLab::BaseTuple."
        << Lib::endl;
    return 0;
  }
  tuple->setRow(row);
  return baseTuple->getByte(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::getBoolean(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getBoolean(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::ChainedTuple::getString(
 int aColumn
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return 0;
  tuple->setRow(row);
  return tuple->getString(aColumn);
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::ChainedTuple::columns(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return 0;
  AIDA::ITuple* ref = fTuples[0];
  return ref->columns();
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::ChainedTuple::columnName(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return "";
  AIDA::ITuple* ref = fTuples[0];
  return ref->columnName(aIndex);
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> BatchLab::ChainedTuple::columnNames(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return std::vector<std::string>();
  AIDA::ITuple* ref = fTuples[0];
  return ref->columnNames();
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::ChainedTuple::columnType(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return "";
  AIDA::ITuple* ref = fTuples[0];
  return ref->columnType(aIndex);
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> BatchLab::ChainedTuple::columnTypes(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return std::vector<std::string>();
  AIDA::ITuple* ref = fTuples[0];
  return ref->columnTypes();
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::ChainedTuple::resetRow(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = 0;
  int row = 0;
  if(!findRow(fRow,tuple,row)) return;
  return tuple->resetRow();
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::ChainedTuple::reset(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  unsigned int tuplen = fTuples.size();
  for(unsigned int tuplei=0;tuplei<tuplen;tuplei++) {
    AIDA::ITuple* tuple = fTuples[tuplei];
    tuple->reset();
  }
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::ChainedTuple::columnMin(
 int aColumn
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return 0;
  AIDA::ITuple* tuple = fTuples[0];
  double vmin = tuple->columnMin(aColumn);
  for(unsigned int tuplei=1;tuplei<tuplen;tuplei++) {
    AIDA::ITuple* tuple = fTuples[tuplei];
    if(tuple->rows()) {
      double v = tuple->columnMin(aColumn);
      vmin = MINIMUM(vmin,v);
    }
  }
  return vmin;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::ChainedTuple::columnMax(
 int aColumn
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  unsigned int tuplen = fTuples.size();
  if(!tuplen) return 0;
  AIDA::ITuple* tuple = fTuples[0];
  double vmax = tuple->columnMax(aColumn);
  for(unsigned int tuplei=1;tuplei<tuplen;tuplei++) {
    AIDA::ITuple* tuple = fTuples[tuplei];
    if(tuple->rows()) {
      double v = tuple->columnMax(aColumn);
      vmax = MAXIMUM(vmax,v);
    }
  }
  return vmax;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::ChainedTuple::columnMean(
 int aColumn
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int rown = 0;
  double sv = 0;
  unsigned int tuplen = fTuples.size();
  for(unsigned int tuplei=0;tuplei<tuplen;tuplei++) {
    AIDA::ITuple* tuple = fTuples[tuplei];
    int trown = tuple->rows();
    if(trown) {
      double tmean = tuple->columnMean(aColumn);
      double tsv = tmean * trown;
      rown += trown;
      sv += tsv;
    }
  }
  if(!rown) return 0;
  return sv/rown;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::ChainedTuple::columnRms(
 int aColumn
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  double sv = 0;
  double sv2 = 0;
  int rown = 0;
  unsigned int tuplen = fTuples.size();
  for(unsigned int tuplei=0;tuplei<tuplen;tuplei++) {
    AIDA::ITuple* tuple = fTuples[tuplei];
    int trown = tuple->rows();
    if(trown) {
      double tmean = tuple->columnMean(aColumn);
      double trms = tuple->columnRms(aColumn);
      // Find back tsv and tsv2 :
      double tsv = tmean * trown;
      double tsv2 = (trms * trms + tmean * tmean) * trown;
      rown += trown;
      sv += tsv;
      sv2 += tsv2;
    }
  }
  if(!rown) return 0;
  double mean = sv/((double)rown);
  double rms = ::sqrt(::fabs(sv2/((double)rown) - mean * mean));
  return rms;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::findRow(
 int aRow
,AIDA::ITuple*& aTuple
,int& aTupleRow
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int tuplen = fTuples.size();
  if(!tuplen) {
    aTuple = 0;
    aTupleRow = 0;
    return false;
  }
  int rown = 0;
  for(unsigned int tuplei=0;tuplei<tuplen;tuplei++) {
    AIDA::ITuple* tuple = fTuples[tuplei];
    int nrown = rown+tuple->rows();
    if((aRow>=rown)&&(aRow<nrown)) {
      aTuple = tuple;
      aTupleRow = aRow-rown;
      return true;
    } 
    rown = nrown;
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::ChainedTuple::visit(
 Slash::Store::IConstVisitor&
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
