// this :
#include <BatchLab/Core/DataPointSet.h>

#include <Lib/Cast.h>
#include <Lib/Sizes.h>

#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::DataPointSet::DataPointSet(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseObject(aPrinter,aName)
,fDimension(0)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::DataPointSet::DataPointSet(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aDimension
)
:BaseObject(aPrinter,aName)
,fDimension(aDimension)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setTitle(aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::DataPointSet::DataPointSet(
 const DataPointSet& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IDataPointSet(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Store::IArray(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseObject(aFrom)
,fAnnotation(aFrom.fAnnotation)
,fTitle(aFrom.fTitle)
,fDimension(aFrom.fDimension)
,fPoints(aFrom.fPoints)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::DataPointSet::~DataPointSet(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Annotation& BatchLab::DataPointSet::getAnnotation(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::DataPointSet::setAnnotation(
 const BatchLab::Annotation& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation = aFrom;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::DataPointSet::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::DataPointSet(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::DataPointSet::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::DataPointSet";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::DataPointSet::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IDataPointSet)
  else if_Lib_SCast(Slash::Store::IStorable)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::DataPointSet)
  else return BaseObject::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::DataPointSet::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IDataPointSet";
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::DataPointSet::title(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fTitle;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::setTitle(
 const std::string& aTitle
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fTitle = aTitle;
  fAnnotation.setValue("Title",aTitle);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IAnnotation& BatchLab::DataPointSet::annotation(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAnnotation& BatchLab::DataPointSet::annotation(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::DataPointSet::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fDimension;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::DataPointSet::clear(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPoints.clear();
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::DataPointSet::size(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fPoints.size();
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IDataPoint* BatchLab::DataPointSet::point(
 int aIndex
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aIndex<0) return 0;
  if(aIndex>=(int)fPoints.size()) return 0;
  return &(fPoints[aIndex]);
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IDataPoint* BatchLab::DataPointSet::point(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aIndex<0) return 0;
  if(aIndex>=(int)fPoints.size()) return 0;
  return &(fPoints[aIndex]);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IDataPoint* BatchLab::DataPointSet::addPoint(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fPoints.push_back(DataPoint(fDimension));
  const DataPoint& dataPoint = fPoints.back();
  return (AIDA::IDataPoint*)&dataPoint;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::addPoint(
 const AIDA::IDataPoint& aPoint
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fPoints.push_back(DataPoint(aPoint));
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::removePoint(
 int aIndex
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aIndex<0) return 0;
  if(aIndex>=(int)fPoints.size()) return 0;
  std::vector<DataPoint>::iterator it = fPoints.begin();
  for(int i=0;i<aIndex;i++) it++;
  fPoints.erase(it);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::DataPointSet::lowerExtent(
 int aCoordinate
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aCoordinate<0) return 0;           //FIXME return nan;
  if(aCoordinate>=fDimension) return 0; //FIXME return nan;
  if(fPoints.size()<=0) return 0;       //FIXME return nan;
  std::vector<DataPoint>::const_iterator it = fPoints.begin();
  const AIDA::IMeasurement* m = (*it).coordinate(aCoordinate);
  double value = m->value();
  ++it;
  for(;it!=fPoints.end();++it) {
    const AIDA::IMeasurement* m = (*it).coordinate(aCoordinate);
    value = MINIMUM(value,m->value());
  }
  return value;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::DataPointSet::upperExtent(
 int aCoordinate
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aCoordinate<0) return 0;           //FIXME return nan;
  if(aCoordinate>=fDimension) return 0; //FIXME return nan;
  if(fPoints.size()<=0) return 0;       //FIXME return nan;
  std::vector<DataPoint>::const_iterator it = fPoints.begin();
  const AIDA::IMeasurement* m = (*it).coordinate(aCoordinate);
  double value = m->value();
  ++it;
  for(;it!=fPoints.end();++it) {
    const AIDA::IMeasurement* m = (*it).coordinate(aCoordinate);
    value = MAXIMUM(value,m->value());
  }
  return value;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::scale(
 double aScaleFactor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  std::vector<DataPoint>::iterator it;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    for(int index=0;index<fDimension;index++) {
      AIDA::IMeasurement* m = (*it).coordinate(index);
      m->setValue(m->value() * aScaleFactor);
      m->setErrorPlus(m->errorPlus() * aScaleFactor);
      m->setErrorMinus(m->errorMinus() * aScaleFactor);
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::scaleValues(
 double aScaleFactor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  std::vector<DataPoint>::iterator it;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    for(int index=0;index<fDimension;index++) {
      AIDA::IMeasurement* m = (*it).coordinate(index);
      m->setValue(m->value() * aScaleFactor);
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::scaleErrors(
 double aScaleFactor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  std::vector<DataPoint>::iterator it;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    for(int index=0;index<fDimension;index++) {
      AIDA::IMeasurement* m = (*it).coordinate(index);
      m->setErrorPlus(m->errorPlus() * aScaleFactor);
      m->setErrorMinus(m->errorMinus() * aScaleFactor);
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::setCoordinate(
 int aCoordinate
,const std::vector<double>& aValues
,const std::vector<double>& aErrors
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aCoordinate<0) return false;           //FIXME return nan;
  if(aCoordinate>=fDimension) return false; //FIXME return nan;
  unsigned int number = aValues.size();
  if(!number) return false;
  if(aErrors.size()!=number) return false;
  if(fPoints.empty()){
    for(unsigned int index=0;index<number;index++) addPoint();
  }
  if(fPoints.size()!=number) return false;
  std::vector<DataPoint>::const_iterator it;
  unsigned int index = 0;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    AIDA::IMeasurement* m = (AIDA::IMeasurement*)(*it).coordinate(aCoordinate);
    m->setValue(aValues[index]);
    m->setErrorPlus(aErrors[index]);
    m->setErrorMinus(aErrors[index]);
    index++;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::setCoordinate(
 int aCoordinate
,const std::vector<double>& aValues
,const std::vector<double>& aErrorsPlus
,const std::vector<double>& aErrorsMinus
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aCoordinate<0) return false;           //FIXME return nan;
  if(aCoordinate>=fDimension) return false; //FIXME return nan;
  unsigned int number = aValues.size();
  if(!number) return false;
  if(aErrorsPlus.size()!=number) return false;
  if(aErrorsMinus.size()!=number) return false;
  if(fPoints.empty()){
    fPoints.resize(number,DataPoint(fDimension));
  }
  if(fPoints.size()!=number) return false;
  std::vector<DataPoint>::const_iterator it;
  unsigned int index = 0;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    AIDA::IMeasurement* m = (AIDA::IMeasurement*)(*it).coordinate(aCoordinate);
    m->setValue(aValues[index]);
    m->setErrorPlus(aErrorsPlus[index]);
    m->setErrorMinus(aErrorsMinus[index]);
    index++;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::setPoints(
 unsigned int aNumber
,const double* aData
)
//////////////////////////////////////////////////////////////////////////////
// Expected in aData :
//   aData[0 + 3 * icoord + 3 * dimension * ipoint] = value
//   aData[1 + 3 * icoord + 3 * dimension * ipoint] = error minus
//   aData[2 + 3 * icoord + 3 * dimension * ipoint] = error plus
// with :
//   icoord in [0,dimension[
//   ipoint in [0,aNumber[
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aNumber) return false;
  if(fPoints.empty()) fPoints.resize(aNumber,DataPoint(fDimension));
  if(fPoints.size()!=aNumber) return false;
  std::vector<DataPoint>::const_iterator it;
  double* data = (double*)aData;
  for(it=fPoints.begin();it!=fPoints.end();++it) {
    for(int icoord=0;icoord<fDimension;icoord++) {
      AIDA::IMeasurement* m = (AIDA::IMeasurement*)(*it).coordinate(icoord);
      m->setValue(*data);data++;
      m->setErrorMinus(*data);data++;
      m->setErrorPlus(*data);data++;
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<BatchLab::DataPoint>& BatchLab::DataPointSet::points(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPoints;
}
SLASH_STORE_SOURCE(BatchLab::DataPointSet)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::DataPointSet)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!aVisitor.visit("fAnnotation",fAnnotation)) return false;
  if(!aVisitor.visit("fTitle",fTitle)) return false;
  if(!aVisitor.visit("fDimension",fDimension)) return false;
  if(!aVisitor.visit_double("fPoints",*this)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!fAnnotation.read(aVisitor)) return false;
  if(!aVisitor.visit(fTitle)) return false;
  if(!aVisitor.visit(fDimension)) return false;
  if(!aVisitor.visit_double(*this)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IArray::Orders BatchLab::DataPointSet::orders(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<unsigned int> dims(3);
  dims[0] = fPoints.size();
  dims[1] = fDimension;
  dims[2] = 3;
  return dims;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::write(
 char*& aPos
,char* aEnd
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = fPoints.size();
  unsigned int dim = fDimension;
  unsigned int array_sz = number*dim*3*LIB_SIZE_DOUBLE;
  if((aPos+array_sz)>aEnd) return false;
  if(array_sz) {
    double* data = (double*)aPos;
    for(unsigned int index=0;index<number;index++) {
      const DataPoint& point = fPoints[index];
      for(unsigned int icoord=0;icoord<dim;icoord++) {
        const AIDA::IMeasurement* measurement = point.coordinate(icoord);
        *data = measurement->value();data++;
        *data = measurement->errorMinus();data++;
        *data = measurement->errorPlus();data++;
      }
    }
    aPos += array_sz;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::DataPointSet::read(
 const Slash::Store::IArray::Orders& aOrders
,char*& aPos
,char* aEnd
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int array_sz = 1;
  unsigned int dimn = aOrders.size();
  for(unsigned int dimi=0;dimi<dimn;dimi++) array_sz *= aOrders[dimi];
  array_sz *= LIB_SIZE_DOUBLE;
  if((aPos+array_sz)>aEnd) {
    fDimension = 0;
    fPoints.clear();
    return false;
  }
  fDimension = aOrders[1];
  unsigned int number = aOrders[0];
  fPoints.resize(number,DataPoint(fDimension));
  if(array_sz) {
    double* data = (double*)aPos;
    std::vector<DataPoint>::const_iterator it;
    for(it=fPoints.begin();it!=fPoints.end();++it) {
      for(int icoord=0;icoord<fDimension;icoord++) {
        AIDA::IMeasurement* m = (AIDA::IMeasurement*)(*it).coordinate(icoord);
        m->setValue(*data);data++;
        m->setErrorMinus(*data);data++;
        m->setErrorPlus(*data);data++;
      }
    }
    aPos += array_sz;
  }
  return true;
}
