// this :
#include <BatchLab/Core/Histogram.h>

#include <AIDA/IConstants.h>

#include <HCL/Histogram.h>

#include <Lib/Out.h>
#include <Lib/Cast.h>
#include <Lib/smanip.h>

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Axis::isFixedBinning() const {return fAxis.isFixedBinning();}
double BatchLab::Axis::lowerEdge() const {return fAxis.lowerEdge();}
double BatchLab::Axis::upperEdge() const {return fAxis.upperEdge();}
int BatchLab::Axis::bins() const {return fAxis.bins();}
double BatchLab::Axis::binLowerEdge(int aIndex) const {
  return fAxis.binLowerEdge(aIndex);}
double BatchLab::Axis::binUpperEdge(int aIndex) const {
  return fAxis.binUpperEdge(aIndex);}
double BatchLab::Axis::binWidth(int aIndex) const {
  return fAxis.binWidth(aIndex);}
double BatchLab::Axis::binCenter(int aIndex) const {
  return fAxis.binCenter(aIndex);
}
int BatchLab::Axis::coordToIndex(double aCoord) const {
  return fAxis.coordToIndex(aCoord);}

BatchLab::Axis::Axis() {}
BatchLab::Axis::Axis(const Axis& aFrom)
:AIDA::IAxis(aFrom)
,Slash::Store::IStorable(aFrom)
,fAxis(aFrom.fAxis){}
BatchLab::Axis::~Axis() {}
void BatchLab::Axis::copy(const HCL::Axis& aAxis) { fAxis = aAxis;}
const HCL::Axis& BatchLab::Axis::bareAxis() const { return fAxis;}

//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Axis::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IAxis)
  else if_Lib_SCast(Slash::Store::IStorable)
  else if_Lib_SCast(BatchLab::Axis)
  else return 0;
}
SLASH_STORE_SOURCE(BatchLab::Axis)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Axis::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Axis)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!aVisitor.visit("fOffset",fAxis.fOffset)) return false;
  if(!aVisitor.visit("fNumberOfBins",fAxis.fNumberOfBins)) return false;
  if(!aVisitor.visit("fMinimumValue",fAxis.fMinimumValue)) return false;
  if(!aVisitor.visit("fMaximumValue",fAxis.fMaximumValue)) return false;
  if(!aVisitor.visit("fFixed",fAxis.fFixed)) return false;
  if(!aVisitor.visit("fBinWidth",fAxis.fBinWidth)) return false;
  if(!aVisitor.visit("fEdges",fAxis.fEdges)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Axis::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!aVisitor.visit(fAxis.fOffset)) return false;
  if(!aVisitor.visit(fAxis.fNumberOfBins)) return false;
  if(!aVisitor.visit(fAxis.fMinimumValue)) return false;
  if(!aVisitor.visit(fAxis.fMaximumValue)) return false;
  if(!aVisitor.visit(fAxis.fFixed)) return false;
  if(!aVisitor.visit(fAxis.fBinWidth)) return false;
  if(!aVisitor.visit(fAxis.fEdges)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseHistogram::BaseHistogram(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseObject(aPrinter,aName)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseHistogram::BaseHistogram(
 const BaseHistogram& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,Slash::Store::IStorable(aFrom)
,BaseObject(aFrom)
,fAnnotation(aFrom.fAnnotation)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseHistogram::~BaseHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::BaseHistogram::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Slash::Store::IStorable)
  else if_Lib_SCast(BatchLab::BaseHistogram)
  else return BaseObject::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Annotation& BatchLab::BaseHistogram::getAnnotation(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const BatchLab::Annotation& BatchLab::BaseHistogram::getAnnotation(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseHistogram::setAnnotation(
 const BatchLab::Annotation& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation = aFrom;
}
//////////////////////////////////////////////////////////////////////////////
static bool visitHCL(
 const HCL::HistogramData& aData
,Slash::Store::IConstVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.visit("fTitle",aData.fTitle)) return false;
  if(!aVisitor.visit("fDimension",aData.fDimension)) return false;
  if(!aVisitor.visit("fBinNumber",aData.fBinNumber)) return false;
  if(!aVisitor.visit("fBinEntries",aData.fBinEntries)) return false;
  if(!aVisitor.visit("fBinSw",aData.fBinSw)) return false;
  if(!aVisitor.visit("fBinSw2",aData.fBinSw2)) return false;
  if(!aVisitor.visit("fBinSxw",aData.fBinSxw)) return false;
  if(!aVisitor.visit("fBinSx2w",aData.fBinSx2w)) return false;
  for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
    std::string name = "fAxes_" + Lib::smanip::tostring(iaxis);   
    BatchLab::Axis axis;
    axis.copy(aData.fAxes[iaxis]);
    if(!aVisitor.visit(name,axis)) return false;
  }
  if(!aVisitor.visit("fMode",aData.fMode)) return false;
  if(!aVisitor.visit("fProfile",aData.fProfile)) return false;
  if(!aVisitor.visit("fBinSvw",aData.fBinSvw)) return false;
  if(!aVisitor.visit("fBinSv2w",aData.fBinSv2w)) return false;
  if(!aVisitor.visit("fCutV",aData.fCutV)) return false;
  if(!aVisitor.visit("fMinV",aData.fMinV)) return false;
  if(!aVisitor.visit("fMaxV",aData.fMaxV)) return false;
  // Not written :
  //aData.fDoubles
  //aData.fInts
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool readHCL(
 HCL::HistogramData& aData
,Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.visit(aData.fTitle)) return false;
  if(!aVisitor.visit(aData.fDimension)) return false;
  if(!aVisitor.visit(aData.fBinNumber)) return false;
  if(!aVisitor.visit(aData.fBinEntries)) return false;
  if(!aVisitor.visit(aData.fBinSw)) return false;
  if(!aVisitor.visit(aData.fBinSw2)) return false;
  if(!aVisitor.visit(aData.fBinSxw)) return false;
  if(!aVisitor.visit(aData.fBinSx2w)) return false;
  aData.fAxes.clear();
  for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
    BatchLab::Axis baxis;
    if(!baxis.read(aVisitor)) return false;
    aData.fAxes.push_back(baxis.bareAxis());
  }
  if(!aVisitor.visit(aData.fMode)) return false;
  if(!aVisitor.visit(aData.fProfile)) return false;
  if(!aVisitor.visit(aData.fBinSvw)) return false;
  if(!aVisitor.visit(aData.fBinSv2w)) return false;
  if(!aVisitor.visit(aData.fCutV)) return false;
  if(!aVisitor.visit(aData.fMinV)) return false;
  if(!aVisitor.visit(aData.fMaxV)) return false;
  // Not written :
  //aData.fDoubles
  //aData.fInts

  aData.fDoubles.resize(aData.fDimension,0);
  aData.fInts.resize(aData.fDimension,0);
  return true;
}
SLASH_STORE_SOURCE(BatchLab::BaseHistogram)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseHistogram::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::BaseHistogram)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!aVisitor.visit("fAnnotation",fAnnotation)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseHistogram::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!fAnnotation.read(aVisitor)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram1D::Histogram1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram1D::Histogram1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const HCL::Histogram& aHclHisto
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create a valid histo :
  if(aHclHisto.getDimension()!=1) {
    Lib::Out out(printer());
    out << "BatchLab::Histogram1D::Histogram1D :"
        << " dimension mismatch." << Lib::endl;
  } else {
    fHistogram.copy(aHclHisto);
  }
  fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",fHistogram.title());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram1D::Histogram1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aXnumber,aXmin,aXmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram1D::Histogram1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdges
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aEdges)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram1D::Histogram1D(
 const Histogram1D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,Slash::Store::IStorable(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::IHistogram(aFrom)
,AIDA::IHistogram1D(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseHistogram(aFrom)
,fHistogram(aFrom.fHistogram)
,fAxis(aFrom.fAxis)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram1D::~Histogram1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Histogram1D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Histogram1D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram1D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Histogram1D";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Histogram1D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IHistogram1D)
  else if_Lib_SCast(AIDA::IHistogram)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::Histogram1D)
  else return BaseHistogram::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram1D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IHistogram1D";
}
//////////////////////////////////////////////////////////////////////////////
HCL::Histogram& BatchLab::Histogram1D::bareHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram& BatchLab::Histogram1D::bareHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram1D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram.isValid();
}
SLASH_STORE_SOURCE(BatchLab::Histogram1D)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram1D::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Histogram1D)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!BaseHistogram::visit(aVisitor)) return false;

  if(!visitHCL(fHistogram.getData(),aVisitor)) return false;

  //if(!aVisitor.visit("fAxis",fAxis)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram1D::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!BaseHistogram::read(aVisitor)) return false;

  if(!readHCL(fHistogram.getData(),aVisitor)) return false;

  fAxis.copy(fHistogram.getAxis(0));

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram1D& BatchLab::Histogram1D::hclHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram1D::fill(
 double aX
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool status = fHistogram.fill(aX,aWeight);
  if(status) emitUpdate();
  return status;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram1D::add(
 const AIDA::IHistogram1D& aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const BatchLab::Histogram1D* batchlabHisto = 
    Lib_ConstCast(aHisto,BatchLab::Histogram1D);
  if(!batchlabHisto) return false;
  fHistogram.add(batchlabHisto->bareHistogram());
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram1D::title() const {
  return fHistogram.title();}
bool BatchLab::Histogram1D::setTitle(const std::string& aTitle) {
  fAnnotation.setValue("Title",aTitle);
  return fHistogram.setTitle(aTitle);
}
AIDA::IAnnotation& BatchLab::Histogram1D::annotation() {
  return fAnnotation;}
const AIDA::IAnnotation& BatchLab::Histogram1D::annotation() const {
  return fAnnotation;}
int BatchLab::Histogram1D::dimension() const {
  return fHistogram.dimension();}
bool BatchLab::Histogram1D::reset() {
  return fHistogram.reset();}
int BatchLab::Histogram1D::entries() const {
  return fHistogram.entries();}
int BatchLab::Histogram1D::nanEntries() const { return 0;} //FIXME
int BatchLab::Histogram1D::allEntries() const {
  return fHistogram.allEntries();}
int BatchLab::Histogram1D::extraEntries() const {
  return fHistogram.extraEntries();}
double BatchLab::Histogram1D::equivalentBinEntries() const {
  return fHistogram.equivalentBinEntries();}
double BatchLab::Histogram1D::sumBinHeights() const {
  return fHistogram.sumBinHeights();}
double BatchLab::Histogram1D::sumAllBinHeights() const {
  return fHistogram.sumAllBinHeights();}
double BatchLab::Histogram1D::sumExtraBinHeights() const {
  return fHistogram.sumExtraBinHeights();}
double BatchLab::Histogram1D::minBinHeight() const {
  return fHistogram.minBinHeight();}
double BatchLab::Histogram1D::maxBinHeight() const {
  return fHistogram.maxBinHeight();}
bool BatchLab::Histogram1D::scale(double aScale) {
  return fHistogram.scale(aScale);}
double BatchLab::Histogram1D::binMean(int aIndex) const {
  return fHistogram.binMean(aIndex);}
double BatchLab::Histogram1D::binStdDev(int aIndex) const {
  return fHistogram.binStdDev(aIndex);}
int BatchLab::Histogram1D::binEntries(int aIndex) const {
  return fHistogram.binEntries(aIndex);}
double BatchLab::Histogram1D::binHeight(int aIndex) const {
  return fHistogram.binHeight(aIndex);}
double BatchLab::Histogram1D::binError(int aIndex) const {
  return fHistogram.binError(aIndex);}
double BatchLab::Histogram1D::mean() const {
  return fHistogram.mean();}
double BatchLab::Histogram1D::rms() const {
  return fHistogram.rms();}
const AIDA::IAxis& BatchLab::Histogram1D::axis() const {return fAxis;}
int BatchLab::Histogram1D::coordToIndex(double aCoord) const {
  return fHistogram.coordToIndex(aCoord);}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const HCL::Histogram& aHclHisto
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create a valid histo :
  if(aHclHisto.getDimension()!=1) {
    Lib::Out out(printer());
    out << "BatchLab::Profile1D::Profile1D :"
        << " dimension mismatch." << Lib::endl;
  } else {
    fHistogram.copy(aHclHisto);
  }
  fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",fHistogram.title());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aXnumber,aXmin,aXmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,double aVmin
,double aVmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aXnumber,aXmin,aXmax,aVmin,aVmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdges
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aEdges)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdges
,double aVmin
,double aVmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aEdges,aVmin,aVmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile1D::Profile1D(
 const Profile1D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,Slash::Store::IStorable(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::IProfile(aFrom)
,AIDA::IProfile1D(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseHistogram(aFrom)
,fHistogram(aFrom.fHistogram)
,fAxis(aFrom.fAxis)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile1D::~Profile1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Profile1D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Profile1D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Profile1D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Profile1D";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Profile1D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IProfile1D)
  else if_Lib_SCast(AIDA::IProfile)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::Profile1D)
  else return BaseHistogram::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Profile1D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IProfile1D";
}
//////////////////////////////////////////////////////////////////////////////
HCL::Histogram& BatchLab::Profile1D::bareHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram& BatchLab::Profile1D::bareHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile1D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram.isValid();
}
SLASH_STORE_SOURCE(BatchLab::Profile1D)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile1D::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Profile1D)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!BaseHistogram::visit(aVisitor)) return false;

  if(!visitHCL(fHistogram.getData(),aVisitor)) return false;

  //if(!aVisitor.visit("fAxis",fAxis)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile1D::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!BaseHistogram::read(aVisitor)) return false;

  if(!readHCL(fHistogram.getData(),aVisitor)) return false;

  fAxis.copy(fHistogram.getAxis(0));

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile1D::fill(
 double aX
,double aY
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool status = fHistogram.fill(aX,aY,aWeight);
  if(status) emitUpdate();
  return status;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile1D::add(
 const AIDA::IProfile1D& aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const BatchLab::Profile1D* batchlabHisto = 
    Lib_ConstCast(aHisto,BatchLab::Profile1D);
  if(!batchlabHisto) return false;
  fHistogram.add(batchlabHisto->bareHistogram());
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Profile1D::title() const {
  return fHistogram.title();}
bool BatchLab::Profile1D::setTitle(const std::string& aTitle) {
  fAnnotation.setValue("Title",aTitle);
  return fHistogram.setTitle(aTitle);
}
AIDA::IAnnotation& BatchLab::Profile1D::annotation() {
  return fAnnotation;}
const AIDA::IAnnotation& BatchLab::Profile1D::annotation() const {
  return fAnnotation;}
int BatchLab::Profile1D::dimension() const {
  return fHistogram.dimension();}
bool BatchLab::Profile1D::reset() {
  return fHistogram.reset();}
int BatchLab::Profile1D::entries() const {
  return fHistogram.entries();}
int BatchLab::Profile1D::nanEntries() const { return 0;} //FIXME
int BatchLab::Profile1D::allEntries() const {
  return fHistogram.allEntries();}
int BatchLab::Profile1D::extraEntries() const {
  return fHistogram.extraEntries();}
double BatchLab::Profile1D::equivalentBinEntries() const {
  return fHistogram.equivalentBinEntries();}
double BatchLab::Profile1D::sumBinHeights() const {
  return fHistogram.sumBinHeights();}
double BatchLab::Profile1D::sumAllBinHeights() const {
  return fHistogram.sumAllBinHeights();}
double BatchLab::Profile1D::sumExtraBinHeights() const {
  return fHistogram.sumExtraBinHeights();}
double BatchLab::Profile1D::minBinHeight() const {
  return fHistogram.minBinHeight();}
double BatchLab::Profile1D::maxBinHeight() const {
  return fHistogram.maxBinHeight();}
bool BatchLab::Profile1D::scale(double aScale) {
  return fHistogram.scale(aScale);}
double BatchLab::Profile1D::binMean(int aIndex) const {
  return fHistogram.binMean(aIndex);}
double BatchLab::Profile1D::binStdDev(int aIndex) const {
  return fHistogram.binStdDev(aIndex);}
int BatchLab::Profile1D::binEntries(int aIndex) const {
  return fHistogram.binEntries(aIndex);}
double BatchLab::Profile1D::binHeight(int aIndex) const {
  return fHistogram.binHeight(aIndex);}
double BatchLab::Profile1D::binError(int aIndex) const {
  return fHistogram.binError(aIndex);}
double BatchLab::Profile1D::binRms(int aIndex) const {
  return fHistogram.binRms(aIndex);}
double BatchLab::Profile1D::mean() const {
  return fHistogram.mean();}
double BatchLab::Profile1D::rms() const {
  return fHistogram.rms();}
const AIDA::IAxis& BatchLab::Profile1D::axis() const {return fAxis;}
int BatchLab::Profile1D::coordToIndex(double aCoord) const {
  return fHistogram.coordToIndex(aCoord);}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram2D::Histogram2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1,10,0,1)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram2D::Histogram2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const HCL::Histogram& aHclHisto
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1,10,0,1)
//////////////////////////////////////////////////////////////////////////////
// It is assumed that BatchLab::Histogram will take care of the aHclHisto.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create a valid histo :
  if(aHclHisto.getDimension()!=2) {
    Lib::Out out(printer());
    out << "Lba::Histogram2D::Histogram2D :"
        << " dimension mismatch." << Lib::endl;
  } else {
    fHistogram.copy(aHclHisto);
  }
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",fHistogram.title());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram2D::Histogram2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram2D::Histogram2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aEdgesX,aEdgesY)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram2D::Histogram2D(
 const Histogram2D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,Slash::Store::IStorable(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::IHistogram(aFrom)
,AIDA::IHistogram2D(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseHistogram(aFrom)
,fHistogram(aFrom.fHistogram)
,fAxisX(aFrom.fAxisX)
,fAxisY(aFrom.fAxisY)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram2D::~Histogram2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Histogram2D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Histogram2D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram2D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Histogram2D";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Histogram2D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IHistogram2D)
  else if_Lib_SCast(AIDA::IHistogram)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::Histogram2D)
  else return BaseHistogram::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram2D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IHistogram2D";
}
//////////////////////////////////////////////////////////////////////////////
HCL::Histogram& BatchLab::Histogram2D::bareHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram& BatchLab::Histogram2D::bareHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram2D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram.isValid();
}
SLASH_STORE_SOURCE(BatchLab::Histogram2D)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram2D::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Histogram2D)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!BaseHistogram::visit(aVisitor)) return false;

  if(!visitHCL(fHistogram.getData(),aVisitor)) return false;

  //if(!aVisitor.visit("fAxisX",fAxisX)) return false;
  //if(!aVisitor.visit("fAxisY",fAxisY)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram2D::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!BaseHistogram::read(aVisitor)) return false;

  if(!readHCL(fHistogram.getData(),aVisitor)) return false;

  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram2D& BatchLab::Histogram2D::hclHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram2D::fill(
 double aX
,double aY
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  bool status = fHistogram.fill(aX,aY,aWeight);
  if(status) emitUpdate();
  return status;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram2D::add(
 const IHistogram2D& aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const BatchLab::Histogram2D* batchlabHisto = 
    Lib_ConstCast(aHisto,BatchLab::Histogram2D);
  if(!batchlabHisto) return false;
  fHistogram.add(batchlabHisto->bareHistogram());
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram2D::title() const {
  return fHistogram.title();}
bool BatchLab::Histogram2D::setTitle(const std::string& aTitle) {
  fAnnotation.setValue("Title",aTitle);
  return fHistogram.setTitle(aTitle);
}
AIDA::IAnnotation& BatchLab::Histogram2D::annotation() {
  return fAnnotation;}
const AIDA::IAnnotation& BatchLab::Histogram2D::annotation() const {
  return fAnnotation;}
int BatchLab::Histogram2D::dimension() const {
  return fHistogram.dimension();}
bool BatchLab::Histogram2D::reset() {
  return fHistogram.reset();}
int BatchLab::Histogram2D::entries() const {
  return fHistogram.entries();}
int BatchLab::Histogram2D::nanEntries() const { return 0;} //FIXME
int BatchLab::Histogram2D::allEntries() const {
  return fHistogram.allEntries();}
int BatchLab::Histogram2D::extraEntries() const {
  return fHistogram.extraEntries();}
double BatchLab::Histogram2D::equivalentBinEntries() const {
  return fHistogram.equivalentBinEntries();}
double BatchLab::Histogram2D::sumBinHeights() const {
  return fHistogram.sumBinHeights();}
double BatchLab::Histogram2D::sumAllBinHeights() const {
  return fHistogram.sumAllBinHeights();}
double BatchLab::Histogram2D::sumExtraBinHeights() const {
  return fHistogram.sumExtraBinHeights();}
double BatchLab::Histogram2D::minBinHeight() const {
  return fHistogram.minBinHeight();}
double BatchLab::Histogram2D::maxBinHeight() const {
  return fHistogram.maxBinHeight();}
bool BatchLab::Histogram2D::scale(double aScale) {
  return fHistogram.scale(aScale);}
double BatchLab::Histogram2D::binMeanX(int aIndexX,int aIndexY) const { 
  return fHistogram.binMeanX(aIndexX,aIndexY);}
double BatchLab::Histogram2D::binMeanY(int aIndexX,int aIndexY) const { 
  return fHistogram.binMeanY(aIndexX,aIndexY);}
double BatchLab::Histogram2D::binStdDevX(int aIndexX,int aIndexY) const {
  return fHistogram.binStdDevX(aIndexX,aIndexY);}
double BatchLab::Histogram2D::binStdDevY(int aIndexX,int aIndexY) const {
  return fHistogram.binStdDevY(aIndexX,aIndexY);}
int BatchLab::Histogram2D::binEntries(int aIndexX,int aIndexY) const { 
  return fHistogram.binEntries(aIndexX,aIndexY);}
int BatchLab::Histogram2D::binEntriesX(int aIndexX) const { 
  return fHistogram.binEntriesX(aIndexX);}
int BatchLab::Histogram2D::binEntriesY(int aIndexY) const { 
  return fHistogram.binEntriesY(aIndexY);}
double BatchLab::Histogram2D::binHeight(int aIndexX,int aIndexY) const { 
  return fHistogram.binHeight(aIndexX,aIndexY);}
double BatchLab::Histogram2D::binHeightX(int aIndexX) const { 
  return fHistogram.binHeightX(aIndexX);}
double BatchLab::Histogram2D::binHeightY(int aIndexY) const{ 
  return fHistogram.binHeightY(aIndexY);}
double BatchLab::Histogram2D::binError(int aIndexX,int aIndexY) const{ 
  return fHistogram.binError(aIndexX,aIndexY);}
double BatchLab::Histogram2D::meanX() const{ 
  return fHistogram.meanX();}
double BatchLab::Histogram2D::meanY() const{ 
  return fHistogram.meanY();}
double BatchLab::Histogram2D::rmsX() const{ 
  return fHistogram.rmsX();}
double BatchLab::Histogram2D::rmsY() const{ 
  return fHistogram.rmsY();}
const AIDA::IAxis& BatchLab::Histogram2D::xAxis() const{ return fAxisX;}
const AIDA::IAxis& BatchLab::Histogram2D::yAxis() const{ return fAxisY;}
int BatchLab::Histogram2D::coordToIndexX(double aCoordX) const{ 
  return fHistogram.coordToIndexX(aCoordX);}
int BatchLab::Histogram2D::coordToIndexY(double aCoordY) const{ 
  return fHistogram.coordToIndexY(aCoordY);}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1,10,0,1)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const HCL::Histogram& aHclHisto
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1,10,0,1)
//////////////////////////////////////////////////////////////////////////////
// It is assumed that BatchLab::Histogram will take care of the aHclHisto.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create a valid histo :
  if(aHclHisto.getDimension()!=2) {
    Lib::Out out(printer());
    out << "Lba::Profile2D::Profile2D :"
        << " dimension mismatch." << Lib::endl;
  } else {
    fHistogram.copy(aHclHisto);
  }
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",fHistogram.title());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
,double aVmin
,double aVmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax,aVmin,aVmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aEdgesX,aEdgesY)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
,double aVmin
,double aVmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aEdgesX,aEdgesY,aVmin,aVmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile2D::Profile2D(
 const Profile2D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,Slash::Store::IStorable(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::IProfile(aFrom)
,AIDA::IProfile2D(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseHistogram(aFrom)
,fHistogram(aFrom.fHistogram)
,fAxisX(aFrom.fAxisX)
,fAxisY(aFrom.fAxisY)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Profile2D::~Profile2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Profile2D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Profile2D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Profile2D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Profile2D";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Profile2D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IProfile2D)
  else if_Lib_SCast(AIDA::IProfile)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::Profile2D)
  else return BaseHistogram::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Profile2D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IProfile2D";
}
//////////////////////////////////////////////////////////////////////////////
HCL::Histogram& BatchLab::Profile2D::bareHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram& BatchLab::Profile2D::bareHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile2D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram.isValid();
}
SLASH_STORE_SOURCE(BatchLab::Profile2D)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile2D::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Profile2D)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!BaseHistogram::visit(aVisitor)) return false;

  if(!visitHCL(fHistogram.getData(),aVisitor)) return false;

  //if(!aVisitor.visit("fAxisX",fAxisX)) return false;
  //if(!aVisitor.visit("fAxisY",fAxisY)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile2D::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!BaseHistogram::read(aVisitor)) return false;

  if(!readHCL(fHistogram.getData(),aVisitor)) return false;

  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile2D::fill(
 double aX
,double aY
,double aZ
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  bool status = fHistogram.fill(aX,aY,aZ,aWeight);
  if(status) emitUpdate();
  return status;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Profile2D::add(
 const IProfile2D& aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const BatchLab::Profile2D* batchlabHisto = 
    Lib_ConstCast(aHisto,BatchLab::Profile2D);
  if(!batchlabHisto) return false;
  fHistogram.add(batchlabHisto->bareHistogram());
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Profile2D::title() const {
  return fHistogram.title();}
bool BatchLab::Profile2D::setTitle(const std::string& aTitle) {
  fAnnotation.setValue("Title",aTitle);
  return fHistogram.setTitle(aTitle);
}
AIDA::IAnnotation& BatchLab::Profile2D::annotation() {
  return fAnnotation;}
const AIDA::IAnnotation& BatchLab::Profile2D::annotation() const {
  return fAnnotation;}
int BatchLab::Profile2D::dimension() const {
  return fHistogram.dimension();}
bool BatchLab::Profile2D::reset() {
  return fHistogram.reset();}
int BatchLab::Profile2D::entries() const {
  return fHistogram.entries();}
int BatchLab::Profile2D::nanEntries() const { return 0;} //FIXME
int BatchLab::Profile2D::allEntries() const {
  return fHistogram.allEntries();}
int BatchLab::Profile2D::extraEntries() const {
  return fHistogram.extraEntries();}
double BatchLab::Profile2D::equivalentBinEntries() const {
  return fHistogram.equivalentBinEntries();}
double BatchLab::Profile2D::sumBinHeights() const {
  return fHistogram.sumBinHeights();}
double BatchLab::Profile2D::sumAllBinHeights() const {
  return fHistogram.sumAllBinHeights();}
double BatchLab::Profile2D::sumExtraBinHeights() const {
  return fHistogram.sumExtraBinHeights();}
double BatchLab::Profile2D::minBinHeight() const {
  return fHistogram.minBinHeight();}
double BatchLab::Profile2D::maxBinHeight() const {
  return fHistogram.maxBinHeight();}
bool BatchLab::Profile2D::scale(double aScale) {
  return fHistogram.scale(aScale);}
double BatchLab::Profile2D::binMeanX(int aIndexX,int aIndexY) const { 
  return fHistogram.binMeanX(aIndexX,aIndexY);}
double BatchLab::Profile2D::binRms(int aIndexX,int aIndexY) const {
  return fHistogram.binRms(aIndexX,aIndexY);}
double BatchLab::Profile2D::binMeanY(int aIndexX,int aIndexY) const { 
  return fHistogram.binMeanY(aIndexX,aIndexY);}
double BatchLab::Profile2D::binStdDevX(int aIndexX,int aIndexY) const {
  return fHistogram.binStdDevX(aIndexX,aIndexY);}
double BatchLab::Profile2D::binStdDevY(int aIndexX,int aIndexY) const {
  return fHistogram.binStdDevY(aIndexX,aIndexY);}
int BatchLab::Profile2D::binEntries(int aIndexX,int aIndexY) const { 
  return fHistogram.binEntries(aIndexX,aIndexY);}
int BatchLab::Profile2D::binEntriesX(int aIndexX) const { 
  return fHistogram.binEntriesX(aIndexX);}
int BatchLab::Profile2D::binEntriesY(int aIndexY) const { 
  return fHistogram.binEntriesY(aIndexY);}
double BatchLab::Profile2D::binHeight(int aIndexX,int aIndexY) const { 
  return fHistogram.binHeight(aIndexX,aIndexY);}
double BatchLab::Profile2D::binHeightX(int aIndexX) const { 
  return fHistogram.binHeightX(aIndexX);}
double BatchLab::Profile2D::binHeightY(int aIndexY) const{ 
  return fHistogram.binHeightY(aIndexY);}
double BatchLab::Profile2D::binError(int aIndexX,int aIndexY) const{ 
  return fHistogram.binError(aIndexX,aIndexY);}
double BatchLab::Profile2D::meanX() const{ 
  return fHistogram.meanX();}
double BatchLab::Profile2D::meanY() const{ 
  return fHistogram.meanY();}
double BatchLab::Profile2D::rmsX() const{ 
  return fHistogram.rmsX();}
double BatchLab::Profile2D::rmsY() const{ 
  return fHistogram.rmsY();}
const AIDA::IAxis& BatchLab::Profile2D::xAxis() const{ return fAxisX;}
const AIDA::IAxis& BatchLab::Profile2D::yAxis() const{ return fAxisY;}
int BatchLab::Profile2D::coordToIndexX(double aCoordX) const{ 
  return fHistogram.coordToIndexX(aCoordX);}
int BatchLab::Profile2D::coordToIndexY(double aCoordY) const{ 
  return fHistogram.coordToIndexY(aCoordY);}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram3D::Histogram3D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1,10,0,1,10,0,1)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram3D::Histogram3D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const HCL::Histogram& aHclHisto
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1,10,0,1,10,0,1)
//////////////////////////////////////////////////////////////////////////////
// It is assumed that BatchLab::Histogram will take care of the aHclHisto.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create a valid histo :
  if(aHclHisto.getDimension()!=3) {
    Lib::Out out(printer());
    out << "BatchLab::Histogram3D::Histogram3D :"
        << " dimension mismatch." << Lib::endl;
  } else {
    fHistogram.copy(aHclHisto);
  }
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAxisZ.copy(fHistogram.getAxis(2));
  fAnnotation.setValue("Title",fHistogram.title());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram3D::Histogram3D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
,int aZnumber
,double aZmin
,double aZmax
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax,aZnumber,aZmin,aZmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAxisZ.copy(fHistogram.getAxis(2));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram3D::Histogram3D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
,const std::vector<double>& aEdgesZ
)
:BaseHistogram(aPrinter,aName)
,fHistogram(aTitle,aEdgesX,aEdgesY,aEdgesZ)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAxisZ.copy(fHistogram.getAxis(2));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram3D::Histogram3D(
 const Histogram3D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,Slash::Store::IStorable(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::IHistogram(aFrom)
,AIDA::IHistogram3D(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseHistogram(aFrom)
,fHistogram(aFrom.fHistogram)
,fAxisX(aFrom.fAxisX)
,fAxisY(aFrom.fAxisY)
,fAxisZ(aFrom.fAxisZ)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Histogram3D::~Histogram3D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Histogram3D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Histogram3D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram3D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Histogram3D";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Histogram3D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IHistogram3D)
  else if_Lib_SCast(AIDA::IHistogram)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::Histogram3D)
  else return BaseHistogram::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram3D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IHistogram3D";
}
//////////////////////////////////////////////////////////////////////////////
HCL::Histogram& BatchLab::Histogram3D::bareHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram& BatchLab::Histogram3D::bareHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram3D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram.isValid();
}
SLASH_STORE_SOURCE(BatchLab::Histogram3D)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram3D::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Histogram3D)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!BaseHistogram::visit(aVisitor)) return false;

  if(!visitHCL(fHistogram.getData(),aVisitor)) return false;

  //if(!aVisitor.visit("fAxisX",fAxisX)) return false;
  //if(!aVisitor.visit("fAxisY",fAxisY)) return false;
  //if(!aVisitor.visit("fAxisZ",fAxisY)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram3D::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!BaseHistogram::read(aVisitor)) return false;

  if(!readHCL(fHistogram.getData(),aVisitor)) return false;

  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAxisZ.copy(fHistogram.getAxis(2));

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram3D& BatchLab::Histogram3D::hclHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram3D::setTitle(const std::string& aTitle) {
  fAnnotation.setValue("Title",aTitle);
  return fHistogram.setTitle(aTitle);
}
AIDA::IAnnotation& BatchLab::Histogram3D::annotation() {
  return fAnnotation;}
const AIDA::IAnnotation& BatchLab::Histogram3D::annotation() const {
  return fAnnotation;}
int BatchLab::Histogram3D::dimension() const {
  return fHistogram.dimension();}
bool BatchLab::Histogram3D::reset() {
  return fHistogram.reset();}
int BatchLab::Histogram3D::entries() const {
  return fHistogram.entries();}
int BatchLab::Histogram3D::nanEntries() const { return 0;} //FIXME
int BatchLab::Histogram3D::allEntries() const {
  return fHistogram.allEntries();}
int BatchLab::Histogram3D::extraEntries() const {
  return fHistogram.extraEntries();}
double BatchLab::Histogram3D::equivalentBinEntries() const {
  return fHistogram.equivalentBinEntries();}
double BatchLab::Histogram3D::sumBinHeights() const {
  return fHistogram.sumBinHeights();}
double BatchLab::Histogram3D::sumAllBinHeights() const {
  return fHistogram.sumAllBinHeights();}
double BatchLab::Histogram3D::sumExtraBinHeights() const {
  return fHistogram.sumExtraBinHeights();}
double BatchLab::Histogram3D::minBinHeight() const {
  return fHistogram.minBinHeight();}
double BatchLab::Histogram3D::maxBinHeight() const {
  return fHistogram.maxBinHeight();}
bool BatchLab::Histogram3D::scale(double aScale) {
  return fHistogram.scale(aScale);}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram3D::fill(
 double aX
,double aY
,double aZ
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  bool status = fHistogram.fill(aX,aY,aZ,aWeight);
  if(status) emitUpdate();
  return status;
}
double BatchLab::Histogram3D::binMeanX(int aIndexX,int aIndexY,int aIndexZ) const { 
  return fHistogram.binMeanX(aIndexX,aIndexY,aIndexZ);}
double BatchLab::Histogram3D::binMeanY(int aIndexX,int aIndexY,int aIndexZ) const { 
  return fHistogram.binMeanY(aIndexX,aIndexY,aIndexZ);}
double BatchLab::Histogram3D::binMeanZ(int aIndexX,int aIndexY,int aIndexZ) const { 
  return fHistogram.binMeanZ(aIndexX,aIndexY,aIndexZ);}
double BatchLab::Histogram3D::binStdDevX(int aIndexX,int aIndexY,int aIndexZ) const{
  return fHistogram.binStdDevX(aIndexX,aIndexY,aIndexZ);}
double BatchLab::Histogram3D::binStdDevY(int aIndexX,int aIndexY,int aIndexZ) const{
  return fHistogram.binStdDevY(aIndexX,aIndexY,aIndexZ);}
double BatchLab::Histogram3D::binStdDevZ(int aIndexX,int aIndexY,int aIndexZ) const{
  return fHistogram.binStdDevZ(aIndexX,aIndexY,aIndexZ);}

int BatchLab::Histogram3D::binEntries(int aIndexX,int aIndexY,int aIndexZ) const { 
  return fHistogram.binEntries(aIndexX,aIndexY,aIndexZ);}
int BatchLab::Histogram3D::binEntriesX(int aIndexX) const { 
  return fHistogram.binEntriesX(aIndexX);}
int BatchLab::Histogram3D::binEntriesY(int aIndexY) const { 
  return fHistogram.binEntriesY(aIndexY);}
int BatchLab::Histogram3D::binEntriesZ(int aIndexZ) const { 
  return fHistogram.binEntriesZ(aIndexZ);}
double BatchLab::Histogram3D::binHeight(int aIndexX,int aIndexY,int aIndexZ) const { 
  return fHistogram.binHeight(aIndexX,aIndexY,aIndexZ);}
double BatchLab::Histogram3D::binHeightX(int aIndexX) const { 
  return fHistogram.binHeightX(aIndexX);}
double BatchLab::Histogram3D::binHeightY(int aIndexY) const{ 
  return fHistogram.binHeightY(aIndexY);}
double BatchLab::Histogram3D::binHeightZ(int aIndexZ) const{ 
  return fHistogram.binHeightZ(aIndexZ);}
double BatchLab::Histogram3D::binError(int aIndexX,int aIndexY,int aIndexZ) const{ 
  return fHistogram.binError(aIndexX,aIndexY,aIndexZ);}
double BatchLab::Histogram3D::meanX() const{ 
  return fHistogram.meanX();}
double BatchLab::Histogram3D::meanY() const{ 
  return fHistogram.meanY();}
double BatchLab::Histogram3D::meanZ() const{ 
  return fHistogram.meanZ();}
double BatchLab::Histogram3D::rmsX() const{ 
  return fHistogram.rmsX();}
double BatchLab::Histogram3D::rmsY() const{ 
  return fHistogram.rmsY();}
double BatchLab::Histogram3D::rmsZ() const{ 
  return fHistogram.rmsZ();}
const AIDA::IAxis& BatchLab::Histogram3D::xAxis() const{ return fAxisX;}
const AIDA::IAxis& BatchLab::Histogram3D::yAxis() const{ return fAxisY;}
const AIDA::IAxis& BatchLab::Histogram3D::zAxis() const{ return fAxisZ;}
int BatchLab::Histogram3D::coordToIndexX(double aCoordX) const{ 
  return fHistogram.coordToIndexX(aCoordX);}
int BatchLab::Histogram3D::coordToIndexY(double aCoordY) const{ 
  return fHistogram.coordToIndexY(aCoordY);}
int BatchLab::Histogram3D::coordToIndexZ(double aCoordZ) const{ 
  return fHistogram.coordToIndexZ(aCoordZ);}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Histogram3D::title(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram.title();
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Histogram3D::add(
 const IHistogram3D& aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const BatchLab::Histogram3D* batchlabHisto = 
    Lib_ConstCast(aHisto,BatchLab::Histogram3D);
  if(!batchlabHisto) return false;
  fHistogram.add(batchlabHisto->bareHistogram());
  return true;
}
