// this :
#include <BatchLab/Core/Main.h>

#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Session.h>
#include <Lib/LibraryManager.h>
#include <Lib/Arguments.h>
#include <Lib/Debug.h>
#include <Lib/System.h>
#include <Lib/smanip.h>

#include <BatchLab/Core/AnalysisFactory.h>

//////////////////////////////////////////////////////////////////////////////
BatchLab::Main::Main(
 const std::vector<std::string>& aArgs
)
:fPrinter(0)
,fSession(0)
,fAnalysisFactory(0)
,fVerboseLevel(0)
,fBalance(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Arguments args(aArgs);
 {std::string senv;
  if(Lib::System::getenv("BATCHLABARGS",senv)) {
    args.add(Lib::smanip::words(senv," "));
  }}


  std::string sverbose;
  if(args.find("-verbose",sverbose)) {
    if(sverbose=="") fVerboseLevel = 1;
  }

  if(args.isAnArgument("-balance")) {
    fBalance = true;
  }

  if(fVerboseLevel) {    
    Lib::Printer printer;  
    Lib::Out out(printer);
    out << "BatchLab::Main::Main :"
        << " " << this << Lib::endl;
  }

  if(args.isAnArgument("-debug_check_class")) {
    Lib::Debug::checkByClass(true);
  }

  Lib::Debug::increment("BatchLab::Main");

  fPrinter = new Lib::Printer;

 {Lib::Session* session = new Lib::Session(*fPrinter);
  session->setArguments(args.args());
  fSession = session;}

  if(!fSession->findManager("LibraryManager")) {
    Lib::LibraryManager* libraryManager = 
      new Lib::LibraryManager(*fSession);
    fSession->addManager(libraryManager);
  }
 
  BatchLab::AnalysisFactory* factory = 
    new BatchLab::AnalysisFactory(*fSession);
  fSession->addManager(factory);

  fAnalysisFactory = factory;
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Main::~Main(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fSession; //It will delete the fAnalysisFactory.

  delete fPrinter;

  Lib::Debug::decrement("BatchLab::Main");

  if(fBalance) {
    Lib::Printer printer;
    Lib::Debug::balance(printer);
  }

  if(fVerboseLevel) {
    Lib::Printer printer;  
    Lib::Out out(printer);
    out << "BatchLab::Main::~Main :"
        << " " << this << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Main::isValid(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return (fAnalysisFactory?true:false);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
//////////////////////////////////////////////////////////////////////////////
AIDA::ITreeFactory* BatchLab::Main::createTreeFactory(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTreeFactory();
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IHistogramFactory* BatchLab::Main::createHistogramFactory(
 AIDA::ITree& aTree
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createHistogramFactory(aTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITupleFactory* BatchLab::Main::createTupleFactory(
 AIDA::ITree& aTree
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTupleFactory(aTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFunctionFactory* BatchLab::Main::createFunctionFactory(
 AIDA::ITree& aTree
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFunctionFactory(aTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterFactory* BatchLab::Main::createPlotterFactory(
 int aArgc
,char** aArgv
,const std::string& aImp
,const std::string& aLib
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createPlotterFactory(aArgc,aArgv,aImp,aLib);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IDataPointSetFactory* BatchLab::Main::createDataPointSetFactory(
 AIDA::ITree& aTree
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createDataPointSetFactory(aTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFitFactory* BatchLab::Main::createFitFactory(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFitFactory();
}
#else
//////////////////////////////////////////////////////////////////////////////
AIDA::ITreeFactory* BatchLab::Main::createTreeFactory(
 const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTreeFactory(aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IHistogramFactory* BatchLab::Main::createHistogramFactory(
 AIDA::ITree& aTree
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createHistogramFactory(aTree,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITupleFactory* BatchLab::Main::createTupleFactory(
 AIDA::ITree& aTree
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTupleFactory(aTree,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFunctionFactory* BatchLab::Main::createFunctionFactory(
 AIDA::ITree& aTree
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFunctionFactory(aTree,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterFactory* BatchLab::Main::createPlotterFactory(
 int aArgc
,char** aArgv
,const std::string& aImp
,const std::string& aLib
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return 
    fAnalysisFactory->createPlotterFactory(aArgc,aArgv,aImp,aLib,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IDataPointSetFactory* BatchLab::Main::createDataPointSetFactory(
 AIDA::ITree& aTree
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createDataPointSetFactory(aTree,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFitFactory* BatchLab::Main::createFitFactory(
 const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFitFactory(aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IGenericFactory* BatchLab::Main::createGenericFactory(
 const std::string& aFactoryType
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createGenericFactory(aFactoryType,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IGenericFactory* BatchLab::Main::createManagedObjectGenericFactory(
 const std::string& aFactoryType
,AIDA::ITree& aTree
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createManagedObjectGenericFactory
           (aFactoryType,aTree,aOptions);
}
#endif
