// this :
#include <BatchLab/Rio/DataReader.h>

// Slash :
#include <Slash/Store/IStorable.h>

// Lib :
#include <Lib/Out.h>
#include <Lib/sout.h>

// Rio :
#include <Rio/Interfaces/IDictionary.h>
#include <Rio/Interfaces/IObject.h>
#include <Rio/Interfaces/IFile.h>
#include <Rio/Tree/Tree.h>

// BatchLab :
#include <BatchLab/Rio/ReadContext.h>

#include <BatchLab/Core/Histogram.h>

#include <BatchLab/Rio/ROOT.h>

#include <BatchLab/Rio/Tuple.h>

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Rio::read_TH1F(
 Rio::ReadContext& aContext
,bool& aDelete
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aDelete = true;
  Rio::TH1F* rioHisto = ::Rio::cast<Rio::TH1F>(aContext.object());
  if(!rioHisto) return 0;

  HCL::Histogram* hclHisto = rioHisto->hclHistogram();
  if(!hclHisto) return 0; //Reading could fail.

  BatchLab::BaseHistogram* baseHistogram = 0;
  const std::string& aSID = aContext.SID();
  int dim = hclHisto->getDimension();
  if(dim!=1) {
    Lib::Out out(aContext.printer());
    out << "BatchLab::Rio::read_TH1F :"
        << " dimension mismatch for " << Lib::sout(aSID)
        << " dimension " << dim << " (1 expected)."
        << Lib::endl;
  } else {
    baseHistogram = new BatchLab::Histogram1D(aContext.printer(),aSID,*hclHisto);
    baseHistogram->setAnnotation(Annotation());
  }
  delete hclHisto;

  return baseHistogram;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Rio::read_TH2F(
 Rio::ReadContext& aContext
,bool& aDelete
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aDelete = true;
  Rio::TH2F* rioHisto = ::Rio::cast<Rio::TH2F>(aContext.object());
  if(!rioHisto) return 0;

  HCL::Histogram* hclHisto = rioHisto->hclHistogram();
  if(!hclHisto) return 0; //Reading could fail.

  BatchLab::BaseHistogram* baseHistogram = 0;
  const std::string& aSID = aContext.SID();
  int dim = hclHisto->getDimension();
  if(dim!=2) {
    Lib::Out out(aContext.printer());
    out << "BatchLab::Rio::read_TH2F :"
        << " dimension mismatch for " << Lib::sout(aSID)
        << " dimension " << dim << " (1 expected)."
        << Lib::endl;
  } else {
    baseHistogram = new BatchLab::Histogram2D(aContext.printer(),aSID,*hclHisto);
    baseHistogram->setAnnotation(Annotation());
  }
  delete hclHisto;

  return baseHistogram;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Rio::read_TH1D(
 Rio::ReadContext& aContext
,bool& aDelete
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aDelete = true;
  Rio::TH1D* rioHisto = ::Rio::cast<Rio::TH1D>(aContext.object());
  if(!rioHisto) return 0;

  HCL::Histogram* hclHisto = rioHisto->hclHistogram();
  if(!hclHisto) return 0; //Reading could fail.

  BatchLab::BaseHistogram* baseHistogram = 0;
  const std::string& aSID = aContext.SID();
  int dim = hclHisto->getDimension();
  if(dim!=1) {
    Lib::Out out(aContext.printer());
    out << "BatchLab::Rio::read_TH1D :"
        << " dimension mismatch for " << Lib::sout(aSID)
        << " dimension " << dim << " (2 expected)." 
        << Lib::endl;
  } else {
    baseHistogram = 
      new BatchLab::Histogram1D(aContext.printer(),aSID,*hclHisto);
    baseHistogram->setAnnotation(Annotation());
  }
  delete hclHisto;

  return baseHistogram;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Rio::read_TH2D(
 Rio::ReadContext& aContext
,bool& aDelete
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aDelete = true;
  Rio::TH2D* rioHisto = ::Rio::cast<Rio::TH2D>(aContext.object());
  if(!rioHisto) return 0;

  HCL::Histogram* hclHisto = rioHisto->hclHistogram();
  if(!hclHisto) return 0; //Reading could fail.

  BatchLab::BaseHistogram* baseHistogram = 0;
  const std::string& aSID = aContext.SID();
  int dim = hclHisto->getDimension();
  if(dim!=2) {
    Lib::Out out(aContext.printer());
    out << "BatchLab::Rio::read_TH2D :"
        << " dimension mismatch for " << Lib::sout(aSID)
        << " dimension " << dim << " (2 expected)."
        << Lib::endl;
  } else {
    baseHistogram = 
      new BatchLab::Histogram2D(aContext.printer(),aSID,*hclHisto);
    baseHistogram->setAnnotation(Annotation());
  }
  delete hclHisto;

  return baseHistogram;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Rio::read_Tree(
 Rio::ReadContext& aContext
,bool& aDelete
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aDelete = true;
  ::Rio::Tree* rioTree = ::Rio::cast< ::Rio::Tree>(aContext.object());
  if(!rioTree) return 0;
  aDelete = false;
  const std::string& aSID = aContext.SID();
  return new BatchLab::Rio::Tuple
           (aContext.printer(),aSID,rioTree,aContext.verboseLevel());
}
