// this :
#include <BatchLab/Rio/ROOT.h>

// Rio :
#include <Rio/Interfaces/IClass.h>
#include <Rio/Interfaces/IBuffer.h>
#include <Rio/Core/List.h>
#include <Rio/Core/Named.h>
#include <Rio/Core/Array_double.h>
#include <Rio/Core/Array_float.h>
#include <Rio/Core/Out.h>
#include <Rio/Graf/AttLine.h>
#include <Rio/Graf/AttFill.h>
#include <Rio/Graf/AttMarker.h>

// HCL :
#include <HCL/Histogram.h>

// BatchLab :
#include <BatchLab/Rio/Axis.h>

// Lib :
#include <Lib/vmanip.h>

#include <stdlib.h> //rand.

//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH::TH(
 const ::Rio::IDictionary& aDictionary
)
:Named(aDictionary,"","")
,fHistogram(0)
,fEntries(0)
,fSw(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH::~TH(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
HCL::Histogram* BatchLab::Rio::TH::hclHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TH::streamTH1(
 ::Rio::IBuffer& aBuffer
,HCL::Histogram& aHisto
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TH::streamTH1 : version %d\n",v);

    // Version 3 streaming (ROOT/v3-00-6).

    if(!Named::stream(aBuffer)) return false;

    HCL::HistogramData& histo = aHisto.getData();

    histo.fTitle = title();

    {::Rio::AttLine dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;}
    {::Rio::AttFill dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;}
    {::Rio::AttMarker dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;}

    int Ncells;
    if(!aBuffer.read(Ncells)) return false;

    //fXAxis
    {Rio::Axis axis(dictionary(),histo.fAxes[0]);
    if(!axis.stream(aBuffer)) return false;}
    histo.fAxes[0].fOffset = 1;

    if(histo.fDimension==2) {
      Rio::Axis yaxis(dictionary(),histo.fAxes[1]);
      if(!yaxis.stream(aBuffer)) return false; //fYAxis
      histo.fAxes[1].fOffset = 
        histo.fAxes[0].fOffset * (histo.fAxes[0].bins()+2);

      HCL::Axis dummy;
      Rio::Axis zaxis(dictionary(),dummy);
      if(!zaxis.stream(aBuffer)) return false; //fZAxis
    } else {
      HCL::Axis dummy;
      Rio::Axis yaxis(dictionary(),dummy);
      if(!yaxis.stream(aBuffer)) return false; //fYAxis
      Rio::Axis zaxis(dictionary(),dummy);
      if(!zaxis.stream(aBuffer)) return false; //fZAxis
    }

    short barOffset;
    if(!aBuffer.read(barOffset)) return false;

    short barWidth;
    if(!aBuffer.read(barWidth)) return false;

    if(!aBuffer.read(fEntries)) return false;

    if(!aBuffer.read(fSw)) return false;
    //printf("debug : BatchLab::Rio::TH::streamTH1 : \"%s\" %g %g\n",
      //histo.fTitle.c_str(),fEntries,fSw);

    double sw2;
    if(!aBuffer.read(sw2)) return false;

    double xSxw;
    if(!aBuffer.read(xSxw)) return false;

    double xSx2w;
    if(!aBuffer.read(xSx2w)) return false;

    double max;
    if(!aBuffer.read(max)) return false;

    double min;
    if(!aBuffer.read(min)) return false;

    double NormFactor;
    if(!aBuffer.read(NormFactor)) return false;

    {::Rio::Array_double dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;} //fContour TArrayD

    ::Rio::Array_double sumw2(dictionary()); //fSumw2 TArrayD
    if(!sumw2.stream(aBuffer)) return false;

    //TString
    std::string Option;
    if(!aBuffer.read(Option)) return false;

    {::Rio::List dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;} //Functions

    if(v>=4) {
      int BufferSize;
      if(!aBuffer.read(BufferSize)) return false;
      
      //Double_t[fBufferSize]
      {char isArray;
      if(!aBuffer.read(isArray)) return false;
      if(isArray!=0) {
        if(BufferSize) {
          double* Buffer = new double[BufferSize];
          if(!aBuffer.readFastArray(Buffer,BufferSize)) return false;
          delete [] Buffer;
        }
      }}
    }

    // Add two for outflows.
    if(histo.fDimension==1) {
      histo.fBinNumber = histo.fAxes[0].fNumberOfBins + 2;
    } else if(histo.fDimension==2) {
      int xbinn = histo.fAxes[0].fNumberOfBins + 2;
      int ybinn = histo.fAxes[1].fNumberOfBins + 2;
      histo.fBinNumber = xbinn * ybinn;
    }

    int binn = histo.fBinNumber;
    histo.fBinSw2.resize(binn,0);
    if(binn==sumw2.size()) {
      for(int index=0;index<binn;index++){
        histo.fBinSw2[index] = sumw2.array()[index];
      }
    }

    if(!aBuffer.checkByteCount(s,c,"TH1")) return false;

  } else {

    if(!aBuffer.writeVersion(3)) return false;

    if(!Named::stream(aBuffer)) return false;

    {::Rio::AttLine dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;}
    {::Rio::AttFill dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;}
    {::Rio::AttMarker dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;}

    if(!aBuffer.write(aHisto.getBins())) return false;

    HCL::HistogramData& histo = aHisto.getData();

    //fXAxis
    {Rio::Axis axis(dictionary(),histo.fAxes[0]);
    axis.setName("xaxis");
    if(!axis.stream(aBuffer)) return false;}

    if(histo.fDimension==2) {
      Rio::Axis yaxis(dictionary(),histo.fAxes[1]);
      yaxis.setName("yaxis");
      if(!yaxis.stream(aBuffer)) return false;

      HCL::Axis dummy;
      dummy.configure(1,0,1);
      Rio::Axis zaxis(dictionary(),dummy);
      zaxis.setName("zaxis");
      if(!zaxis.stream(aBuffer)) return false;
    } else {
      HCL::Axis dummy;
      dummy.configure(1,0,1);

      Rio::Axis yaxis(dictionary(),dummy);
      yaxis.setName("yaxis");
      if(!yaxis.stream(aBuffer)) return false;

      Rio::Axis zaxis(dictionary(),dummy);
      zaxis.setName("zaxis");
      if(!zaxis.stream(aBuffer)) return false;
    }

    if(!aBuffer.write((short)(1000 * 0.25))) return false; //fBarOffset
    if(!aBuffer.write((short)(1000 * 0.5))) return false; //fBarWidth
    if(!aBuffer.write((double)aHisto.getEntries())) return false;
    if(!aBuffer.write(aHisto.getSw())) return false;
    if(!aBuffer.write(aHisto.getSw2())) return false;

    if(!aBuffer.write((double)aHisto.getIthAxisSxw(0))) return false;
    if(!aBuffer.write((double)aHisto.getIthAxisSx2w(0))) return false;

    if(!aBuffer.write((double)-1111)) return false; //fMaximum
    if(!aBuffer.write((double)-1111)) return false; //fMinimum
    if(!aBuffer.write((double)0)) return false; //NormFactor

    {::Rio::Array_double dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;} //fContour TArrayD

    {::Rio::Array_double sumw2(dictionary(),histo.fBinSw2);
    if(!sumw2.stream(aBuffer)) return false;} //fSumw2 TArrayD

    //TString
    if(!aBuffer.write(std::string())) return false; //Option

    {::Rio::List dummy(dictionary());
    if(!dummy.stream(aBuffer)) return false;} //Functions

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TH::streamTH2(
 ::Rio::IBuffer& aBuffer
,HCL::Histogram& aHisto
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TH::streamTH2 : version %d\n",v);

    // Version 3 streaming (ROOT/v3-00-6).

    if(!streamTH1(aBuffer,aHisto)) return false;

    double ScaleFactor;
    if(!aBuffer.read(ScaleFactor)) return false;
    double Tsumwy;
    if(!aBuffer.read(Tsumwy)) return false;
    double Tsumwy2;
    if(!aBuffer.read(Tsumwy2)) return false;
    double Tsumwxy;
    if(!aBuffer.read(Tsumwxy)) return false;

    if(!aBuffer.checkByteCount(s,c,"TH2")) return false;

  } else {

    if(!aBuffer.writeVersion(3)) return false;

    if(!streamTH1(aBuffer,aHisto)) return false;

    if(!aBuffer.write((double)1)) return false; //ScaleFactor

    if(!aBuffer.write((double)aHisto.getIthAxisSxw(1))) return false;
    if(!aBuffer.write((double)aHisto.getIthAxisSx2w(1))) return false;

    if(!aBuffer.write((double)0)) return false; //Tsumwxy

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RIO_META_SOURCE(BatchLab::Rio::TH1F)
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH1F::TH1F(
 const ::Rio::IDictionary& aDictionary
)
:BatchLab::Rio::TH(aDictionary)
,fClass(BatchLab::Rio::TH1F::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH1F::TH1F(
 const ::Rio::IDictionary& aDictionary
,const HCL::Histogram* aHisto
,const std::string& aName
)
:BatchLab::Rio::TH(aDictionary)
,fClass(BatchLab::Rio::TH1F::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHistogram = const_cast<HCL::Histogram*>(aHisto);
  setName(aName);
  setTitle(aHisto->getTitle());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH1F::~TH1F(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Rio::TH1F::cast(
 const ::Rio::IClass& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(&aClass==&fClass) return Rio_SCast(BatchLab::Rio::TH1F);
  else return Named::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TH1F::stream(
 ::Rio::IBuffer& aBuffer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    fHistogram = 0;

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TH1F::stream : version %d\n",v);

    // Version 1 streaming (ROOT/v3-00-6).

    // Now we have to reconstruct a valid Histogram from aBuffer :
    HCL::Histogram* hclHistogram = new HCL::Histogram;
    HCL::HistogramData& histo = hclHistogram->getData();

    histo.fDimension = 1;
    histo.fDoubles.resize(histo.fDimension,0);
    histo.fInts.resize(histo.fDimension,0);
    histo.fAxes.resize(1);

    if(!streamTH1(aBuffer,*hclHistogram)) {
      delete hclHistogram;
      return false;
    }

    ::Rio::Array_float bins(dictionary()); //fBins TArrayF
    if(!bins.stream(aBuffer)) {
      delete hclHistogram;
      return false; 
    }

    int binn = histo.fBinNumber;
    //printf("debug : BatchLab::Rio::TH1F::stream : histo bins %d\n",binn);
    histo.fBinSw.resize(binn,0);
    double asw = 0;
   {for(int index=0;index<binn;index++){
      double h = double(bins.array()[index]);
      histo.fBinSw[index] = h;
      asw += h;
     }}
   {double sw = hclHistogram->getSw();
    if(sw!=fSw) {
      ::Rio::Out out(aBuffer.printer());
      out << "BatchLab::Rio::TH1F::stream : "
          << " WARNING : inconsistent total weight" 
          << " for histo with title \"" << histo.fTitle << "\" :"
          << ::Rio::endl;
      out << "BatchLab::Rio::TH1F::stream : "
          << " read fSw is " << fSw  
          << " whilst sum of in-range bins weight is " << sw
          << ::Rio::endl;
    }}

    // Fill Sxw and Sx2w by using in range bins :
    std::vector<double> empty;
    empty.resize(1,0);
    histo.fBinSxw.resize(binn,empty);
    histo.fBinSx2w.resize(binn,empty);    
    std::vector<int> is(histo.fDimension);
   {for(int index=0;index<binn;index++){
      if(!hclHistogram->isOut(index)) {
        float height = bins.array()[index];
        hclHistogram->getIndices(index,is);
        for(int iaxis=0;iaxis<histo.fDimension;iaxis++) {
          double x = histo.fAxes[iaxis].binCenter(is[iaxis]); 
          histo.fBinSxw[index][iaxis] = x * height;
          histo.fBinSx2w[index][iaxis] = x * x * height;
        }
      }
    }}

    //ROOT does not store the number of entries per bin.
    // We have the global number of entries and weight ; with that
    // we do our best....
    histo.fBinEntries.resize(binn,0);
    if(asw) {
      for(int index=0;index<binn;index++){
        float height = bins.array()[index];
        int number = (int)((fEntries * height) / asw);
        if(number<0) {
        } else {
          histo.fBinEntries[index] = number;
        }
      }
      int allEntries = hclHistogram->getAllEntries();
      if(allEntries<int(fEntries)) { //FIXME
        //Correct some bins randomly (beurk, do you have a better idea ?) :
        int diff = int(fEntries)-allEntries;
        int nx = histo.fAxes[0].fNumberOfBins;
        for(int i=0;i<diff;i++) {
          int ri = ::rand();
          // ibin in [1,nx-1]
          int ibin =  int((nx-1) * ((double)ri/(double)RAND_MAX)) + 1;
          int offset = ibin;
          histo.fBinEntries[offset]++;
        }
      } else if(allEntries>int(fEntries)) { //FIXME
        ::Rio::Out out(aBuffer.printer());
        out << "BatchLab::Rio::TH1F::stream : "
            << " WARNING : can't reemulate number of entries per bin " 
            << " for histo with title \"" << histo.fTitle << "\" :"
            << ::Rio::endl;
      } else {
      }
    }

    if(!aBuffer.checkByteCount(s,c,"TH1F")) {
      delete hclHistogram;
      return false;
    }

    hclHistogram->updateFastGetters();

    // We have now a valid HCL histogram.
    fHistogram = hclHistogram;

  } else {
    if(!aBuffer.writeVersion(1)) return false;

    if(!streamTH1(aBuffer,*fHistogram)) return false;

    HCL::HistogramData& histo = fHistogram->getData();

    {::Rio::Array_float sumw(dictionary(),Lib::vmanip::convert(histo.fBinSw));
    if(!sumw.stream(aBuffer)) return false;} //fSumw TArrayD

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RIO_META_SOURCE(BatchLab::Rio::TH1D)
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH1D::TH1D(
 const ::Rio::IDictionary& aDictionary
)
:BatchLab::Rio::TH(aDictionary)
,fClass(BatchLab::Rio::TH1D::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH1D::TH1D(
 const ::Rio::IDictionary& aDictionary
,const HCL::Histogram* aHisto
,const std::string& aName
)
:BatchLab::Rio::TH(aDictionary)
,fClass(BatchLab::Rio::TH1D::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHistogram = const_cast<HCL::Histogram*>(aHisto);
  setName(aName);
  setTitle(aHisto->getTitle());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH1D::~TH1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Rio::TH1D::cast(
 const ::Rio::IClass& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(&aClass==&fClass) return Rio_SCast(BatchLab::Rio::TH1D);
  else return Named::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TH1D::stream(
 ::Rio::IBuffer& aBuffer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    fHistogram = 0;

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TH1D::stream : version %d\n",v);

    // Version 1 streaming (ROOT/v3-00-6).

    // Now we have to reconstruct a valid Histogram from aBuffer :
    HCL::Histogram* hclHistogram = new HCL::Histogram;
    HCL::HistogramData& histo = hclHistogram->getData();

    histo.fDimension = 1;
    histo.fDoubles.resize(histo.fDimension,0);
    histo.fInts.resize(histo.fDimension,0);
    histo.fAxes.resize(1);

    if(!streamTH1(aBuffer,*hclHistogram)) {
      delete hclHistogram;
      return false;
    }

    ::Rio::Array_double bins(dictionary()); //fBins TArrayD
    if(!bins.stream(aBuffer)) {
      delete hclHistogram;
      return false; 
    }

    int binn = histo.fBinNumber;
    //printf("debug : BatchLab::Rio::TH1D::stream : histo bins %d\n",binn);
    histo.fBinSw.resize(binn,0);
    double asw = 0;
   {for(int index=0;index<binn;index++){
      double h = bins.array()[index];
      histo.fBinSw[index] = h;
      asw += h;
    }}
   {double sw = hclHistogram->getSw();
    if(sw!=fSw) {
      ::Rio::Out out(aBuffer.printer());
      out << "BatchLab::Rio::TH1D::stream : "
          << " WARNING : inconsistent total weight" 
          << " for histo with title \"" << histo.fTitle << "\" :"
          << ::Rio::endl;
      out << "BatchLab::Rio::TH1D::stream : "
          << " read fSw is " << fSw  
          << " whilst sum of in-range bins weight is " << sw
          << ::Rio::endl;
    }}

    // Fill Sxw and Sx2w by using in range bins :
    std::vector<double> empty;
    empty.resize(1,0);
    histo.fBinSxw.resize(binn,empty);
    histo.fBinSx2w.resize(binn,empty);
    std::vector<int> is(histo.fDimension);
    for(int index=0;index<binn;index++){
      if(!hclHistogram->isOut(index)) {
        double height = bins.array()[index];
        hclHistogram->getIndices(index,is);
        for(int iaxis=0;iaxis<histo.fDimension;iaxis++) {
          double x = histo.fAxes[iaxis].binCenter(is[iaxis]); 
          histo.fBinSxw[index][iaxis] = x * height;
          histo.fBinSx2w[index][iaxis] = x * x * height;
        }
      }
    }

    //ROOT does not store the number of entries per bin.
    // We have the global number of entries and weight ; with that
    // we do our best....
    histo.fBinEntries.resize(binn,0);
    if(asw) {
      for(int index=0;index<binn;index++){
        double height = bins.array()[index];
        int number = (int)((fEntries * height) / asw);        
        if(number<0) {
        } else {
          histo.fBinEntries[index] = number;
        }
      }
      int allEntries = hclHistogram->getAllEntries();
      if(allEntries<int(fEntries)) { //FIXME
        //Correct some bins randomly (beurk, do you have a better idea ?) :
        int diff = int(fEntries)-allEntries;
        int nx = histo.fAxes[0].fNumberOfBins;
        for(int i=0;i<diff;i++) {
          int ri = ::rand();
          int ibin =  int((nx-1) * ((double)ri/(double)RAND_MAX)) + 1;
          int offset = ibin;
          histo.fBinEntries[offset]++;
        }
      } else if(allEntries>int(fEntries)) { //FIXME
        ::Rio::Out out(aBuffer.printer());
        out << "BatchLab::Rio::TH1D::stream : "
            << " WARNING : can't reemulate number of entries per bin " 
            << " for histo with title \"" << histo.fTitle << "\" :"
            << ::Rio::endl;
      } else {
      }
    }

    if(!aBuffer.checkByteCount(s,c,"TH1D")) {
      delete hclHistogram;
      return false;
    }

    hclHistogram->updateFastGetters();

    // We have now a valid HCL histogram.
    fHistogram = hclHistogram;

  } else {

    if(!aBuffer.writeVersion(1)) return false;

    if(!streamTH1(aBuffer,*fHistogram)) return false;

    HCL::HistogramData& histo = fHistogram->getData();

    {::Rio::Array_double sumw(dictionary(),histo.fBinSw);
    if(!sumw.stream(aBuffer)) return false;} //fSumw TArrayD

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RIO_META_SOURCE(BatchLab::Rio::TH2F)
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH2F::TH2F(
 const ::Rio::IDictionary& aDictionary
)
:BatchLab::Rio::TH(aDictionary)
,fClass(BatchLab::Rio::TH2F::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH2F::TH2F(
 const ::Rio::IDictionary& aDictionary
,const HCL::Histogram* aHisto
,const std::string& aName
)
:BatchLab::Rio::TH(aDictionary)
,fClass(BatchLab::Rio::TH2F::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHistogram = const_cast<HCL::Histogram*>(aHisto);
  setName(aName);
  setTitle(aHisto->getTitle());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH2F::~TH2F(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Rio::TH2F::cast(
 const ::Rio::IClass& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(&aClass==&fClass) return Rio_SCast(BatchLab::Rio::TH2F);
  else return Named::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TH2F::stream(
 ::Rio::IBuffer& aBuffer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    fHistogram = 0;

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TH2F::stream : version %d\n",v);

    // Version 3 streaming (ROOT/v3-00-6).

    // Now we have to reconstruct a valid Histogram from aBuffer :
    HCL::Histogram* hclHistogram = new HCL::Histogram;
    HCL::HistogramData& histo = hclHistogram->getData();

    histo.fDimension = 2;
    histo.fDoubles.resize(histo.fDimension,0);
    histo.fInts.resize(histo.fDimension,0);
    histo.fAxes.resize(2);

    if(!streamTH2(aBuffer,*hclHistogram)) {
      delete hclHistogram;
      return false;
    }

    ::Rio::Array_float bins(dictionary()); //fBins TArrayF
    if(!bins.stream(aBuffer)) {
      delete hclHistogram;
      return false; 
    }

    //printf("debug : BatchLab::Rio::TH2F::stream : histo bins %d\n",n);
    int binn = histo.fBinNumber;
    histo.fBinSw.resize(binn,0);
    double asw = 0;
   {for(int index=0;index<binn;index++){
      double h = double(bins.array()[index]);
      histo.fBinSw[index] = h;
      asw += h;
    }}
   {double sw = hclHistogram->getSw();
    if(sw!=fSw) {
      ::Rio::Out out(aBuffer.printer());
      out << "BatchLab::Rio::TH2F::stream : "
          << " WARNING : inconsistent total weight" 
          << " for histo with title \"" << histo.fTitle << "\" :"
          << ::Rio::endl;
      out << "BatchLab::Rio::TH2F::stream : "
          << " read fSw is " << fSw  
          << " whilst sum of in-range bins weight is " << sw
          << ::Rio::endl;
    }}

    // Fill Sxw and Sx2w by using in range bins :
    std::vector<double> empty;
    empty.resize(2,0);
    histo.fBinSxw.resize(binn,empty);
    histo.fBinSx2w.resize(binn,empty);
    std::vector<int> is(histo.fDimension);
    for(int index=0;index<binn;index++){
      if(!hclHistogram->isOut(index)) {
        float height = bins.array()[index];
        hclHistogram->getIndices(index,is);
        for(int iaxis=0;iaxis<histo.fDimension;iaxis++) {
          double x = histo.fAxes[iaxis].binCenter(is[iaxis]); 
          histo.fBinSxw[index][iaxis] = x * height;
          histo.fBinSx2w[index][iaxis] = x * x * height;
        }
      }
    }

    //ROOT does not store the number of entries per bin.
    // We have the global number of entries and weight ; with that
    // we do our best....
    histo.fBinEntries.resize(binn,0);
    if(asw) {
      for(int index=0;index<binn;index++){
        float height = bins.array()[index];
        int number = (int)((fEntries * height) / asw);
        if(number<0) {
        } else {
          histo.fBinEntries[index] = number;
        } 
      }
      int allEntries = hclHistogram->getAllEntries();
      if(allEntries<(int)fEntries) { //FIXME
        //Correct some bins randomly (beurk, do you have a better idea ?) :
        int diff = int(fEntries-allEntries);
        int nx = histo.fAxes[0].fNumberOfBins;
        int ny = histo.fAxes[1].fNumberOfBins;
        for(int i=0;i<diff;i++) {
          int ri = ::rand();
          int ibin =  int((nx-1) * ((double)ri/(double)RAND_MAX)) + 1;
          int rj = ::rand();
          int jbin =  int((ny-1) * ((double)rj/(double)RAND_MAX)) + 1;
          int offset = ibin + jbin * histo.fAxes[1].fOffset;
          histo.fBinEntries[offset]++;
        }
      } else if(allEntries>int(fEntries)) { //FIXME
        ::Rio::Out out(aBuffer.printer());
        out << "BatchLab::Rio::TH2F::stream : "
            << " WARNING : can't reemulate number of entries per bin " 
            << " for histo with title \"" << histo.fTitle << "\" :"
            << ::Rio::endl;
      } else {
      }
    }
    
    if(!aBuffer.checkByteCount(s,c,"TH2F")) {
      delete hclHistogram;
      return false;
    }

    hclHistogram->updateFastGetters();

    // We have now a valid HCL histogram.
    fHistogram = hclHistogram;

  } else {

    if(!aBuffer.writeVersion(3)) return false;

    if(!streamTH2(aBuffer,*fHistogram)) return false;

    HCL::HistogramData& histo = fHistogram->getData();

    {::Rio::Array_float sumw(dictionary(),Lib::vmanip::convert(histo.fBinSw));
    if(!sumw.stream(aBuffer)) return false;} //fSumw TArrayD

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RIO_META_SOURCE(BatchLab::Rio::TH2D)
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH2D::TH2D(
 const ::Rio::IDictionary& aDictionary
)
:BatchLab::Rio::TH(aDictionary)
,fClass(BatchLab::Rio::TH2D::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH2D::TH2D(
 const ::Rio::IDictionary& aDictionary
,const HCL::Histogram* aHisto
,const std::string& aName
)
:BatchLab::Rio::TH(aDictionary)
,fClass(BatchLab::Rio::TH2D::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHistogram = const_cast<HCL::Histogram*>(aHisto);
  setName(aName);
  setTitle(aHisto->getTitle());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TH2D::~TH2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Rio::TH2D::cast(
 const ::Rio::IClass& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(&aClass==&fClass) return Rio_SCast(BatchLab::Rio::TH2D);
  else return Named::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TH2D::stream(
 ::Rio::IBuffer& aBuffer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    fHistogram = 0;

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TH2D::stream : version %d\n",v);

    // Version 3 streaming (ROOT/v3-00-6).

    // Now we have to reconstruct a valid Histogram from aBuffer :
    HCL::Histogram* hclHistogram = new HCL::Histogram;
    HCL::HistogramData& histo = hclHistogram->getData();

    histo.fDimension = 2;
    histo.fDoubles.resize(histo.fDimension,0);
    histo.fInts.resize(histo.fDimension,0);
    histo.fAxes.resize(2);

    if(!streamTH2(aBuffer,*hclHistogram)) {
      delete hclHistogram;
      return false;
    }

    ::Rio::Array_double bins(dictionary()); //fBins TArrayD
    if(!bins.stream(aBuffer)) {
      delete hclHistogram;
      return false; 
    }

    //printf("debug : BatchLab::Rio::TH2D::stream : histo bins %d\n",n);
    int binn = histo.fBinNumber;
    histo.fBinSw.resize(binn,0);
    double asw = 0;
   {for(int index=0;index<binn;index++){
      double h = bins.array()[index];
      histo.fBinSw[index] = h;
      asw += h;
     }}
   {double sw = hclHistogram->getSw();
    if(sw!=fSw) {
      ::Rio::Out out(aBuffer.printer());
      out << "BatchLab::Rio::TH2D::stream : "
          << " WARNING : inconsistent total weight" 
          << " for histo with title \"" << histo.fTitle << "\" :"
          << ::Rio::endl;
      out << "BatchLab::Rio::TH2D::stream : "
          << " read fSw is " << fSw  
          << " whilst sum of in-range bins weight is " << sw
          << ::Rio::endl;
    }}

    // Fill Sxw and Sx2w by using in range bins :
    std::vector<double> empty;
    empty.resize(2,0);
    histo.fBinSxw.resize(binn,empty);
    histo.fBinSx2w.resize(binn,empty);
    std::vector<int> is(histo.fDimension);
    for(int index=0;index<binn;index++){
      if(!hclHistogram->isOut(index)) {
        double height = bins.array()[index];
        hclHistogram->getIndices(index,is);
        for(int iaxis=0;iaxis<histo.fDimension;iaxis++) {
          double x = histo.fAxes[iaxis].binCenter(is[iaxis]); 
          histo.fBinSxw[index][iaxis] = x * height;
          histo.fBinSx2w[index][iaxis] = x * x * height;
        }
      }
    }

    //ROOT does not store the number of entries per bin.
    // We have the global number of entries and weight ; with that
    // we do our best....
    histo.fBinEntries.resize(binn,0);
    if(asw) {
      for(int index=0;index<binn;index++){
        double height = bins.array()[index];
        int number = (int)((fEntries * height) / asw);
        if(number<0) {
        } else {
          histo.fBinEntries[index] = number;
        } 
      }
      int allEntries = hclHistogram->getAllEntries();
      if(allEntries<(int)fEntries) { //FIXME
        //Correct some bins randomly (beurk, do you have a better idea ?) :
        int diff = int(fEntries-allEntries);
        int nx = histo.fAxes[0].fNumberOfBins;
        int ny = histo.fAxes[1].fNumberOfBins;
        for(int i=0;i<diff;i++) {
          int ri = ::rand();
          int ibin =  int((nx-1) * ((double)ri/(double)RAND_MAX)) + 1;
          int rj = ::rand();
          int jbin =  int((ny-1) * ((double)rj/(double)RAND_MAX)) + 1;
          int offset = ibin + jbin * histo.fAxes[1].fOffset;
          histo.fBinEntries[offset]++;
        }
      } else if(allEntries>int(fEntries)) { //FIXME
        ::Rio::Out out(aBuffer.printer());
        out << "BatchLab::Rio::TH2D::stream : "
            << " WARNING : can't reemulate number of entries per bin " 
            << " for histo with title \"" << histo.fTitle << "\" :"
            << ::Rio::endl;
      } else {
      }
    }
    
    if(!aBuffer.checkByteCount(s,c,"TH2D")) {
      delete hclHistogram;
      return false;
    }

    hclHistogram->updateFastGetters();

    // We have now a valid HCL histogram.
    fHistogram = hclHistogram;

  } else {

    if(!aBuffer.writeVersion(3)) return false;

    if(!streamTH2(aBuffer,*fHistogram)) return false;

    HCL::HistogramData& histo = fHistogram->getData();

    {::Rio::Array_double sumw(dictionary(),histo.fBinSw);
    if(!sumw.stream(aBuffer)) return false;} //fSumw TArrayD

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RIO_META_SOURCE(BatchLab::Rio::TProfile)
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TProfile::TProfile(
 const ::Rio::IDictionary& aDictionary
)
:BatchLab::Rio::TH1D(aDictionary)
,fClass(BatchLab::Rio::TProfile::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TProfile::~TProfile(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Rio::TProfile::cast(
 const ::Rio::IClass& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(&aClass==&fClass) return Rio_SCast(BatchLab::Rio::TProfile);
  else return BatchLab::Rio::TH1D::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TProfile::stream(
 ::Rio::IBuffer& aBuffer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TVectorD::stream : version %d\n",v);

    // Version 3 streaming (ROOT/v3-00-6).

    if(!BatchLab::Rio::TH1D::stream(aBuffer)) return false;

    ::Rio::Array_double bins(dictionary()); //fBinEntries TArrayD
    if(!bins.stream(aBuffer)) {
      delete fHistogram;
      fHistogram = 0;
      return false; 
    }

    int errorMode;
    if(!aBuffer.read(errorMode)) {
      delete fHistogram;
      fHistogram = 0;
      return false;
    }

    double ymin;
    if(!aBuffer.read(ymin)) {
      delete fHistogram;
      fHistogram = 0;
      return false;
    }

    double ymax;
    if(!aBuffer.read(ymax)) {
      delete fHistogram;
      fHistogram = 0;
      return false;
    }

    HCL::HistogramData& histo = fHistogram->getData();
    histo.fProfile = true;
    histo.fCutV = true;
    histo.fMinV = ymin;
    histo.fMaxV = ymax;

    int binn = histo.fBinNumber;
    histo.fBinSvw.resize(binn,0);
    histo.fBinSv2w.resize(binn,0);

    for(int index=0;index<binn;index++){
      double svw = histo.fBinSw[index];
      double sw = bins.array()[index];
      histo.fBinEntries[index] = (int)sw; //FIXME : w = 1 only !
      histo.fBinSw[index] = (double)sw;
      //FIXME : histo.fBinSxw
      //FIXME : histo.fBinSx2w
      histo.fBinSvw[index] = svw;
      histo.fBinSv2w[index] = 0; //FIXME
    }

    if(!aBuffer.checkByteCount(s,c,"TProfile")) return false;

    fHistogram->updateFastGetters();

  } else {

    //FIXME
    return false;

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RIO_META_SOURCE(BatchLab::Rio::TVectorD)
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TVectorD::TVectorD(
 const ::Rio::IDictionary& aDictionary
)
: ::Rio::Object(aDictionary)
,fClass(BatchLab::Rio::TVectorD::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TVectorD::~TVectorD(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Rio::TVectorD::cast(
 const ::Rio::IClass& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(&aClass==&fClass) return Rio_SCast(BatchLab::Rio::TVectorD);
  else return Object::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TVectorD::stream(
 ::Rio::IBuffer& aBuffer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TVectorD::stream : version %d\n",v);

    // Version 2 streaming (ROOT/v3-00-6).

    if(!Object::stream(aBuffer)) return false;

    int Rows;
    if(!aBuffer.read(Rows)) return false;
    int RowLwb;
    if(!aBuffer.read(RowLwb)) return false;

    //Double_t[fNrows]
    {char isArray;
    if(!aBuffer.read(isArray)) return false;
    if(isArray!=0) {
      if(Rows) {
        double* Elements = new double[Rows];
        if(!aBuffer.readFastArray(Elements,Rows)) return false;
        delete [] Elements;
      }
    }}

    if(!aBuffer.checkByteCount(s,c,"TVectorD")) return false;

  } else {

    //FIXME
    return false;

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
RIO_META_SOURCE(BatchLab::Rio::TMatrix)
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TMatrix::TMatrix(
 const ::Rio::IDictionary& aDictionary
)
: ::Rio::Object(aDictionary)
,fClass(BatchLab::Rio::TMatrix::is(aDictionary))
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::TMatrix::~TMatrix(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Rio::TMatrix::cast(
 const ::Rio::IClass& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(&aClass==&fClass) return Rio_SCast(BatchLab::Rio::TMatrix);
  else return Object::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::TMatrix::stream(
 ::Rio::IBuffer& aBuffer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    unsigned int s, c;
    short v;
    if(!aBuffer.readVersion(v,s,c)) return false;

    //printf("debug : BatchLab::Rio::TMatrix::stream : version %d\n",v);

    // Version 2 streaming (ROOT/v3-00-6).

    if(!Object::stream(aBuffer)) return false;

    int Nrows;
    if(!aBuffer.read(Nrows)) return false;
    int Ncols;
    if(!aBuffer.read(Ncols)) return false;
    int Nelems;
    if(!aBuffer.read(Nelems)) return false;
    int RowLwb;
    if(!aBuffer.read(RowLwb)) return false;
    int ColLwb;
    if(!aBuffer.read(ColLwb)) return false;

    //Real_t[fNelems]
    {char isArray;
    if(!aBuffer.read(isArray)) return false;
    if(isArray!=0) {
      if(Nelems) {
        float* Elements = new float[Nelems];
        if(!aBuffer.readFastArray(Elements,Nelems)) return false;
        delete [] Elements;
      }
    }}

    if(!aBuffer.checkByteCount(s,c,"TMatrix")) return false;

  } else {

    //FIXME
    return false;

  }
  return true;
}
