// this :
#include <BatchLab/XML/Tree.h>

#include <AIDA/IConstants.h>

#include <Slash/Core/ISession.h>
#include <Slash/Store/IObject.h> //For delete object.
#include <Slash/Store/IFolderVisitor.h>

#include <Lib/Out.h>
#include <Lib/fmanip.h>
#include <Lib/smanip.h>
#include <Lib/FileWriter.h>
#include <Lib/GZipWriter.h>
#include <Lib/ItemML.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>

#include <BatchLab/Version.h>
#include <BatchLab/Core/amanip.h> //FIXME

#include <BatchLab/XML/Manager.h>
#include <BatchLab/XML/ReadContext.h>
#include <BatchLab/XML/WriteContext.h>

#include <BatchLab/XML/AIDA_ML.h>

//////////////////////////////////////////////////////////////////////////////
BatchLab::XML::Tree::Tree(
 Slash::Core::ISession& aSession
,const std::string& aStoreName
,bool aReadOnly
,bool aCreateNew
,const std::string& aOptions
,XML::Manager& aManager
,int aVerboseLevel
)
:MemoryTree(aSession,aStoreName) //FIXME : have a name # storeName
,fManager(aManager)
,fStoreName(aStoreName)
,fNameCounter(0)
,fReadOnly(aReadOnly)
,fCreateNew(aCreateNew)
,fCompress(false)
,fIsValid(true)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setVerboseLevel(aVerboseLevel);

  std::string value;
  if(Lib::smanip::option(aOptions,"compress",value)) {
    bool bval;
    if(Lib::smanip::tobool(value,bval)) {
      fCompress = bval;
    }
  }

  //printf("debug : XML::Tree :<-- \"%s\" \"%s\" %d -->\n",
  // aStoreName.c_str(),aOptions.c_str(),fCompress);

  if(fCompress && !Lib::GZipWriter::available()) {
    Lib::Out out(printer());
    out << "BatchLab::XML::Tree::XML::Tree :"
        << " compression not available. " 
        << " check installation of Lib package "
        << " and Lib::GZipWriter in particular. "
        << Lib::endl;
    fCompress = false;
  }
  
  if(fCreateNew) {
    // Handled in the commit.
  } else {
    // Read the existing file :
    if(verboseLevel()) {
      Lib::Out out(printer());
      out << "BatchLab::XML::Tree::XML::Tree :"
          << " open " << Lib::sout(aStoreName) 
          << " with compression=" 
          << std::string(fCompress?"yes":"no") << "..." << Lib::endl;
    }
    Lib::ItemMLFactory factory; //FIXME
    AIDA_ML aidaML(factory,*this);
    if(!aidaML.loadFile(aStoreName,fCompress,printer()) || aidaML.errors()) {
      Lib::Out out(printer());
      out << "BatchLab::XML::Tree::XML::Tree :"
          << " problem loading file " << Lib::sout(aStoreName)
          << Lib::endl;
      fIsValid = false;
      return;
    }
    if(verboseLevel()) {
      Lib::Out out(printer());
      out << "BatchLab::XML::Tree::XML::Tree :"
          << " load of " << Lib::sout(aStoreName) << " done."
          << Lib::endl;
    }

  }
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::XML::Tree::~Tree(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::XML::Tree::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::XML::Tree)
  else return MemoryTree::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::XML::Tree::storeName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fStoreName;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::XML::Tree::storeType(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::XML::Tree";
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::XML::Tree::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fIsValid;
}

//////////////////////////////////////////////////////////////////////////////
/// Data specific area ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

namespace BatchLab {
  class XML_StoreVisitor : public virtual Slash::Store::IFolderVisitor {
  public:
    XML_StoreVisitor(XML::Tree& aTree,
                     Lib::ItemMLFactory& aFactory,
                     Slash::Core::IWriter& aWriter,
                     Slash::Core::IWriter& aPrinter)
    :fTree(aTree)
    ,fAIDA_ML(aFactory,aTree)
    ,fWriter(aWriter),fPrinter(aPrinter) {
    }
  public:
    virtual bool visit(const Slash::Store::IFolder& aFolder,
                       const Slash::Store::IObject& aObject){
      if(fTree.verboseLevel()) {
        Lib::Out out(fPrinter);
        out << "BatchLab::XML_StoreVisitor::visit :" 
            << " path : " << Lib::sout(aFolder.path())
            << " name : " << Lib::sout(aObject.name())
            << Lib::endl;
      }
      if(!fAIDA_ML.write(fWriter,aFolder.path(),aObject)) {
        //continue anyway.
      }
      return true;
    }
    virtual bool beginVisit(const Slash::Store::IFolder&,
                            const Slash::Store::IFolder&){ return true;}
    virtual bool endVisit(const Slash::Store::IFolder&,
                          const Slash::Store::IFolder&){ return true;}
    virtual bool isRecursive() const { return true;}
  private:
    XML::Tree& fTree;  
    AIDA_ML fAIDA_ML;
    Slash::Core::IWriter& fWriter;
    Slash::Core::IWriter& fPrinter;
};
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::XML::Tree::commit(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fReadOnly) return false;
  std::string fileName = name(); //!!!
  if(fCreateNew) {
    Lib::fmanip::remove(fileName);
  } else {
    //Can't update a file !
    return false;
  }

  // The name may contain env variables.
  std::string fullName = Lib::fmanip::name(fileName);
  Slash::Core::IWriter* writer = 0;
  if(fCompress) {
    writer = new Lib::GZipWriter(fullName,printer());
    if(!writer->enabled()) {
      delete writer;
      return false;
    }
  } else {
    writer = new Lib::FileWriter(fullName,printer());
    if(!writer->enabled()) {
      delete writer;
      return false;
    }
  }
  Lib::Out out(*writer);
  
  // Header :
  out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << Lib::endl;
  out << "<!DOCTYPE aida SYSTEM \"http://aida.freehep.org/schemas/3.0/aida.dtd\">" << Lib::endl;
  out << "<aida version=" << Lib::sout(AIDA_VERSION) << ">" << Lib::endl;
  out << "  <implementation package=" << Lib::sout("BatchLab") 
      << " version=" << Lib::sout(BATCHLAB_VERSION) << "/>" << Lib::endl;

  Lib::ItemMLFactory factory; //FIXME
  XML_StoreVisitor visitor(*this,factory,*writer,printer());
  rootFolder().visit(visitor);

  // Footer :
  out << "</aida>" << Lib::endl;

  delete writer;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::XML::Tree::close(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return true;
}

