// this :
#include <BatchLab/Zebra/Histogram.h>

#include <Lib/Out.h>
#include <Lib/Cast.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>

#include <BatchLab/Zebra/CHBOOK.h>

//////////////////////////////////////////////////////////////////////////////
inline int hindex(
 int aIndex
,const AIDA::IAxis& aAxis
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aIndex==AIDA::IAxis::UNDERFLOW_BIN) 
    return 0;
  else if(aIndex==AIDA::IAxis::OVERFLOW_BIN) 
    return aAxis.bins()+1;
  else 
    return aIndex+1;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Axis::Axis(
 int aID
,int aDimension
,bool aIsX
,bool aFixed
)
:fID(aID)
,fDimension(aDimension)
,fIsX(aIsX)
,fFixed(aFixed)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Axis::Axis(
 const Axis& aFrom
)
:AIDA::IAxis(aFrom)
,fID(aFrom.fID)
,fDimension(aFrom.fDimension)
,fIsX(aFrom.fIsX)
,fFixed(aFrom.fFixed)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Axis::~Axis(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Axis::isFixedBinning(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFixed;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Axis::lowerEdge(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  int xn,yn;
  float xmin,xmax,ymin,ymax;
  CHGIVE(fID,s,xn,xmin,xmax,yn,ymin,ymax);
  return fIsX?xmin:ymin;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Axis::upperEdge(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  int xn,yn;
  float xmin,xmax,ymin,ymax;
  CHGIVE(fID,s,xn,xmin,xmax,yn,ymin,ymax);
  return fIsX?xmax:ymax;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Axis::bins(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  int xn,yn;
  float xmin,xmax,ymin,ymax;
  CHGIVE(fID,s,xn,xmin,xmax,yn,ymin,ymax);
  return fIsX?xn:yn;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Axis::binLowerEdge(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int index = aIndex + 1; //HBOOK counts in [1,NX]
  if(fDimension==1) {
    return CHIX(fID,index);
  } else { //dimension 2
    float x,y;
    if(fIsX) {
      CHIJXY(fID,index,1,x,y);
      return x;
    } else {
      CHIJXY(fID,1,index,x,y);
      return y;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Axis::binUpperEdge(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : is it correct to ask for the lower bound of the next bin ?
  int index = aIndex + 1 + 1; //HBOOK counts in [1,NX]
  if(fDimension==1) {
    return CHIX(fID,index);
  } else { //dimension 2
    float x,y;
    if(fIsX) {
      CHIJXY(fID,index,1,x,y);
      return x;
    } else {
      CHIJXY(fID,1,index,x,y);
      return y;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Axis::binWidth(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return binUpperEdge(aIndex) - binLowerEdge(aIndex);
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Axis::coordToIndex(
 double aCoord
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fDimension==1) {
    return CHXI(fID,float(aCoord))-1; //HBOOK counts in [1,NX]
  } else { //dimension 2
    int i,j;
    if(fIsX) {
      CHXYIJ(fID,float(aCoord),0,i,j);
      return i-1;
    } else {
      CHXYIJ(fID,0,float(aCoord),i,j);
      return j-1;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Axis::binCenter(
 int aIndex
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (binLowerEdge(aIndex) + binUpperEdge(aIndex))/2.0;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::BaseHistogram::BaseHistogram(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseObject(aPrinter,aName)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::BaseHistogram::BaseHistogram(
 Slash::Core::IWriter& //aPrinter
,const BaseHistogram& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,BaseObject(aFrom)
,fAnnotation(aFrom.fAnnotation)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::BaseHistogram::~BaseHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Annotation& BatchLab::Zebra::BaseHistogram::annotation(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Zebra::BaseHistogram::setAnnotation(
 const BatchLab::Annotation& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation = aFrom;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Histogram1D::Histogram1D(
 Slash::Core::IWriter& aPrinter
,int aID
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
)
:BaseHistogram(aPrinter,Lib::smanip::tostring(aID))
,fID(aID)
,fAxis(aID,1,true,true)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
  CHBOOK1(fID,aTitle,aXnumber,(float)aXmin,(float)aXmax);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Histogram1D::Histogram1D(
 Slash::Core::IWriter& aPrinter
,int aID
,const std::string& aTitle
,const std::vector<double>& aEdges
)
:BaseHistogram(aPrinter,Lib::smanip::tostring(aID))
,fID(aID)
,fAxis(aID,1,true,false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation.setValue("Title",aTitle);
  unsigned int edgen = aEdges.size();
  std::vector<float> edges(edgen);
  for(unsigned int i=0;i<edgen;i++) edges[i] = float(aEdges[i]);
  CHBOOKB(fID,aTitle,edges);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Histogram1D::Histogram1D(
 const Histogram1D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::IHistogram(aFrom)
,AIDA::IHistogram1D(aFrom)
,BaseHistogram(aFrom.printer(),aFrom)
,fAxis(aFrom.fAxis)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  printf("debug : BatchLab::Zebra::Histogram1D::Histogram1D(3) : dummy\n");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Histogram1D::~Histogram1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Zebra::Histogram1D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Zebra::Histogram1D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Histogram1D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Zebra::Histogram1D";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Zebra::Histogram1D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if_Lib_SCast(AIDA::IHistogram1D)
  else if_Lib_SCast(AIDA::IHistogram)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(BatchLab::Zebra::Histogram1D)
  else if_Lib_SCast(BatchLab::Zebra::BaseHistogram)
  else return BaseObject::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Histogram1D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IHistogram1D";
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram1D::fill(
 double aX
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  CHFILL(fID,(float)aX,0,(float)aWeight);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram1D::add(
 const AIDA::IHistogram1D& //aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  printf("debug : BatchLab::Zebra::Histogram1D::add : dummy\n");
  return false;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Histogram1D::title(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string title;
  int ncx,ncy;
  float xmin,xmax,ymin,ymax;
  CHGIVE(fID,title,ncx,xmin,xmax,ncy,ymin,ymax);
  return title;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram1D::setTitle(
 const std::string& aTitle
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation.setValue("Title",aTitle);
  printf("debug : BatchLab::Zebra::Histogram1D::setTitle : dummy\n");
  return false;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IAnnotation& BatchLab::Zebra::Histogram1D::annotation(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAnnotation& BatchLab::Zebra::Histogram1D::annotation(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Histogram1D::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{  
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram1D::reset(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  CHRESET(fID," ");
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Histogram1D::allEntries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHNOENT(fID);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::minBinHeight(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHMIN(fID);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::maxBinHeight(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHMAX(fID);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::mean(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHSTATI(fID,1," ",0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::rms(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHSTATI(fID,2," ",0);
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAxis& BatchLab::Zebra::Histogram1D::axis(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAxis;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Histogram1D::coordToIndex(
 double aCoord
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAxis.coordToIndex(aCoord);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::binHeight(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHI(fID,hindex(aIndex,axis()));
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::sumBinHeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NX = axis().bins();
  float w = 0;
  for(int i=1;i<=NX;i++) {
    w += CHI(fID,i);
  }
  return w;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::sumAllBinHeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NX1 = axis().bins()+1;
  float w = 0;
  for(int i=0;i<=NX1;i++) {
    w += CHI(fID,i);
  }
  return w;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::sumExtraBinHeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NX1 = axis().bins()+1;
  float w = 0;
  w += CHI(fID,0);
  w += CHI(fID,NX1);
  return w;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram1D::binError(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHIE(fID,aIndex+1);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram1D::scale(
 double aScale
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : Do we want the E option ?
  CHOPERA(fID,"+E",fID,fID,aScale,0);
  return true;
}

//FIXME : below are yet dummy.
int BatchLab::Zebra::Histogram1D::entries() const {
  printf("debug : BatchLab::Zebra::Histogram1D::entries : dummy\n");
  //return CHNOENT(fID);
  return 0;
}
int BatchLab::Zebra::Histogram1D::nanEntries() const {
  printf("debug : BatchLab::Zebra::Histogram1D::nanEntries : dummy\n");
  return 0;
}
int BatchLab::Zebra::Histogram1D::extraEntries() const {
  printf("debug : BatchLab::Zebra::Histogram1D::extraEntries : dummy\n");
  return 0;
}
double BatchLab::Zebra::Histogram1D::equivalentBinEntries() const {
  printf("debug : BatchLab::Zebra::Histogram1D::equivalentBinEntries : dummy\n");
  return 0;
}
double BatchLab::Zebra::Histogram1D::binMean(int /*aIndex*/) const {
  printf("debug : BatchLab::Zebra::Histogram1D::binMean : dummy\n");
  return 0;
}
double BatchLab::Zebra::Histogram1D::binStdDev(int /*aIndex*/) const {
  printf("debug : BatchLab::Zebra::Histogram1D::binStdDev : dummy\n");
  return 0;
}
int BatchLab::Zebra::Histogram1D::binEntries(int /*aIndex*/) const {
  printf("debug : BatchLab::Zebra::Histogram1D::binEntries : dummy\n");
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/*
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,int aID
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
)
:BaseHistogram(aPrinter,Lib::smanip::tostring(aID))
,fID(aID)
,fAxis(aID,1,true,true)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation.setValue("Title",aTitle);
  CHBPROF(fID,aTitle,aXnumber,(float)aXmin,(float)aXmax,"");
}
*/
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,int aID
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,double aVmin
,double aVmax
)
:BaseHistogram(aPrinter,Lib::smanip::tostring(aID))
,fID(aID)
,fAxis(aID,1,true,true)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation.setValue("Title",aTitle);
  CHBPROF(fID,aTitle,
          aXnumber,(float)aXmin,(float)aXmax,
          (float)aVmin,(float)aVmax,
          ""); //FIXME : or "S" or "T" ?
}
/*
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdges
)
:BaseHistogram(aPrinter,aName)
//,fHistogram(aTitle,aEdges)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile1D::Profile1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdges
,double aVmin
,double aVmax
)
:BaseHistogram(aPrinter,aName)
//,fHistogram(aTitle,aEdges,aVmin,aVmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxis.copy(fHistogram.getAxis(0));
  fAnnotation.setValue("Title",aTitle);
}
*/
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile1D::Profile1D(
 const Profile1D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::IProfile(aFrom)
,AIDA::IProfile1D(aFrom)
,BaseHistogram(aFrom.printer(),aFrom)
,fAxis(aFrom.fAxis)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  printf("debug : BatchLab::Zebra::Profile1D::Profile1D(3) : dummy\n");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile1D::~Profile1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Zebra::Profile1D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Zebra::Profile1D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Profile1D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Zebra::Profile1D";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Zebra::Profile1D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if_Lib_SCast(AIDA::IProfile1D)
  else if_Lib_SCast(AIDA::IProfile)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(BatchLab::Zebra::Profile1D)
  else if_Lib_SCast(BatchLab::Zebra::BaseHistogram)
  else return BaseObject::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Profile1D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IProfile1D";
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Profile1D::fill(
 double aX
,double aY
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  CHFILL(fID,(float)aX,(float)aY,(float)aWeight);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Profile1D::add(
 const AIDA::IProfile1D& aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  printf("debug : BatchLab::Zebra::Profile1D::add : dummy\n");
  return false;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Profile1D::title(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string title;
  int ncx,ncy;
  float xmin,xmax,ymin,ymax;
  CHGIVE(fID,title,ncx,xmin,xmax,ncy,ymin,ymax);
  return title;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Profile1D::setTitle(
 const std::string& aTitle
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation.setValue("Title",aTitle);
  printf("debug : BatchLab::Zebra::Profile1D::setTitle : dummy\n");
  return false;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IAnnotation& BatchLab::Zebra::Profile1D::annotation(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAnnotation& BatchLab::Zebra::Profile1D::annotation(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Profile1D::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Profile1D::reset(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  CHRESET(fID," ");
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Profile1D::allEntries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHNOENT(fID);
}

//FIXME : below are yet dummy.
int BatchLab::Zebra::Profile1D::entries() const {
  printf("debug : BatchLab::Zebra::Profile1D::entries : dummy\n");
  return 0;
}
int BatchLab::Zebra::Profile1D::extraEntries() const {
  printf("debug : BatchLab::Zebra::Profile1D::extraEntries : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::equivalentBinEntries() const {
  printf("debug : BatchLab::Zebra::Profile1D::equivalentBinEntries : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::sumBinHeights() const {
  printf("debug : BatchLab::Zebra::Profile1D::sumBinHeights : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::sumAllBinHeights() const {
  printf("debug : BatchLab::Zebra::Profile1D::sumAllBinHeights : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::sumExtraBinHeights() const {
  printf("debug : BatchLab::Zebra::Profile1D::sumExtraBinHeights : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::minBinHeight() const {
  printf("debug : BatchLab::Zebra::Profile1D::minBinHeight : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::maxBinHeight() const {
  printf("debug : BatchLab::Zebra::Profile1D::maxBinHeight : dummy\n");
  return 0;
}
bool BatchLab::Zebra::Profile1D::scale(double aScale) {
  printf("debug : BatchLab::Zebra::Profile1D::scale : dummy\n");
  return false;
}
double BatchLab::Zebra::Profile1D::binMean(int aIndex) const {
  printf("debug : BatchLab::Zebra::Profile1D::binMean : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::binStdDev(int aIndex) const {
  printf("debug : BatchLab::Zebra::Profile1D::binStdDev : dummy\n");
  return 0;
}
int BatchLab::Zebra::Profile1D::binEntries(int aIndex) const {
  printf("debug : BatchLab::Zebra::Profile1D::binEntries : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::binHeight(int aIndex) const {
  printf("debug : BatchLab::Zebra::Profile1D::binHeight : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::binError(int aIndex) const {
  printf("debug : BatchLab::Zebra::Profile1D::binError : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::binRms(int aIndex) const {
  printf("debug : BatchLab::Zebra::Profile1D::binRms : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::mean() const {
  printf("debug : BatchLab::Zebra::Profile1D::mean : dummy\n");
  return 0;
}
double BatchLab::Zebra::Profile1D::rms() const {
  printf("debug : BatchLab::Zebra::Profile1D::mean : rms\n");
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAxis& BatchLab::Zebra::Profile1D::axis(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAxis;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Profile1D::coordToIndex(
 double aCoord
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAxis.coordToIndex(aCoord);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Histogram2D::Histogram2D(
 Slash::Core::IWriter& aPrinter
,int aID
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
)
:BaseHistogram(aPrinter,Lib::smanip::tostring(aID))
,fID(aID)
,fAxisX(aID,2,true,true)
,fAxisY(aID,2,false,true)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxisX.copy(fHistogram.getAxis(0));
  //fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
  CHBOOK2(fID,aTitle,aXnumber,(float)aXmin,(float)aXmax,
                     aYnumber,(float)aYmin,(float)aYmax);
}
/*
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Histogram2D::Histogram2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
)
:BaseHistogram(aPrinter,aName)
//,fHistogram(aTitle,aEdgesX,aEdgesY)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxisX.copy(fHistogram.getAxis(0));
  //fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
*/
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Histogram2D::Histogram2D(
 const Histogram2D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::IHistogram(aFrom)
,AIDA::IHistogram2D(aFrom)
,BaseHistogram(aFrom.printer(),aFrom)
,fAxisX(aFrom.fAxisX)
,fAxisY(aFrom.fAxisY)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  printf("debug : BatchLab::Zebra::Histogram2D::Histogram2D(3) : dummy\n");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Histogram2D::~Histogram2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Zebra::Histogram2D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Zebra::Histogram2D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Histogram2D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Zebra::Histogram2D";
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Zebra::Histogram2D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if_Lib_SCast(AIDA::IHistogram2D)
  else if_Lib_SCast(AIDA::IHistogram)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(BatchLab::Zebra::Histogram2D)
  else if_Lib_SCast(BatchLab::Zebra::BaseHistogram)
  else return BaseObject::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Histogram2D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IHistogram2D";
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram2D::fill(
 double aX
,double aY
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  CHFILL(fID,(float)aX,(float)aY,(float)aWeight);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram2D::add(
 const AIDA::IHistogram2D& //aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //BatchLab::Zebra::Histogram2D* batchlabHisto = 
  //   Lib_ConstCast(aHisto,BatchLab::Zebra::Histogram2D);
  //if(!batchlabHisto) return false;
  //fHistogram.add(batchlabHisto->bareHistogram());
  printf("debug : BatchLab::Zebra::Histogram2D::add : dummy\n");
  return false;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Histogram2D::title(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string title;
  int ncx,ncy;
  float xmin,xmax,ymin,ymax;
  CHGIVE(fID,title,ncx,xmin,xmax,ncy,ymin,ymax);
  return title;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram2D::setTitle(
 const std::string& aTitle
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation.setValue("Title",aTitle);
  printf("debug : BatchLab::Zebra::Histogram2D::setTitle : dummy\n");
  return false;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IAnnotation& BatchLab::Zebra::Histogram2D::annotation(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAnnotation& BatchLab::Zebra::Histogram2D::annotation(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Histogram2D::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 2;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram2D::reset(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  CHRESET(fID," ");
  return true;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::meanX(
) const{
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
 
  return CHSTATI(fID,1,"PROX",0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::meanY(
) const{
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
 
  return CHSTATI(fID,1,"PROY",0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::rmsX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return CHSTATI(fID,2,"PROX",0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::rmsY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHSTATI(fID,2,"PROY",0);
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAxis& BatchLab::Zebra::Histogram2D::xAxis(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fAxisX;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAxis& BatchLab::Zebra::Histogram2D::yAxis(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fAxisY;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Histogram2D::coordToIndexX(
 double aCoordX
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fAxisX.coordToIndex(aCoordX);
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Histogram2D::coordToIndexY(
 double aCoordY
) const
/////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fAxisY.coordToIndex(aCoordY);
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Zebra::Histogram2D::allEntries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHNOENT(fID);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::minBinHeight(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHMIN(fID);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::maxBinHeight(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return CHMAX(fID);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::binHeight(
 int aIndexX
,int aIndexY
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return CHIJ(fID,
              hindex(aIndexX,xAxis()),
              hindex(aIndexY,yAxis()));
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::sumBinHeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NX = xAxis().bins();
  int NY = yAxis().bins();
  float w = 0;
  for(int i=1;i<=NX;i++) {
    for(int j=1;j<=NY;j++) {
      w += CHIJ(fID,i,j);
    }
  }
  return w;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::sumAllBinHeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NX1 = xAxis().bins()+1;
  int NY1 = yAxis().bins()+1;
  float w = 0;
  for(int i=0;i<=NX1;i++) {
    for(int j=0;j<=NY1;j++) {
      w += CHIJ(fID,i,j);
    }
  }
  return w;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::sumExtraBinHeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NX = xAxis().bins();
  int NY = yAxis().bins();
  int NX1 = NX+1;
  int NY1 = NY+1;
  float w = 0;
 {for(int j=0;j<=NY1;j++) {
    w += CHIJ(fID,0,j);
  }}
  for(int i=1;i<=NX;i++) {
    w += CHIJ(fID,i,0);
    w += CHIJ(fID,i,NY1);
  }
 {for(int j=0;j<=NY1;j++) {
    w += CHIJ(fID,NX1,j);
  }}
  return w;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::binError(
 int aIndexX
,int aIndexY
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return CHIJE(fID,aIndexX+1,aIndexY+1);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::binHeightX(
 int aIndexX
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int i = hindex(aIndexX,xAxis());
  int NY1 = yAxis().bins()+1;
  float w = 0;
  for(int j=0;j<=NY1;j++) {
    w += CHIJ(fID,i,j);
  }
  return w;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Zebra::Histogram2D::binHeightY(
 int aIndexY
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int j = hindex(aIndexY,yAxis());
  int NX1 = xAxis().bins()+1;
  float w = 0;
  for(int i=0;i<=NX1;i++) {
    w += CHIJ(fID,i,j);
  }
  return w;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Histogram2D::scale(
 double aScale
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : Do we want the E option ?
  CHOPERA(fID,"+E",fID,fID,aScale,0);
  return true;
}

//FIXME : below are yet dummy.
int BatchLab::Zebra::Histogram2D::entries() const {
  printf("debug : BatchLab::Zebra::Histogram2D::entries : dummy\n");
  return 0;
}
int BatchLab::Zebra::Histogram2D::nanEntries() const {
  printf("debug : BatchLab::Zebra::Histogram2D::nanEntries : dummy\n");
  //return CHNOENT(fID);
  return 0;
}
int BatchLab::Zebra::Histogram2D::extraEntries() const {
  printf("debug : BatchLab::Zebra::Histogram2D::extraEntries : dummy\n");
  return 0;
}
double BatchLab::Zebra::Histogram2D::equivalentBinEntries() const {
  printf("debug : BatchLab::Zebra::Histogram2D::equivalentBinEntries : dummy\n");
  return 0;
}
double BatchLab::Zebra::Histogram2D::binMeanX(int /*aIndexX*/,int /*aIndexY*/) const { 
  printf("debug : BatchLab::Zebra::Histogram2D::binMeanX : dummy\n");
  return 0;
}
double BatchLab::Zebra::Histogram2D::binMeanY(int /*aIndexX*/,int /*aIndexY*/) const { 
  printf("debug : BatchLab::Zebra::Histogram2D::binMeanY : dummy\n");
  return 0;
}
double BatchLab::Zebra::Histogram2D::binStdDevX(int /*aIndexX*/,int /*aIndexY*/) const {
  printf("debug : BatchLab::Zebra::Histogram2D::binStdDevX : dummy\n");
  return 0;
}
double BatchLab::Zebra::Histogram2D::binStdDevY(int /*aIndexX*/,int /*aIndexY*/) const {
  printf("debug : BatchLab::Zebra::Histogram2D::binStdDevY : dummy\n");
  return 0;
}
int BatchLab::Zebra::Histogram2D::binEntries(int /*aIndexX*/,int /*aIndexY*/) const { 
  printf("debug : BatchLab::Zebra::Histogram2D::binEntries : dummy\n");
  return 0;
}
int BatchLab::Zebra::Histogram2D::binEntriesX(int /*aIndexX*/) const { 
  printf("debug : BatchLab::Zebra::Histogram2D::binEntriesX : dummy\n");
  return 0;
}
int BatchLab::Zebra::Histogram2D::binEntriesY(int /*aIndexY*/) const { 
  printf("debug : BatchLab::Zebra::Histogram2D::binEntriesY : dummy\n");
  return 0;
}
/*
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,HCL::Histogram& aHclHisto
)
:BaseHistogram(aPrinter,aName)
,fHistogram("",10,0,1,10,0,1)
//////////////////////////////////////////////////////////////////////////////
// It is assumed that BatchLab::Zebra::Histogram will take care of the aHclHisto.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create a valid histo :
  if(aHclHisto.getDimension()!=2) {
    Lib::Out out(printer());
    out << "Lba::Profile2D::Profile2D :"
        << " dimension mismatch." << Lib::endl;
  } else {
    fHistogram.copy(aHclHisto);
  }
  fAxisX.copy(fHistogram.getAxis(0));
  fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",fHistogram.title());
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
)
:BaseHistogram(aPrinter,aName)
//,fHistogram(aTitle,aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxisX.copy(fHistogram.getAxis(0));
  //fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
,double aVmin
,double aVmax
)
:BaseHistogram(aPrinter,aName)
//,fHistogram(aTitle,aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax,aVmin,aVmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxisX.copy(fHistogram.getAxis(0));
  //fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
)
:BaseHistogram(aPrinter,aName)
//,fHistogram(aTitle,aEdgesX,aEdgesY)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxisX.copy(fHistogram.getAxis(0));
  //fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile2D::Profile2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
,double aVmin
,double aVmax
)
:BaseHistogram(aPrinter,aName)
//,fHistogram(aTitle,aEdgesX,aEdgesY,aVmin,aVmax)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //fAxisX.copy(fHistogram.getAxis(0));
  //fAxisY.copy(fHistogram.getAxis(1));
  fAnnotation.setValue("Title",aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile2D::Profile2D(
 const Profile2D& aFrom
)
:BaseHistogram(aFrom.printer(),aFrom)
//,fHistogram(aFrom.fHistogram)
//,fAxisX(aFrom.fAxisX)
//,fAxisY(aFrom.fAxisY)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Zebra::Profile2D::~Profile2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Zebra::Profile2D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Zebra::Profile2D(*this);
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Zebra::Profile2D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if_Lib_SCast(AIDA::IProfile2D)
  else if_Lib_SCast(AIDA::IProfile)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(BatchLab::Zebra::Profile2D)
  else if_Lib_SCast(BatchLab::Zebra::BaseHistogram)
  else return BaseObject::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Profile2D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IProfile2D";
}
//////////////////////////////////////////////////////////////////////////////
const HCL::Histogram& BatchLab::Zebra::Profile2D::bareHistogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Profile2D::fill(
 double aX
,double aY
,double aZ
,double aWeight
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  bool status = fHistogram.fill(aX,aY,aZ,aWeight);
  if(status) emitUpdate();
  return status;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Zebra::Profile2D::add(
 const IProfile2D& aHisto
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  BatchLab::Zebra::Profile2D* batchlabHisto = Lib_Cast(aHisto,BatchLab::Zebra::Profile2D);
  if(!batchlabHisto) return false;
  //fHistogram.add(batchlabHisto->bareHistogram());
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Zebra::Profile2D::title() const {
  return fHistogram.title();}
bool BatchLab::Zebra::Profile2D::setTitle(const std::string& aTitle) {
  fAnnotation.setValue("Title",aTitle);
  return fHistogram.setTitle(aTitle);
}
AIDA::IAnnotation& BatchLab::Zebra::Profile2D::annotation() {
  return fAnnotation;
}
const AIDA::IAnnotation& BatchLab::Zebra::Profile2D::annotation() const {
  return fAnnotation;
}
int BatchLab::Zebra::Profile2D::dimension() const {
  return 2;
}
bool BatchLab::Zebra::Profile2D::reset() {
  CHRESET(fID," ");
  return true;
}
int BatchLab::Zebra::Profile2D::entries() const {
  return fHistogram.entries();}
int BatchLab::Zebra::Profile2D::allEntries() const {
  return fHistogram.allEntries();}
int BatchLab::Zebra::Profile2D::extraEntries() const {
  return fHistogram.extraEntries();}
double BatchLab::Zebra::Profile2D::equivalentBinEntries() const {
  return fHistogram.equivalentBinEntries();}
double BatchLab::Zebra::Profile2D::sumBinHeights() const {
  return fHistogram.sumBinHeights();}
double BatchLab::Zebra::Profile2D::sumAllBinHeights() const {
  return fHistogram.sumAllBinHeights();}
double BatchLab::Zebra::Profile2D::sumExtraBinHeights() const {
  return fHistogram.sumExtraBinHeights();}
double BatchLab::Zebra::Profile2D::minBinHeight() const {
  return fHistogram.minBinHeight();}
double BatchLab::Zebra::Profile2D::maxBinHeight() const {
  return fHistogram.maxBinHeight();}
bool BatchLab::Zebra::Profile2D::scale(double aScale) {
  return fHistogram.scale(aScale);}
double BatchLab::Zebra::Profile2D::binMeanX(int aIndexX,int aIndexY) const { 
  return fHistogram.binMeanX(aIndexX,aIndexY);}
double BatchLab::Zebra::Profile2D::binRms(int aIndexX,int aIndexY) const {
  return fHistogram.binRms(aIndexX,aIndexY);}
double BatchLab::Zebra::Profile2D::binMeanY(int aIndexX,int aIndexY) const { 
  return fHistogram.binMeanY(aIndexX,aIndexY);}
double BatchLab::Zebra::Profile2D::binStdDevX(int aIndexX,int aIndexY) const {
  return fHistogram.binStdDevX(aIndexX,aIndexY);}
double BatchLab::Zebra::Profile2D::binStdDevY(int aIndexX,int aIndexY) const {
  return fHistogram.binStdDevY(aIndexX,aIndexY);}
int BatchLab::Zebra::Profile2D::binEntries(int aIndexX,int aIndexY) const { 
  return fHistogram.binEntries(aIndexX,aIndexY);}
int BatchLab::Zebra::Profile2D::binEntriesX(int aIndexX) const { 
  return fHistogram.binEntriesX(aIndexX);}
int BatchLab::Zebra::Profile2D::binEntriesY(int aIndexY) const { 
  return fHistogram.binEntriesY(aIndexY);}
double BatchLab::Zebra::Profile2D::binHeight(int aIndexX,int aIndexY) const { 
  return fHistogram.binHeight(aIndexX,aIndexY);}
double BatchLab::Zebra::Profile2D::binHeightX(int aIndexX) const { 
  return fHistogram.binHeightX(aIndexX);}
double BatchLab::Zebra::Profile2D::binHeightY(int aIndexY) const{ 
  return fHistogram.binHeightY(aIndexY);}
double BatchLab::Zebra::Profile2D::binError(int aIndexX,int aIndexY) const{ 
  return fHistogram.binError(aIndexX,aIndexY);}
double BatchLab::Zebra::Profile2D::meanX() const{ 
  return fHistogram.meanX();}
double BatchLab::Zebra::Profile2D::meanY() const{ 
  return fHistogram.meanY();}
double BatchLab::Zebra::Profile2D::rmsX() const{ 
  return fHistogram.rmsX();}
double BatchLab::Zebra::Profile2D::rmsY() const{ 
  return fHistogram.rmsY();}
const AIDA::IAxis& BatchLab::Zebra::Profile2D::xAxis() const{ return fAxisX;}
const AIDA::IAxis& BatchLab::Zebra::Profile2D::yAxis() const{ return fAxisY;}
int BatchLab::Zebra::Profile2D::coordToIndexX(double aCoordX) const{ 
  return fHistogram.coordToIndexX(aCoordX);}
int BatchLab::Zebra::Profile2D::coordToIndexY(double aCoordY) const{ 
  return fHistogram.coordToIndexY(aCoordY);}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
*/
