#ifndef BatchLab_HDF5_Tools_h
#define BatchLab_HDF5_Tools_h

#include <string>
#include <vector>

#include <hdf5.h>

namespace BatchLab {

namespace HDF5 {

  class Helper {
  public:
    static bool checkSizes();

    static hid_t string_datatype(unsigned int);

    static hid_t basic_mem_type(hid_t);
    static hid_t compound_mem_type(hid_t);

    // std::string :
    static bool read_string(hid_t,const std::string&,
                            std::string&);
    static bool read_array_string(hid_t,const std::string&,
                                  std::vector<std::string>&);
    static bool put_array_string(const std::vector<std::string>&,char*);
    static bool put_array_string(unsigned int,char*,
                                 std::vector<std::string>&);

    // atbs :
    static bool read_atb(hid_t,const std::string&,
                         std::string&,unsigned int = 100);
    static bool read_atb(hid_t,const std::string&,int&);
    static bool read_atb(hid_t,const std::string&,unsigned int&);

    // Else :
    static void dump(hid_t,const std::string&);
  public:
    Helper(int level = 1);
    virtual ~Helper();
    bool write_atb(hid_t,const std::string&,const std::string&);
    bool write_atb(hid_t,const std::string&,int);
    bool write_atb(hid_t,const std::string&,unsigned int);
    bool write_object(hid_t,const std::string&,hid_t,unsigned int,char*);
    bool read_object(hid_t,const std::string&,unsigned int&,char*&);

    bool write_int(hid_t,const std::string&,int);
    bool write_uint(hid_t,const std::string&,unsigned int);
    bool write_double(hid_t,const std::string&,double);
    bool write_bool(hid_t,const std::string&,bool);
    bool write_float(hid_t,const std::string&,float);
    bool write_string(hid_t,const std::string& dset,const std::string&);
    bool write_array_string(hid_t,const std::string&,
                            const std::vector<std::string>&);
    bool write_array_int(hid_t,const std::string&,
                         unsigned int,const int []);
    bool write_array_int(hid_t,const std::string&,
                         const std::vector<int>&);
    bool write_array_uchar(hid_t,const std::string&,
                           const std::vector<unsigned char>&);
    bool write_array_double(hid_t,const std::string&,
                            unsigned int,const double[]);
    bool write_array_double(hid_t,const std::string&,
                            const std::vector<double>&);
    bool write_array_float(hid_t,const std::string&,
                           unsigned int,const float[]);
    bool write_array_float(hid_t,const std::string&,
                           const std::vector<float>&);

    template <class T>
      bool write_struct(hid_t aLocation,const std::string& aName, 
                        hid_t aCreateType,hid_t aWriteType,const T& aData) {
        hid_t dataset = 
          H5Dcreate(aLocation,aName.c_str(),aCreateType,fScalar,fCompact);
        if(dataset<0) return false;
        if(H5Dwrite(dataset,aWriteType,H5S_ALL,H5S_ALL,H5P_DEFAULT,&aData)<0) {
          H5Dclose(dataset);                 
          return false;
        }
        H5Dclose(dataset);                 
        return true;
      }
  private:
    int fCompress;
    hid_t fScalar;
    hid_t fCompact;
    hid_t fSimple;
    hid_t fCPT;
  };

  bool read_array_int(hid_t,const std::string&,unsigned int&,int*&);
  bool read_int(hid_t,const std::string&,int&);
  bool read_uint(hid_t,const std::string&,unsigned int&);
  bool read_bool(hid_t,const std::string&,bool&);
  bool read_array_int(hid_t,const std::string&,std::vector<int>&);
  bool read_array_uchar(hid_t,const std::string&,std::vector<unsigned char>&);
  bool read_array_double(hid_t,const std::string&,
                         unsigned int&,double*&);
  bool read_double(hid_t,const std::string&,double&);
  bool read_array_double(hid_t,const std::string&,std::vector<double>&);
  bool read_array_float(hid_t,const std::string&,
                         unsigned int&,float*&);
  bool read_float(hid_t,const std::string&,float&);
  bool read_array_float(hid_t,const std::string&,std::vector<float>&);

} //HDF5

} //BatchLab

#endif
