// From Geant4/source/visualization/FukuiRenderer/G4FRBaseSocket code.
// Itself coming from dawn-3.85a.
//  tag : geant4-07-00-cand-00

// this :
#include <Lib/BaseSocket.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/Debug.h>

#ifdef WIN32
#include <io.h>
#include <winsock.h>
#else
#include <sys/socket.h>
#include <errno.h>
#include <unistd.h>  //read,close
#include <string.h> //strerror
#endif

//////////////////////////////////////////////////////////////////////////////
Lib::BaseSocket::BaseSocket(
 Slash::Core::IWriter& aPrinter
,int aVerboseLevel
)
:fPrinter(aPrinter)
,fVerboseLevel(aVerboseLevel)
,fSocket(-1)           
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Debug::increment("Lib::BaseSocket");
}
//////////////////////////////////////////////////////////////////////////////
Lib::BaseSocket::~BaseSocket(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  closeSocket();
  Debug::decrement("Lib::BaseSocket");
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::BaseSocket::isConnected(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (fSocket==(-1)?false:true);           
}
//////////////////////////////////////////////////////////////////////////////
int Lib::BaseSocket::socket(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fSocket; 
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool Lib::BaseSocket::sendString(
 const std::string& aMessage 
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fSocket==(-1)) return false;
  if(::send(fSocket,(char*)aMessage.c_str(),aMessage.size()+1,0)<0) {
    Out out(fPrinter);
    out << "Lib::BaseSocket::sendString :"
        << " send : " << std::string(strerror(errno))
        << Lib::endl;
    disconnect();
    return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::BaseSocket::sendText(
 const std::vector<std::string>& aText
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  unsigned int l = aText.size();
  for(unsigned int index=0;index<l;index++) {
    if(!sendString(aText[index])) return false;
  }
  if(!sendString("Lib::end_of_text")) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::BaseSocket::waitString(
 std::string& aLine
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fSocket==(-1)) return false;
#define RECV_BUFMAX 1024
  char buf[ RECV_BUFMAX ];
  int index = 0 ;
  while(1) {
    char ch ;
    //int num_read_char = ::read(fSocket,&ch,1);
    int flag = 0;
    int num_read_char = ::recv(fSocket,&ch,1,flag);
    if(num_read_char==1) {
      buf[index++] = ch ; 
      if( ch=='\0' ) { 
        aLine = buf;        
        return true;
      }
    } else if (num_read_char==0) {
      Out out(fPrinter);
      out << "Lib::BaseSocket::waitString :"
          << " recv(1) : " << std::string(strerror(errno))
          << " num_read_char " << num_read_char
          << Lib::endl;
      disconnect();
      aLine = "";
      return false;
    } else {
      Out out(fPrinter);
      out << "Lib::BaseSocket::waitString :"
          << " recv(2) : " << std::string(strerror(errno))
          << " num_read_char " << num_read_char
          << Lib::endl;
      disconnect();
      aLine = "";
      return false;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::BaseSocket::waitText(
 std::vector<std::string>& aText
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  aText.clear();
  while(1) {
    std::string s;
    if(!waitString(s)) {
      aText.clear();
      return false;
    }
    if(s=="Lib::end_of_text") return true;
    aText.push_back(s);
  }
}
//////////////////////////////////////////////////////////////////////////////
void Lib::BaseSocket::closeSocket(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fSocket!=(-1)) {
    if( ::shutdown(fSocket,2) < 0 ) { 
      Out out(fPrinter);
      out << "Lib::BaseSocket::closeSocket :"
            << " shutdown : " << std::string(strerror(errno))
            << Lib::endl;
    }
    ::close(fSocket);
    fSocket = -1;
  }
}
