// this :
#include <Lib/BaseZip.h>

#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/Debug.h>

#ifdef LIB_HAS_ZLIB
#include <zip.h>
#include <unzip.h>
#endif

#define WRITEBUFFERSIZE (16384)

//////////////////////////////////////////////////////////////////////////////
bool Lib::BaseZip::available(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef LIB_HAS_ZLIB
  return true;
#else
  return false;
#endif
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::BaseZip::BaseZip(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::BaseZip");
}
//////////////////////////////////////////////////////////////////////////////
Lib::BaseZip::~BaseZip(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("Lib::BaseZip");
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::BaseZip::loadFile(
 const std::string& aFile
,Slash::Core::IWriter& aPrinter
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef LIB_HAS_ZLIB
  //Lib::Out out(aPrinter);
  //out << "Lib::BaseZip::read :"
  //    << " load " << aFile << Lib::endl;

  unzFile uf = unzOpen(aFile.c_str());
  if(!uf) {
    Lib::Out out(aPrinter);
    out << "Lib::BaseZip::read :"
        << " error opening " << aFile << Lib::endl;
    return false;
  }

  unz_global_info gi;
  int err = unzGetGlobalInfo(uf,&gi);
  if(err!=UNZ_OK) {
    Lib::Out out(aPrinter);
    out << "Lib::BaseZip::read :"
        << " error in file " << aFile << Lib::endl;
    unzClose(uf);
    return false;
  }

  //out << "Lib::BaseZip::read :"
  //    << " number of entries " << int(gi.number_entry) << Lib::endl;

  for (uLong i=0;i<gi.number_entry;i++) {

    uLong ratio=0;

    unz_file_info file_info;
    char file_in_zip[256];
    err = unzGetCurrentFileInfo
      (uf,&file_info,file_in_zip,sizeof(file_in_zip),NULL,0,NULL,0);
    if(err!=UNZ_OK) {
      Lib::Out out(aPrinter);
      out << "Lib::BaseZip::read :"
          << " error in file " << aFile << Lib::endl;
      unzClose(uf);
      return false;
    }

    //out << std::string(file_in_zip) << Lib::endl;

    err = unzOpenCurrentFilePassword(uf,NULL);
    if(err!=UNZ_OK) {
      Lib::Out out(aPrinter);
      out << "Lib::BaseZip::read :"
          << " error in file " << aFile << Lib::endl;
      unzClose(uf);
      return false;
    }

    uLong size_buf = file_info.uncompressed_size;
    if(size_buf) {
      char* buf = new char[size_buf];
      int nread = unzReadCurrentFile(uf,buf,size_buf);
      if(nread<0) {
        Lib::Out out(aPrinter);
        out << "Lib::BaseZip::read :"
            << " error in file " << aFile << Lib::endl;
        delete [] buf;
        unzClose(uf);
        return false;
      }
/*
      if(nread>0) {
        for(int i=0;i<nread;i++) printf("%c",buf[i]);
        printf("\n");
      }
*/
      if(!visitEntry(file_in_zip,nread,buf)) {
        unzClose(uf);
        return false;
      }
      delete [] buf;
    } else {
      if(!visitEntry(file_in_zip,0,0)) {
        unzClose(uf);
        return false;
      }
    }

    if (i<gi.number_entry-1) {
      err = unzGoToNextFile(uf);
      if(err!=UNZ_OK) {
        Lib::Out out(aPrinter);
        out << "Lib::BaseZip::read :"
            << " error in file " << aFile << Lib::endl;
        unzClose(uf);
        return false;
      }
    }
  }


 {int err = unzClose(uf);
  if(err!=UNZ_OK) {
    Lib::Out out(aPrinter);
    out << "Lib::BaseZip::read :"
        << " error in closing file " << aFile << Lib::endl;
    return false;
  }}

  return true;
#else
  Lib::Out out(aPrinter);
  out << "Lib::BaseZip::read :"
      << " Lib package not reconstructed with zlib." << Lib::endl;
  return false;
#endif
}
