%{

// this :
#include <Lib/Columns.h>

#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/ColumnsLexer.h>
#include <Lib/smanip.h>
#include <Lib/valuemanip.h>

static Lib::Columns* sParser = 0;

extern int yylex();
int yyparse();
static void yyerror(char*); 

#define NotFound (-1)

%}

%union {
  int fIndex;
  unsigned long fInteger;
  double fNumber;
  Slash::Core::IValue::Type fType;
  Lib::Value* fVariable;
}

%token <fIndex> STRG
%token <fIndex> T_NAME
%token <fInteger> NUMBER_INTEGER
%token <fNumber> NUMBER_REAL 

%token T_COS T_SIN T_TAN T_ACOS T_ASIN T_ATAN T_SQRT T_POW T_LOG T_LOG10 
%token T_EXP T_FABS T_COSH T_SINH T_TANH
%token T_GAUSS T_BW T_EXPO T_POL1 T_POL2 T_POL3 T_POL4
%token T_PI T_TRUE T_FALSE
%token T_AND T_OR

%token T_SHORT T_INT T_INT64 T_FLOAT T_DOUBLE T_BOOLEAN T_BYTE T_CHAR T_STRING
%token T_JAVA_STRING T_TUPLE

%type <fType> type
%type <fVariable> value declaration

%left '-' '+'
%left '/' '*'
%nonassoc UMINUS

%%

result 
: error {
  // To reset lex and yacc in case of error.
} | declarations {
};

declarations 
: declaration {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : declarations : declaration : " 
        << $1->label() << Lib::endl;
  }
  sParser->fStack.push_front(*($1));
  delete $1;
  sParser->fTmpVars.pop_back();
} 
| declaration ',' declarations {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : dec , decls : " << $1->label() << Lib::endl;
  }
  sParser->fStack.push_front(*($1));
  delete $1;
  sParser->fTmpVars.pop_back();
}
| declaration ',' {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : dec , decls : " << $1->label() << Lib::endl;
  }
  sParser->fStack.push_front(*($1));
  delete $1;
  sParser->fTmpVars.pop_back();
};

declaration 
: T_TUPLE T_NAME '=' '{' declarations '}' {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_TUPLE T_NAME = {decs} : " 
        << sParser->lexer().variable($2) << Lib::endl;
  }
  Lib::Value* v = 
    new Lib::Value((void*)new std::list<Lib::Value>(sParser->fStack));
  v->setLabel(sParser->lexer().variable($2));
  sParser->fStack.clear();
  sParser->fCurrentType = Slash::Core::IValue::NONE; //FIXME
  $$ = v;
  sParser->fTmpVars.push_back($$);
} 
| T_NAME {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_NAME : " 
        << sParser->lexer().variable($1) << Lib::endl;
  }
  Lib::Value* v = new Lib::Value();
  v->setLabel(sParser->lexer().variable($1));
  v->setType(sParser->fCurrentType);
  $$ = v;
  sParser->fTmpVars.push_back($$);
} 
| type T_NAME {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : type T_NAME : " 
        << sParser->lexer().variable($2) << Lib::endl;
  }
  Lib::Value* v = new Lib::Value();
  v->setLabel(sParser->lexer().variable($2));
  v->setType($1);
  sParser->fCurrentType = $1;
  $$ = v;
  sParser->fTmpVars.push_back($$);
} 
| type '[' ']' T_NAME {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : type [] T_NAME : " 
        << sParser->lexer().variable($4) << Lib::endl;
  }
  Lib::Value* v = new Lib::Value();
  v->setLabel(sParser->lexer().variable($4));
  Slash::Core::IValue::Type atype;
  if(!Lib::Value::arrayType($1,atype)) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : type [] T_NAME :" 
        << " problem to get array type of " << Lib::Value::stype($1)
        << Lib::endl;
  }
  v->setType(atype);
  sParser->fCurrentType = atype;
  $$ = v;
  sParser->fTmpVars.push_back($$);
} 
| T_NAME '=' value {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_NAME = value : " 
        << sParser->lexer().variable($1) << Lib::endl;
  }
  Lib::Value* v = new Lib::Value();
  v->setLabel(sParser->lexer().variable($1));
  v->setType(sParser->fCurrentType);
  std::string serror;
  if(!Lib::valuemanip::assign(*v,*($3),sParser->fError)) 
    sParser->fSuccess = false;
  delete $3;
  $$ = v;
  sParser->fTmpVars.pop_back();
  sParser->fTmpVars.push_back($$);
} 
| type T_NAME '=' value {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : type T_NAME = value : " 
        << sParser->lexer().variable($2) << Lib::endl;
  }
  Lib::Value* v = new Lib::Value();
  v->setLabel(sParser->lexer().variable($2));
  v->setType($1);
  if(!Lib::valuemanip::assign(*v,*($4),sParser->fError)) 
    sParser->fSuccess = false;
  delete $4;
  sParser->fCurrentType = $1;
  $$ = v;
  sParser->fTmpVars.pop_back();
  sParser->fTmpVars.push_back($$);
}; 


type  
: T_SHORT { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_SHORT" << Lib::endl;
  }
  $$ = Slash::Core::IValue::SHORT;
} 
| T_INT { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_INT" << Lib::endl;
  }
  $$ = Slash::Core::IValue::INT;
} 
| T_INT64 { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_INT64" << Lib::endl;
  }
  $$ = Slash::Core::IValue::INT64;
} 
| T_FLOAT { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_FLOAT" << Lib::endl;
  }
  $$ = Slash::Core::IValue::FLOAT;
} 
| T_DOUBLE {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_DOUBLE" << Lib::endl;
  }
  $$ = Slash::Core::IValue::DOUBLE;
} 
| T_BOOLEAN {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_BOOLEAN" << Lib::endl;
  }
  $$ = Slash::Core::IValue::BOOL;
} 
| T_BYTE {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_BYTE" << Lib::endl;
  }
  $$ = Slash::Core::IValue::UNSIGNED_CHAR;
} 
| T_CHAR {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_CHAR" << Lib::endl;
  }
  $$ = Slash::Core::IValue::CHAR;
} 
| T_STRING {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_STRING" << Lib::endl;
  }
  $$ = Slash::Core::IValue::STRING;
} 
| T_JAVA_STRING {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_JAVA_STRING" << Lib::endl;
  }
  $$ = Slash::Core::IValue::STRING;
};

value 
: T_TRUE { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_TRUE" << Lib::endl;
  }
  $$ = new Lib::Value(true);
  sParser->fTmpVars.push_back($$);
} 
| T_FALSE { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : T_FALSE" << Lib::endl;
  }
  $$ = new Lib::Value(false);
  sParser->fTmpVars.push_back($$);
} 
| '-' NUMBER_INTEGER { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : NUMBER_INTEGER" << Lib::endl;
  }
  $$ = new Lib::Value(-(int)$2);
  sParser->fTmpVars.push_back($$);
} 
| NUMBER_INTEGER { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : NUMBER_INTEGER" << Lib::endl;
  }
  $$ = new Lib::Value((int)$1);
  sParser->fTmpVars.push_back($$);
} 
| NUMBER_REAL { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : NUMBER_REAL" << Lib::endl;
  }
  $$ = new Lib::Value((double)$1);
  sParser->fTmpVars.push_back($$);
} 
| '-' NUMBER_REAL { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : NUMBER_REAL" << Lib::endl;
  }
  $$ = new Lib::Value((double)(-1. * $2));
  sParser->fTmpVars.push_back($$);
} 
| STRG {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : STRG" << Lib::endl;
  }
  $$ = new Lib::Value(sParser->lexer().variable($1));
  sParser->fTmpVars.push_back($$);
} 
| T_NAME {
  // For char.
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::Columns::parse : STRG" << Lib::endl;
  }
  $$ = new Lib::Value(sParser->lexer().variable($1));
  sParser->fTmpVars.push_back($$);
};

%%

//////////////////////////////////////////////////////////////////////////////
Lib::Columns::Columns(
 const std::string& aString
,Slash::Core::IWriter& aPrinter
,int aVerboseLevel
) 
:fVerboseLevel(aVerboseLevel)
,fString(aString)
,fPrinter(aPrinter)
,fLexer(aPrinter,aVerboseLevel)
,fSuccess(false)
,fCurrentType(Slash::Core::IValue::NONE)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Columns::~Columns(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
}
//////////////////////////////////////////////////////////////////////////////
Lib::Columns& Lib::Columns::operator=(
 const Columns& aColumns
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVerboseLevel = aColumns.fVerboseLevel;
  fLexer = aColumns.fLexer;
  // Reset other things :
  fString = "";
  fSuccess = false;
  fStack.clear();
  fCurrentType = Slash::Core::IValue::NONE;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Columns::setScript(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fString = aString;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& Lib::Columns::script(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fString;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Columns::findVariables(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  if(fString=="") return false;
  fLexer.set(fString);
  sParser = this;
  fSuccess = true;
  fCurrentType = Slash::Core::IValue::NONE;
  yyparse();
  sParser = 0;
  if(!fSuccess) {
    clear();
  } else {
    if(fTmpVars.size()) {
      Lib::Out out(fPrinter);
      out << "Lib::Columns::findVariables :"
          << " stack anomaly " << (unsigned int)fTmpVars.size()
          << Lib::endl;
    }
  }
  return fSuccess;
}
//////////////////////////////////////////////////////////////////////////////
std::list<Lib::Value> Lib::Columns::result(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Lib::Value> vars;
  copyColumns(fStack,vars);
  return vars;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Columns::clear(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for(unsigned int index=0;index<fTmpVars.size();index++) 
    delete fTmpVars[index];
  fTmpVars.clear();
  deleteColumns(fStack);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Columns::isSuccess(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSuccess;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& Lib::Columns::printer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
const Lib::ColumnsLexer& Lib::Columns::lexer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fLexer;
}
//////////////////////////////////////////////////////////////////////////////
static void yyerror(
 char* aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(sParser) {
    Lib::Out out(sParser->printer());
    out << "Columns : " << aString << Lib::endl;
    sParser->fSuccess = false;
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Lib::Columns::deleteColumns(
 std::list<Lib::Value>& aVars
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Lib::Value>::iterator it;
  for(it=aVars.begin();it!=aVars.end();++it) {
    if((*it).type()==Slash::Core::IValue::VOID_STAR) {
      std::list<Lib::Value>* vars = 
        (std::list<Lib::Value>*)(*it).get_void_star();
      deleteColumns(*vars);
      delete vars;
      //(*it).set((void*)0);
    }
  }
  aVars.clear();
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Columns::copyColumns(
 const std::list<Lib::Value>& aFrom
,std::list<Lib::Value>& aTo
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Lib::Value>::const_iterator it;
  for(it=aFrom.begin();it!=aFrom.end();++it) {
    if((*it).type()==Slash::Core::IValue::VOID_STAR) {
      std::list<Lib::Value>* vars = new std::list<Lib::Value>();
      Lib::Value v((void*)vars);
      v.setLabel((*it).label());
      aTo.push_back(v);
      std::list<Lib::Value>* p = 
        (std::list<Lib::Value>*)(*it).get_void_star(); 
      copyColumns(*p,*vars);
    } else {
      aTo.push_back(*it);
    }
  }
}
