// From tcl8.0.5/win/tclWinLoad.c, tclWinDld.c.

// HP-UX :
// - application that is linked with shl_load must be 
//   linked with +s in order to be able to use SHLIB_PATH variable.
// - tk/wish must be linked with aCC in order to dynamicly load
//   some C++ library.

// Darwin (MacOSX) :
// - we use the OpenDarwin/dlcompat library to emulate dl* functions. 

// Take care of the deletion of a Lib::DLL
// (in particular if allocated on the stack) ;
// if for example a Session have references of things in the lib
// (a Manager), and that the lib is destroyed, Session, when using
// referenced things can crash.

// Take care of the C functions declared to atexit in a DLL.
// On some system (RedHat-6.x) the dlclose does not call them !
// Instead they are called at the (final) exit. Then crashes can
// happen if someone dlclose the DLL before calling exit (or returning
// from the main).
//   Crash example. OnX_onx.exe loading gtk on a RedHat-6.x.
// The gdk_exit_func, gtk_exit_func are called whilsst gdk/gtk are 
// closed !

// this :
#include <Lib/DLL.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Debug.h>

#include <Lib/System.h>
#include <Lib/smanip.h>

#include <stdlib.h>

#ifdef WIN32
#include <windows.h>
#endif

#if defined(__alpha) || defined(__linux) || defined(__sun) || (defined(__CYGWIN__) && defined(__GNUC__))
#define DLOPEN
#endif

#if defined(__APPLE__)
#define NSOPEN
#endif

#ifdef DLOPEN
#include <dlfcn.h>
#endif

#ifdef __hpux
#include <dl.h>
#endif

#ifdef NSOPEN
#include <vector>
static std::vector<std::string> nspaths(); 
static void* nsopen(const std::string&,std::string&);
static void* nssym(void*,const std::string&);
static bool nsclose(void*);
#endif

//////////////////////////////////////////////////////////////////////////////
Lib::DLL::DLL(
 const std::string& aName
,Slash::Core::IWriter& aPrinter
,int aVerboseLevel
,bool aQuiet
)
:fModule(0)
,fPrinter(aPrinter)
,fName(aName)
,fVerboseLevel(aVerboseLevel)
//////////////////////////////////////////////////////////////////////////////
// aName is the full name (path + libname).
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Debug::increment("Lib::DLL");
  if(!open()) {
    if(!aQuiet) {
      Out out(fPrinter);
      out << "Lib::DLL::DLL :" 
          << " can't load " << fName << " library." << endl;
      out << "Lib::DLL::DLL :" 
          << " error  : " << fError 
          << endl;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
Lib::DLL::~DLL(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (fModule) {
    if(fVerboseLevel) {
      Out out(fPrinter);
      out << "Lib::DLL::~DLL :" 
          << " close " << Lib::sout(fName) << "..." << endl;
    }
#ifdef DLOPEN
/*FIXME : too much problems (with atexit function, gtk , Qt.
    //Darwin : comment it anyway. It induces unneceesary warnings.
    if(::dlclose(fModule)) {
      Out out(fPrinter);
      out << "Lib::DLL::~DLL :" 
          << " error  : " << ::dlerror() << endl;
    }
*/
#endif
#ifdef NSOPEN
    nsclose(fModule);
#endif
#ifdef WIN32
    ::FreeLibrary((HMODULE)fModule);
#endif
#ifdef __hpux
    shl_unload((shl_t)fModule);
#endif
    if(fVerboseLevel) {
      Out out(fPrinter);
      out << "Lib::DLL::~DLL :" 
          << " close " << Lib::sout(fName) << " done." << endl;
    }
  }
  fModule = 0;
  Debug::decrement("Lib::DLL");
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::DLL::open(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fError = "";
  std::string dll = fName;  
  if(fVerboseLevel) {
    Out out(fPrinter);
    out << "Lib::DLL::DLL :" 
        << " open " 
        << Lib::sout(dll) << "..." << endl;
  }
#ifdef DLOPEN
 {std::string path,name,suffix;
  Lib::smanip::pathNameSuffix(fName,path,name,suffix);
  if(path=="") { //Relative path
    std::string suffix = Lib::smanip::suffix(dll);
    if(suffix=="") {
      // Only the name "Xxx" had been given, we have to
#if defined(__CYGWIN__) && defined(__GNUC__)
      // build the name "Xxx.dll" :      
      dll = fName + ".dll";
#else
      // build the name "libXxx.so" :      
      dll = "lib" + fName + ".so";
#endif
    }
  }}
  int flags = RTLD_NOW | RTLD_GLOBAL;
  fModule = ::dlopen(dll.c_str(),flags);
  const char* cerror;
  if ((cerror=dlerror())!=NULL) fError = cerror;
#endif
#ifdef NSOPEN
  fModule = nsopen(dll,fError);
  if(!fModule) {
    std::vector<std::string> paths = nspaths(); 
    paths.push_back("."); //For on-the-fly-compilation.
    unsigned int pathn = paths.size();
    for(unsigned int index=0;index<pathn;index++) {
      dll = paths[index]+"/" + fName + ".bundle";
      if(fVerboseLevel) {
        Out out(fPrinter);
        out << "Lib::DLL::DLL :" 
            << " open " 
            << Lib::sout(dll) << "..." << endl;
      }
      fModule = nsopen(dll,fError);
      if(fModule) break;
    }
  }
#endif
#ifdef WIN32
  fModule = ::LoadLibrary(dll.c_str());
#endif
#ifdef __hpux
  shl_t handle;
  //handle = shl_load(dll.c_str(),BIND_DEFERRED | DYNAMIC_PATH, 0L);
  handle = shl_load(dll.c_str(),BIND_IMMEDIATE | BIND_VERBOSE , 0L);
  fModule  = (void*)handle;
#endif
  if(!fModule) return false;
  if(fVerboseLevel) {
    Out out(fPrinter);
    out << "Lib::DLL::DLL :" 
        << " " << Lib::sout(dll) << " opened." << endl;
  }
  fDLL = dll;  
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::DLL::isOpened(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fModule?true:false;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& Lib::DLL::fileName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
Lib::Procedure Lib::DLL::findProcedure(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fModule) return 0;
  std::string name = aName;
  Procedure p = 0;
  if(fVerboseLevel) {
    Out out(fPrinter);
    out << "Lib::DLL::findProcedure :" 
        << " search symbol " 
        << Lib::sout(name) << "..." << endl;
  }
#ifdef DLOPEN
  p = (Procedure)::dlsym(fModule,(char*)name.c_str());
  const char* cerror;
  if ((cerror=dlerror())!=NULL) {
    std::string serror = cerror;
    name = "_" + aName;
    p = (Procedure)::dlsym(fModule,(char*)name.c_str());
    if ((cerror=dlerror())!=NULL) {
      Out out(fPrinter);
      out << "Lib::DLL::findProcedure :" 
          << " problem when searching " << Lib::sout(aName)
          << " in " << Lib::sout(fDLL) << "."
          << " dlerror  : " << serror << endl;
    } 
  }
#endif
#ifdef WIN32
  p = (Procedure)::GetProcAddress((HMODULE)fModule,name.c_str());
  if (!p) {
    name = "_" + aName;
    p = (Procedure)::GetProcAddress((HMODULE)fModule,name.c_str());
    if (!p) {
      Out out(fPrinter);
      out << "Lib : Can't find symbol " << Lib::sout(aName) << "." << endl;
    }
  }
#endif
#ifdef NSOPEN
  p = (Procedure)nssym(fModule,name);
  if(!p) {
    name = "_" + aName;
    if(fVerboseLevel) {
      Out out(fPrinter);
      out << "Lib::DLL::findProcedure :" 
          << " search symbol" 
          << Lib::sout(name) << "..." << endl;
    }
    p = (Procedure)nssym(fModule,name);
    if (!p) {
      Out out(fPrinter);
      out << "Lib::DLL::findProcedure :" 
          << " problem when searching " << Lib::sout(aName)
          << " in " << Lib::sout(fDLL) << "."
          << endl;
    } 
  }
#endif
#ifdef __hpux
  void* symbol;
  shl_findsym ((shl_t*)&fModule,name.c_str(),
               (short)TYPE_PROCEDURE,(void*)&symbol);
  p = (Procedure)symbol;
  if (!p) {
    Out out(fPrinter);
    out << "Lib : Can't find symbol " << Lib::sout(aName) << "." << endl;
  }
#endif
  if(p && fVerboseLevel) {
    Out out(fPrinter);
    out << "Lib::DLL::findProcedure :" 
        << " symbol " << Lib::sout(name) << " found." << endl;
  }
  return p;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::DLL::execute(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  typedef void(*F)();
  F func = (F)findProcedure(aName);
  if(!func) {
    Out out(fPrinter);
    out << "Function " << aName << " not found" << endl;
    return;
  }
  func();
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool Lib::DLL::isValid(
 const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::DLL* dll = new Lib::DLL(aName,aPrinter,0,true);
  bool status = dll->isOpened();
  delete dll;
  return status;
}


#ifdef NSOPEN
#include <Lib/smanip.h>
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> nspaths(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const char* env = ::getenv("DYLD_LIBRARY_PATH");
  if(!env) env = ::getenv("GDB_DYLD_LIBRARY_PATH");
  std::string paths = (env==NULL ? std::string("") : std::string(env));
  return Lib::smanip::words(paths,":");
}
#include <mach-o/dyld.h>
//////////////////////////////////////////////////////////////////////////////
void* nsopen(
 const std::string& aDLL
,std::string& aError
) 
//////////////////////////////////////////////////////////////////////////////
//Foro infos : Darwin> man NSModule, NSObjectFileImage
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSObjectFileImage ofi = 0;
  NSObjectFileImageReturnCode code = 
    NSCreateObjectFileImageFromFile(aDLL.c_str(),&ofi);
  if(code!=NSObjectFileImageSuccess) {
    switch(code)        {
    case NSObjectFileImageSuccess:
      break;
    case NSObjectFileImageInappropriateFile:
      aError = "inappropriate file.";
      break;
    case NSObjectFileImageFailure:
      aError = "object file setup failure.";
      break;
    case NSObjectFileImageArch:
      aError =  "no object for this architecture.";
      break;
    case NSObjectFileImageFormat:
      aError = "bad object file format.";
      break;
    case NSObjectFileImageAccess:
      aError = "can't read object file.";
      break;
    default:
      aError = "unknown error from NSCreateObjectFileImageFromFile().";
      break;
    }
    return 0;
  }
  aError = "";
  NSModule module = NSLinkModule(ofi,aDLL.c_str(),FALSE);
  NSDestroyObjectFileImage(ofi);
  return module;
}
//////////////////////////////////////////////////////////////////////////////
void* nssym(
 void* aModule
,const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSModule module = (NSModule)aModule;
  NSSymbol symbol = NSLookupSymbolInModule(module,aName.c_str());
  if(symbol==NULL) return 0;
  return NSAddressOfSymbol(symbol);
}
//////////////////////////////////////////////////////////////////////////////
bool nsclose(
 void* aModule
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSModule module = (NSModule)aModule;
  NSUnLinkModule(module,FALSE);
  return true;
}
#endif
