// this :
#include <Lib/LibraryManager.h>

#include <Slash/Core/ISession.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/DLL.h>
#include <Lib/Cast.h>
#include <Lib/Debug.h>

//////////////////////////////////////////////////////////////////////////////
Lib::Library::Library(
 Slash::Core::ISession& aSession
,const std::string& aName
,const std::string& aLibrary
,const std::string& aPackage
,bool aQuiet
)
:fSession(aSession)
,fName(aName)
,fLibrary(aLibrary)
,fPackage(aPackage)
,fDLL(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::Library");
  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "Lib::Library::Library :" 
        << " name " << Lib::sout(aName)
        << " library " << Lib::sout(aLibrary)
        << " package " << Lib::sout(aPackage)
        << Lib::endl;
  }
  if(fLibrary=="") return;
  fDLL = new Lib::DLL
    (fLibrary,fSession.printer(),fSession.verboseLevel(),aQuiet);
  if(!fDLL) return;
  if(!fDLL->isOpened()) {
    delete fDLL;
    fDLL = 0;
    return;
  }
  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "Lib::Library::Library :" 
        << " load DLL for " 
        << Lib::sout(fLibrary) << " done." << Lib::endl;
  }
  execute("Initialize");
}
//////////////////////////////////////////////////////////////////////////////
Lib::Library::~Library(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("Lib::Library");
  if(!fDLL) return;
  execute("Finalize");
  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "Lib::Library::~Library :"
        << " \"" << fName << "\" closing DLL..." << Lib::endl;
  }
  delete fDLL;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Library::isInitialized(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDLL ? true : false;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::ILibrary::Procedure Lib::Library::findProcedure(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fDLL) return 0;
  return fDLL->findProcedure(aName);
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Library::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& Lib::Library::package(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPackage;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Library::execute(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fDLL) return false;
  if(fPackage=="") return false;
  typedef void(*F)(Slash::Core::ISession&);
  std::string s = fPackage;
  s += aName;
  F func = (F)fDLL->findProcedure(s);
  if(!func) {
    Out out(fSession.printer());
    out << "Lib::Library::Library :"
        << " Function " << s << " not found" << endl;
    return false;
  }
  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "Lib::Library::Library :" 
        << " execute \"" << s << "\"..." << Lib::endl;
  }
  func(fSession);
  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "Lib::Library::Library :"
        << " execute \"" << s << "\" done." << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::LibraryManager::LibraryManager(
 Slash::Core::ISession& aSession
)
:fSession(aSession)
,fName("LibraryManager")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::LibraryManager");
}
//////////////////////////////////////////////////////////////////////////////
Lib::LibraryManager::~LibraryManager(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Delete in reverse order :
  std::list<Slash::Core::ILibrary*>::iterator it;
  while(!fList.empty()) {
    it = fList.end();
    it--;
    Slash::Core::ILibrary* entry  = *it;
    fList.erase(it);
    delete entry;
  }
  Lib::Debug::decrement("Lib::LibraryManager");
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::LibraryManager::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Lib::LibraryManager)
  else if_Lib_SCast(Slash::Core::ILibraryManager)
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::LibraryManager::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::ILibrary* Lib::LibraryManager::addLibrary(
 const std::string& aName
,const std::string& aLibrary
,const std::string& aPackage
,bool aQuiet
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="") return 0;
  Slash::Core::ILibrary* entry = findLibrary(aName);
  if(entry) return entry;
  entry = new Library(fSession,aName,aLibrary,aPackage,aQuiet);
  if(!entry->isInitialized()) {
    delete entry;
    return 0;
  }
  fList.push_back(entry);
  return entry;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::ILibrary* Lib::LibraryManager::findLibrary(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::Core::ILibrary*>::iterator it;
  for(it=fList.begin();it!=fList.end();++it) {
    if(aString==(*it)->name()) return (*it);
  }
  return 0;
}
