%{

// this :
#include <Lib/Processor.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/ProcessorLexer.h>
#include <Lib/smanip.h>
#include <Lib/sout.h>

static Lib::Processor* sParser = 0;

extern int yylex();
int yyparse();
static void yyerror(char*); 

#define NotFound (-1)

//#define DEBUG

%}

%union {
  unsigned int fIndex;
  unsigned int fInteger;
  double fNumber;
  Lib::Expression* fExpression;
}

%token <fIndex> STRG
%token <fIndex> T_NAME
%token <fInteger> NUMBER_INTEGER
%token <fNumber> NUMBER_REAL 

%token T_COS T_SIN T_TAN T_ACOS T_ASIN T_ATAN T_SQRT T_POW T_LOG T_LOG10 
%token T_EXP T_FABS T_COSH T_SINH T_TANH
%token T_GAUSS T_BW T_EXPO T_POL1 T_POL2 T_POL3 T_POL4 T_CAUCHY
%token T_TOINT T_MOD T_ARRAY T_NCO T_VSUM T_RNDM
%token T_PI T_TRUE T_FALSE
%token T_AND T_OR

%type <fExpression> comparisons comparison expression 

%left '-' '+'
%left '/' '*'
%nonassoc UMINUS

%%

result 
: error {
  // To reset lex and yacc in case of error.
} | comparisons {
  if(sParser->findNames()) {
    sParser->setResult(0);
  } else {
    sParser->setResult($1);
    sParser->stack().pop_back();
  }
} | expression {
  if(sParser->findNames()) {
    sParser->setResult(0);
  } else {
    sParser->setResult($1);
    sParser->stack().pop_back();
  }
};

comparisons 
: '!' comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : ! cps" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::NOT,$2);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparisons '&' '&' comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cps && cps" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::AND,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparisons T_AND comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cps && cps" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::AND,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparisons '|' '|' comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cps || cps" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::OR,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparisons T_OR comparisons {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cps || cps" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::OR,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | '(' comparisons ')'  {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : (cps)" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::ASSIGN,$2);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | comparison {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : cp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::ASSIGN,$1);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
};

comparison 
: expression '>' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp > exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::GT,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '>' '=' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp >= exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::GE,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '<' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp < exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::LT,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '<' '=' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp <= exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::LE,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '=' '=' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp == exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::EQ,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '!' '=' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp != exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::NE,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
};

expression 
: expression '+' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp + exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::ADD,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '-' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp - exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::SUB,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '*' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp * exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::MUL,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '*' '*' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp ** exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::POW,$1,$4);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | expression '/' expression {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : exp / exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::DIV,$1,$3);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | '-' expression %prec UMINUS { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : - exp" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::MINUS,$2);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : (exp)" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::ASSIGN,$2);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_COS '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_COS" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::COS,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_SIN '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_SIN" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::SIN,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_TAN '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_TAN" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::TAN,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_COSH '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_COSH" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::COSH,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_SINH '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_SINH" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::SINH,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_TANH '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_TANH" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::TANH,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_ACOS '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_ACOS" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::ACOS,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_ASIN '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_ASIN" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::ASIN,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_ATAN '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_ATAN" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::ATAN,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_SQRT '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_SQRT" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::SQRT,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_POW '(' expression ',' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_POW" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::POW,$3,$5);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_LOG '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_LOG" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::LOG,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_LOG10 '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_LOG10" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::LOG10,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_EXP '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_EXP" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::EXP,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_FABS '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_FABS" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::FABS,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_GAUSS '(' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_GAUSS" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::GAUSS,$3,$5,$7,$9);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_BW '(' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_BW" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::BW,$3,$5,$7,$9);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_CAUCHY '(' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_CAUCHY" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::CAUCHY,$3,$5,$7,$9);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_EXPO '(' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_EXPO" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::EXPO,$3,$5,$7);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_POL1 '(' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_POL1" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::POL1,$3,$5,$7);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_POL2 '(' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_POL2" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::POL2,$3,$5,$7,$9);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_POL3 '(' expression ',' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_POL3" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::POL3,$3,$5,$7,$9,$11);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_POL4 '(' expression ',' expression ',' expression ',' expression ',' expression ',' expression ')' {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_POL4" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::POL4,$3,$5,$7,$9,$11,$13);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_TOINT '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_TOINT" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::TOINT,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_MOD '(' expression ',' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_MOD" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::MOD,$3,$5);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_ARRAY '(' expression ',' expression ',' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_ARRAY " << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::ARRAY,$3,$5,$7);
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_NCO '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_NCO" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::NCO,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_VSUM '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_VSUM" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::VSUM,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_RNDM '(' expression ')' { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_VSUM" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::FUNC,Lib::Expression::RNDM,$3);
    sParser->stack().pop_back();
    sParser->stack().push_back($$);
  }
} | T_PI { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_PI" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::PI);
    sParser->stack().push_back($$);
  }
} | T_TRUE { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_TRUE" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::BOOL_TRUE);
    sParser->stack().push_back($$);
  }
} | T_FALSE { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_FALSE" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::BOOL_FALSE);
    sParser->stack().push_back($$);
  }
} | NUMBER_INTEGER { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : NUMBER_INTEGER" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression
      (Lib::Expression::UNSIGNED_INTEGER,(unsigned int)$1);
    sParser->stack().push_back($$);
  }
} | NUMBER_REAL { 
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : NUMBER_REAL" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::REAL,$1);
    sParser->stack().push_back($$);
  }
} | T_NAME {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : T_NAME" << Lib::endl;
#endif
  if(sParser->findNames()) {
    sParser->addName(sParser->lexer().variable($1));
    $$ = 0;
  } else {
    std::string var = sParser->lexer().variable($1);
    unsigned int index;
    if(sParser->findVariable(var,index)) {
      $$ = new Lib::Expression(Lib::Expression::NAME,var,int(index));
      sParser->stack().push_back($$);
    } else {
      //Lib::Out out(sParser->printer());
      //out << "debug : Processor.y :"
      //    << " findVariable failed for " << Lib::sout(var) << "."
      //    << Lib::endl;
      $$ = new Lib::Expression(Lib::Expression::NAME,var,NotFound);
      sParser->stack().push_back($$);
    }
  }
} | STRG {
#ifdef DEBUG
  Lib::Out out(sParser->printer());
  out << "debug : Processor.y : STRG" << Lib::endl;
#endif
  if(sParser->findNames()) {
    $$ = 0;
  } else {
    $$ = new Lib::Expression(Lib::Expression::STRING,
                             sParser->lexer().variable($1));
    sParser->stack().push_back($$);
  }
};

%%
//////////////////////////////////////////////////////////////////////////////
Lib::Processor::Processor(
 const std::string& aString
,Slash::Core::IWriter& aPrinter
) 
:fString(aString)
,fPrinter(aPrinter)
,fFindNames(false)
,fLexer(aPrinter)
,fResult(0)
,fSuccess(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Processor::~Processor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
}
//////////////////////////////////////////////////////////////////////////////
Lib::Processor& Lib::Processor::operator=(
 const Processor& aProcessor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  fLexer = aProcessor.fLexer;
  // Reset other things :
  fString = "";
  fFindNames = false;
  fResult = 0;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Processor::setScript(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  fString = aString;
  fFindNames = false;
  fResult = 0;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Processor::clear(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for(unsigned int index=0;index<fTmpExps.size();index++) 
    delete fTmpExps[index];
  fTmpExps.clear();
  fNames.clear();
  delete fResult;
  fResult = 0;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Processor::findVariables(
 std::vector<std::string>& aNames
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aNames.clear();
  clear();
  if(!fString.size()) return true;
  fLexer.set(fString);
  fFindNames = true;
  sParser = this;
  fSuccess = true;
  yyparse();
  sParser = 0;
  if(!fSuccess) {
    clear();
    return false;
  }
  if(fTmpExps.size()) {
    Lib::Out out(fPrinter);
    out << "Lib::Processor::findVariables :"
        << " stack anomaly " << (unsigned int)fTmpExps.size()
        << Lib::endl;
  }
  smanip::unique(fNames);
  aNames = fNames;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Processor::compile(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> vars;
  findVariables(vars);
  return compile(vars);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Processor::compile(
 const std::vector<std::string>& aNames
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  if(!fString.size()) return true;
  fNames = aNames;
  fLexer.set(fString);
  fFindNames = false;
  sParser = this;
  fSuccess = true;
  yyparse();
  sParser = 0;
  if(!fSuccess) {
    clear();
    return false;
  }
  if(fTmpExps.size()) {
    Lib::Out out(fPrinter);
    out << "Lib::Processor::compile :"
        << " stack anomaly " << (unsigned int)fTmpExps.size()
        << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Processor::execute(
 const std::vector<Lib::Value>& aInput
,Lib::Value& aResult
,std::string& aError
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : Lib::Processor::execute : \"%s\" %lu\n",
  //  fString.c_str(),fResult);
  if(!fString.size()) {
    aResult.setNone();
    return true;
  }
  if(!fResult) {
    aResult.setNone();
    aError = "Lib::Processor::execute : not ready : fReturn = 0.";
    return false;
  }
  if(!fResult->execute(aInput,aError)) {
    aResult.setNone();
    return false;
  }
  if(!fSuccess) {
    aResult.setNone();
    aError = "Lib::Processor::execute : failed.";
    return false;
  }
  aResult = fResult->ret();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Processor::cut(
 const std::vector<Lib::Value>& aInput
,bool& aResult
,std::string& aError
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Value ret; //WARNING : this allocation can take time.
  if(!execute(aInput,ret,aError)) {
    aResult = true;
    return false;
  }
  if(ret.type()==Slash::Core::IValue::BOOL) {
    aResult = ret.get_bool();
    return true;
  }
  if(ret.type()==Slash::Core::IValue::NONE) { 
    //This may happen if fString is empty.
    aResult = true;
    return true;
  }
  aResult = true;
  aError = "Lib::Processor::cut : failed.";
  return false;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Processor::addName(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fNames.push_back(aString);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& Lib::Processor::printer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Processor::findNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFindNames;
}
//////////////////////////////////////////////////////////////////////////////
const Lib::ProcessorLexer& Lib::Processor::lexer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fLexer;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Processor::findVariable(
 const std::string& aString
,unsigned int& aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!smanip::lineIndex(fNames,aString,aIndex)) {
    Lib::Out out(fPrinter);
    out << " " << aString << " not a variable." << Lib::endl;
    return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<Lib::Expression*>& Lib::Processor::stack(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTmpExps;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Processor::setResult(
 Lib::Expression* aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fResult = aValue;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Processor::setStatus(
 bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSuccess = aValue;
}
//////////////////////////////////////////////////////////////////////////////
static void yyerror(
 char* aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(sParser) {
    Lib::Out out(sParser->printer());
    out << "Processor : " << aString << Lib::endl;
    sParser->setStatus(false);
  }
}
