// this :
#include <Lib/RelationManager.h>

#include <Slash/Core/IMessage.h>
#include <Slash/Core/IRelationVisitor.h>

#include <Lib/Cast.h>
#include <Lib/Debug.h>

//////////////////////////////////////////////////////////////////////////////
Lib::RelationManager::RelationManager(
 Slash::Core::ISession& aSession
)
:fSession(aSession)
,fName("Lib::RelationManager")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::RelationManager");
}
//////////////////////////////////////////////////////////////////////////////
Lib::RelationManager::~RelationManager(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::Core::IRelation*>::iterator it;
  for(it=fList.begin();it!=fList.end();it = fList.erase(it)) delete *it;
  Lib::Debug::decrement("Lib::RelationManager");
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::RelationManager::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Slash::Core::IRelationManager)
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::RelationManager::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::RelationManager::add(
 Slash::Core::IRelation* aRelation
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fList.push_back(aRelation);
}
//////////////////////////////////////////////////////////////////////////////
void Lib::RelationManager::visit(
 Slash::Core::IRelationVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::Core::IRelation*>::iterator it;
  for(it=fList.begin();it!=fList.end();++it) {
    if(!aVisitor.visit(*(*it))) return;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::RelationManager::isValid(
 const Slash::Core::IRelation& aRelation
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::Core::IRelation*>::const_iterator it;
  for(it=fList.begin();it!=fList.end();++it) {
    if((*it)==&(aRelation)) return true;
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::RelationManager::remove(
 Slash::Core::IRelationVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::Core::IRelation*>::iterator it;
  for(it=fList.begin();it!=fList.end();) {
    Slash::Core::IRelation* relation = *it;
    if(aVisitor.visit(*relation)) {
      it = fList.erase(it);
      delete relation;
    } else {
      ++it;
    }
  }
}
class RelationVisitor : public virtual Slash::Core::IRelationVisitor {
public: //IRelationVisitor
  virtual bool visit(Slash::Core::IRelation& aRelation) {
    if(fObject==aRelation.first()) return true; //remove
    if(fObject==aRelation.second()) return true; //remove
    return false;
  }
public:
  RelationVisitor(void* aObject):fObject(aObject){}
  virtual ~RelationVisitor(){}
private:
  void* fObject;
};
//////////////////////////////////////////////////////////////////////////////
void Lib::RelationManager::removeWith(
 void* aObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  RelationVisitor visitor(aObject);
  remove(visitor);
}
