// this :
#include <Lib/System.h>

#include <Lib/cstring.h>

#include <stdlib.h>

#ifdef __alpha
  extern "C" { extern int putenv(char *);}
#endif

//////////////////////////////////////////////////////////////////////////////
bool Lib::System::kind(
 std::string& aKind
)
//////////////////////////////////////////////////////////////////////////////
//  System kind could compelled by the
// env variable LIBSYSKIND.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(getenv("LIBSYSKIND",aKind)) {
    return true;
  } else {
#if defined(WIN32)
    aKind = "Windows";
    return true;
#elif defined(VMS)
    aKind = "VMS";
    return true;
#else
    aKind = "UNIX";
    return true;
#endif
  }
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::isKindOf(
 const std::string& aString   
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  if(!kind(s)) return false;
  return (s==aString);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::isWindows_NT(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isKindOf("Windows");
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::isUNIX(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isKindOf("UNIX");
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::isVMS(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isKindOf("VMS");
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::type(
 std::string& aType
)
//////////////////////////////////////////////////////////////////////////////
//  System type could compelled by the
// env variable LIBSYSTYPE.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(getenv("LIBSYSTYPE",aType)) {
    return true;
  } else {
#if defined(WIN32)
    return getenv("OS",aType);
#else
    return getenv("OSTYPE",aType);
#endif
  }
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::isTypeOf(
 const std::string& aString   
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  if(!type(s)) return false;
  return (s==aString);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::isenv(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const char* env = ::getenv(aString.c_str());
  return (env?true:false);
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::System::getenv(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const char* env = ::getenv(aString.c_str());
  return std::string(env?env:"");
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::getenv(
 const std::string& aString
,std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const char* env = ::getenv(aString.c_str());
  if(env) {
    aValue = std::string(env?env:"");
    return true;
  } else {
    aValue = "";
    return false;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::putenv(
 const std::string& aString
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string value = aString+"="+aValue;
  if(::putenv(Lib_strdup(value.c_str()))) return false;
  //check:
  std::string s;
  if(!getenv(aString,s)) return false;
  if(s!=aValue) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::envPathAppend(
 const std::string& aString
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return envAppend(aString,aValue,pathSeparator());
}
//////////////////////////////////////////////////////////////////////////////
int Lib::System::execute(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return ::system(aString.c_str());
}
//////////////////////////////////////////////////////////////////////////////
void Lib::System::exit(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  ::exit(EXIT_SUCCESS);
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::System::pathSeparator(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  return ";";
#else
  return ":";
#endif
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::System::fileSeparator(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  return "\\";
#else
  return "/";
#endif
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::System::quote(
 const std::string& aDirectory
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aDirectory.find(' ')==std::string::npos) return aDirectory;
  // Directory with spaces :
  if(aDirectory[0]=='"') return aDirectory; //Already in double quote.
  return "\""+aDirectory+"\"";
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::System::fortranUnitFormat(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef __hpux
  return std::string("ftn%d");
#else
  return std::string("fort.%d");
#endif
}
//////////////////////////////////////////////////////////////////////////////
int Lib::System::exitFailure(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return EXIT_FAILURE;
}
//////////////////////////////////////////////////////////////////////////////
int Lib::System::exitSuccess(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return EXIT_SUCCESS;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::System::dllPrefix(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#if defined(WIN32)
  return "";
#elif defined(__APPLE__)
  return "";
#elif defined(__CYGWIN__) && defined(__GNUC__)
  return "";
#else //UNIX
  return "lib";
#endif
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::System::dllSuffix(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#if defined(WIN32)
  return "dll";
#elif defined(__APPLE__)
  return "bundle";
#elif defined(__CYGWIN__) && defined(__GNUC__)
  return "dll";
#else //UNIX
  return "so";
#endif
}

/*
#ifdef WIN32
//#include <process.h>
//#include <windows.h>
//#include <Win32PsApi.h>
//static PsApiFunctions _psApi;
#else
#include <stdio.h>
#include <unistd.h>
#endif
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::exeName(
 std::string& aName
)    
//////////////////////////////////////////////////////////////////////////////
// From GaudiKernel.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  if ( _psApi.isValid() && processHandle() )   {
    char name[512];
    name[0] = 0;
    _psApi.GetModuleFileNameExA(processHandle(),
                                (HINSTANCE)exeHandle(), 
                                name,sizeof(name));
    aName = name;
    return true;
  } else {
    aName = "";
    return false;
  }
#elif defined(__linux)
  char name[512];
  name[0] = 0;
  char cmd[512];
  ::sprintf(cmd, "/proc/%d/exe", ::getpid());
  ::readlink(cmd, name, sizeof(name));
  aName = name;
  return true;
//#elif __hpux
//    ::realpath(((HMODULE*)exeHandle())->dsc.filename, name);
#elif defined(__APPLE__)
  aName = "";
  return false;
#else
  aName = "";
  return false;
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::args(
 std::vector<std::string>& aArgs
)    
//////////////////////////////////////////////////////////////////////////////
// Command line arguments including executable name as arg[0]
// From GaudiKernel.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  aArgs.clear();
  char exe[1024];
  // For compatibility with UNIX we CANNOT use strtok!
  // If we would use strtok, options like -g="My world" at
  // the command line level would result on NT in TWO options 
  // instead in one as in UNIX.
  char *next, *tmp1, *tmp2;
  for(LPTSTR cmd = ::GetCommandLine(); *cmd; cmd=next)   {
    ::memset(exe,0,sizeof(exe));
    while ( *cmd == ' ' ) cmd++;
    next = ::strchr(cmd,' ');
    if ( !next ) next = cmd + strlen(cmd);
    if ( (tmp1=::strchr(cmd,'\"')) > 0 && tmp1 < next )  {
      tmp2 = ::strchr(++tmp1,'\"');
      if ( tmp2 > 0 )   {
        next = ++tmp2;
        if ( cmd < tmp1 ) strncpy(exe, cmd, tmp1-cmd-1);
        strncpy(&exe[strlen(exe)], tmp1, tmp2-tmp1-1);
      }
      else    {
        std::cout << "Mismatched \" in command line arguments" << std::endl;
        s_argvChars.erase(s_argvChars.begin(), s_argvChars.end());
        s_argvStrings.erase(s_argvStrings.begin(), s_argvStrings.end());
        return s_argvStrings;
      }
    }
    else    {
      strncpy(exe, cmd, next-cmd);
    }
    s_argvStrings.push_back(exe);
    s_argvChars.push_back( s_argvStrings.back().c_str());
  }
  return true;
#elif defined(__linux)
  aArgs.clear();
  std::string sexe;
  if(!exeName(sexe)) return false;
  char exe[1024];
  ::sprintf(exe, "/proc/%d/cmdline", ::getpid());
  FILE* file = ::fopen(exe,"r");
  if(!file) {
    aArgs.clear();
    return false;
  }
  char cmd[1024];
  size_t len = ::fread(cmd, sizeof(char), sizeof(cmd), file);
  if ( len > 0 )   {
    cmd[len] = 0;
    for ( char* token = cmd; token-cmd < len; token += strlen(token)+1 )  {
      aArgs.push_back(token);
    }
    aArgs[0] = sexe; //override the first argument to have full path name.
  }
  ::fclose(file);
  return true;
#elif defined(__APPLE__)
  aArgs.clear();
  return false;
#else
  aArgs.clear();
  return false;
#endif
}
*/
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// embed Lib::smanip::words to avoid #include <Lib/smanip.h>
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include <vector>
//////////////////////////////////////////////////////////////////////////////
static std::vector<std::string> Lib_smanip_words(
 const std::string& aString
,const std::string& aLimiter 
,bool aTakeEmpty = false
)
//////////////////////////////////////////////////////////////////////////////
//  If aLimiter is for exa "|" and for "xxx||xxx" :
//  - aTakeEmpty false : {"xxx","xxx"} will be created 
//    (and NOT {"xxx","","xxx"}).
//  - aTakeEmpty true : {"xxx","","xxx"} will be created.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> words;
  if(aString=="") return words;
  std::string::size_type lim = (aTakeEmpty?0:1);
  if(aLimiter=="") {
    words.push_back(aString);
  } else {
    std::string::size_type l = aString.length();
    std::string::size_type llimiter = aLimiter.length();
    std::string::size_type pos = 0;
    while(1) {
      std::string::size_type index = aString.find(aLimiter,pos);
      if(index==std::string::npos){ // Last word.
        if((l-pos)>=lim) words.push_back(aString.substr(pos,l-pos));
        break;
      } else {
        //     abcxxxef
        //     0  3  67
        if((index-pos)>=lim) words.push_back(aString.substr(pos,index-pos));
        pos = index + llimiter;
      }
    }
  }
  return words;
}
//////////////////////////////////////////////////////////////////////////////
static bool Lib_smanip_remove(
 std::vector<std::string>& aStrings
,const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool found_some = false;
  std::vector<std::string>::iterator it;
  for(it=aStrings.begin();it!=aStrings.end();) {
    if(*it==aString) {
      it = aStrings.erase(it);
      found_some = true;
    } else {
      ++it;
    }
  }
  return found_some;
}
//////////////////////////////////////////////////////////////////////////////
static std::string Lib_smanip_tostring(
 const std::vector<std::string>& aStrings
,const std::string& aSeparator
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = aStrings.size();
  if(number<=0) return "";
  std::string result;
  number--;
  for(unsigned int index=0;index<number;index++) {
    result += aStrings[index];
    result += aSeparator;
  }
  result += aStrings[number];
  return result;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::System::envAppend(
 const std::string& aString
,const std::string& aValue
,const std::string& aSeparator
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string old_value;
  if(isenv(aString)) {
    if(!getenv(aString,old_value)) return false;
  }

  if(old_value!="") {
    std::vector<std::string> words = Lib_smanip_words(old_value,aSeparator);
    // Remove existing one, so that aValue be put at end.
    Lib_smanip_remove(words,aValue);
    old_value = Lib_smanip_tostring(words,aSeparator);
  }

  std::string new_value = old_value;
  if(new_value!="") new_value += aSeparator;
  new_value += aValue;
  if(!putenv(aString,new_value)) return false;

/*
  //Check
 {std::string new_value;
  getenv(aString,new_value);
  printf("debug : Lib::System::envAppend : env : \"%s\"\n",aString.c_str());
  printf("debug :   old value : \"%s\"\n",old_value.c_str());
  printf("debug :   new value : \"%s\"\n",new_value.c_str());}
*/

  return true;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

#ifdef __alpha
#include <sys/time.h>
#endif
//////////////////////////////////////////////////////////////////////////////
void Lib::System::sleep(
 unsigned int aMilliSec
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  // Sleep milliSec milli seconds.
  // The Sleep function suspends the execution of the CURRENT THREAD for
  // a specified interval.
  ::Sleep(aMilliSec);
#else
#if (defined(__CYGWIN__) && defined(__GNUC__))
  //FIXME
#else
  // Sleep milliSec milliseconds.
  struct timeval tv;
  tv.tv_sec  = aMilliSec / 1000;
  tv.tv_usec = (aMilliSec % 1000) * 1000;
  ::select(0, 0, 0, 0, &tv);
#endif
#endif
}
