// this :
#include <Lib/UnixSocket.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/Debug.h>

#ifdef WIN32
#else
#include<sys/socket.h>
#include<sys/un.h>
#include <errno.h>
#include <unistd.h>  //close,sleep
#include <string.h> //strerror
#ifdef __alpha
#define CLIENT_ADDR_LENGTH int
#else
#define CLIENT_ADDR_LENGTH socklen_t
#endif
#endif

const int MAX_CONNECT_TRIAL = 10;

//////////////////////////////////////////////////////////////////////////////
Lib::UnixSocket::UnixSocket(
 Slash::Core::IWriter& aPrinter
,int aVerboseLevel
)
:BaseSocket(aPrinter,aVerboseLevel)
,fSUN_PATH("LIB_TMP3")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Debug::increment("Lib::UnixSocket");
}
//////////////////////////////////////////////////////////////////////////////
Lib::UnixSocket::~UnixSocket(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
#else
  ::unlink(fSUN_PATH.c_str()) ;
#endif
  Debug::decrement("Lib::UnixSocket");
}
//////////////////////////////////////////////////////////////////////////////
void Lib::UnixSocket::setSunPath( 
 const std::string& sun_path 
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{  
  fSUN_PATH = sun_path;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::UnixSocket::accept(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Out out(fPrinter);

#ifdef WIN32 
  out << "Lib::UnixSocket::accept :"
      << " unavailable on WIN32."
      << Lib::endl;
  return false;
#else
  if(fSocket!=(-1)) return true; //done.

  // clear PIPE
  ::unlink(fSUN_PATH.c_str());

  // make socket
  int fd_socket_waiting = ::socket(AF_UNIX,SOCK_STREAM,0);
  if( fd_socket_waiting < 0 ) { 
    out << "Lib::UnixSocket::accept :"
        << " socket : " << std::string(strerror(errno))
        << Lib::endl;
    return false;
  }

  // binding
  sockaddr_un server_address ;
  ::memset((char*)&server_address,'\0',sizeof(server_address)) ; 

  // clear server_address
  server_address.sun_family = AF_UNIX ;
  ::strcpy(server_address.sun_path,fSUN_PATH.c_str());
  if(::bind(fd_socket_waiting,
            (struct sockaddr*)(&server_address),sizeof(server_address))<0) {
    out << "Lib::UnixSocket::accept :"
        << " bind : " << std::string(strerror(errno))
        << Lib::endl;
    return false;
  }

  // listen (wait connection)
  if(::listen(fd_socket_waiting,1) == -1 ) { 
    out << "Lib::UnixSocket::accept :"
        << " listen : " << std::string(strerror(errno))
        << Lib::endl;
    return false;
  }

  // accept connection
  sockaddr_un client_address ;
  CLIENT_ADDR_LENGTH client_addr_length = sizeof(client_address);
  fSocket = ::accept(fd_socket_waiting, 
                     (struct sockaddr*)(&client_address), 
                     &client_addr_length ) ;
  ::close( fd_socket_waiting );
  if(fSocket<0) { 
    out << "Lib::UnixSocket::accept :"
        << " accept : " << std::string(strerror(errno))
        << Lib::endl;
    fSocket = -1;
    return false;
  }
  return true;
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::UnixSocket::connect(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Out out(fPrinter);

#ifdef WIN32 
  out << "Lib::UnixSocket::connect :"
      << " unavailable on WIN32."
      << Lib::endl;
  return false;
#else
  if(fSocket!=(-1)) return true; //Done.

  // make socket
  fSocket = ::socket( AF_UNIX, SOCK_STREAM, 0 );
  if(fSocket<0) { 
    out << "Lib::UnixSocket::connect :"
        << " socket : " << std::string(strerror(errno))
        << Lib::endl;
    fSocket = -1;
    return false;
  }

  // set server address
  struct sockaddr_un server_address ;
  ::memset((char*)&server_address,'\0',sizeof(server_address)) ;
  server_address.sun_family = AF_UNIX ;
  ::strcpy(server_address.sun_path,fSUN_PATH.c_str());

  // connection
  int connection_status = -1;
  int num_trial = 0;
  while( connection_status < 0 && num_trial <= MAX_CONNECT_TRIAL ) {
    num_trial++ ; 
    connection_status = ::connect
      (fSocket,(struct sockaddr*)(&server_address),sizeof(server_address)) ;
    if( connection_status  <0 ) {
      if(fVerboseLevel) {
        out << "Lib::UnixSocket::connect :"
            << " connect : " << std::string(strerror(errno))
            << " : retry..."
            << Lib::endl;
      }
    } else {
      return true;
    }
    ::sleep(1);                
  }

  out << "Lib::UnixSocket::connect :"
      << " can't connect."
      << Lib::endl;

  return false;
#endif
}
//////////////////////////////////////////////////////////////////////////////
void Lib::UnixSocket::disconnect(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  closeSocket();
#ifdef WIN32
#else
  ::unlink(fSUN_PATH.c_str()) ;
#endif
}
