// this :
#include <Lib/argZero.h>

#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/System.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/sout.h>
#include <Lib/dirmanip.h>

//////////////////////////////////////////////////////////////////////////////
bool Lib::argZero::checkEnv(
 const std::string& aArg0
,const std::string& aEnv
,const std::string& aTo
,Slash::Core::IWriter& aPrinter
,bool aWarn
,std::string& a_dir_root
,bool& a_is_Mac_app
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  a_dir_root = "";
  a_is_Mac_app = false;

  if(Lib::System::isenv(aEnv)) return true; //Done by "source setup"

  std::string dir_root;
  if(!Lib::dirmanip::binaryPath(aArg0,dir_root)) {
    if(aWarn) {
      Lib::Out out(aPrinter);
      out << "Lib::argZero::checkEnv :"
          << " " << aEnv << " environment variable not found and" 
          << " we can't guess where the binary is from \"arg0\"." 
          << Lib::endl;
      out << " The \"arg0\" was " << Lib::sout(aArg0) << "."
          << Lib::endl;
    }
    return false;
  }

  std::string bname = Lib::smanip::basename(dir_root);
  bool is_release_area = ((bname=="bin")||(bname=="MacOS"));

  if(!is_release_area) {
    if(aWarn) {
      Lib::Out out(aPrinter);
      out << "Lib::argZero::checkEnv :"
          << " " << aEnv << " environment variable not found and" 
          << " the binary is in an unknown place." 
          << Lib::endl;
      out << " The \"dir_root\" was " << Lib::sout(dir_root) << "."
          << Lib::endl;
      out << " The \"arg0\" was " << Lib::sout(aArg0) << "."
          << Lib::endl;
    }
    return false;
  }
  
  std::string fs = Lib::System::fileSeparator();

  dir_root = Lib::smanip::dirname(dir_root);

  std::string to = dir_root+fs+aTo;

 {std::string p,n,s;
  Lib::smanip::pathNameSuffix(aTo,p,n,s);
  if(n.find('*')!=std::string::npos) { //wildcard in ending file name.
    bool found = false;
    std::vector<std::string> files;
    std::string dir = dir_root+fs+p;
    if(Lib::dirmanip::entries(dir,files,false)) {
      unsigned int number = files.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& file = files[index];
        if(Lib::smanip::match(file,n)) { //Take first matching entry.
          to = dir+fs+file;
          found = true;
          break;
        }
      } 
    } 
    if(!found) {
      Lib::Out out(aPrinter);
      out << "Lib::argZero::checkEnv :"
          << " no file found with pattern " << Lib::sout(aTo) 
          << " under the directory " << Lib::sout(dir_root) << "."
          << Lib::endl;
      out << " The \"arg0\" was " << Lib::sout(aArg0) << "."
          << Lib::endl;
      return false;
    }
  }}

  if(!Lib::dirmanip::exists(to)) {
    if(aWarn) {
      Lib::Out out(aPrinter);
      out << "Lib::argZero::checkEnv :"
          << " try to set " << aEnv << " environment variable" 
          << " to a not existing file or directory " << Lib::sout(to) 
          << Lib::endl;
      out << " The \"dir_root\" was " << Lib::sout(dir_root) << "."
          << Lib::endl;
      out << " The \"arg0\" was " << Lib::sout(aArg0) << "."
          << Lib::endl;
    }
    return false;
  }

  if(!Lib::System::putenv(aEnv,to)) return false;

  a_dir_root = dir_root;
  a_is_Mac_app = (bname=="MacOS");

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::argZero::checkPackEnv(
 const std::string& aArg0
,const std::string& aPack
,Slash::Core::IWriter& aPrinter
,bool aWarn
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string env;
  env = aPack+"ROOT";
  Lib::smanip::touppercase(env);

  std::string fs = Lib::System::fileSeparator();
  std::string to = "Resources"+fs+aPack;

  std::string dir_root;
  bool is_Mac_app;
  return checkEnv(aArg0,env,to,aPrinter,aWarn,dir_root,is_Mac_app);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::argZero::isMacApp(
 const std::string& aArg0
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string dir_root;
  if(!Lib::dirmanip::binaryPath(aArg0,dir_root)) return false;
  std::string bname = Lib::smanip::basename(dir_root);
  return (bname=="MacOS");
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::argZero::startMacApp(
 const std::string& aArg0
,const std::string& aPack
,Slash::Core::IWriter& aPrinter
,bool& a_is_Mac_app
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  a_is_Mac_app = false;
  std::string dir_root;
  Lib::Out out(aPrinter);
  if(!Lib::dirmanip::binaryPath(aArg0,dir_root)) {
    out << "Lib::argZero::checkEnv :"
        << " Can't get a root directory." 
        << Lib::endl;
    return false;
  }
  std::string bname = Lib::smanip::basename(dir_root);
  a_is_Mac_app = (bname=="MacOS");
  if(a_is_Mac_app) {
    // Avoid polluting the / directory :
    if(!Lib::dirmanip::goHome()) {
      out << "Lib::argZero::startMacApp : can't go home." << Lib::endl;
      return false;
    } else if(!Lib::dirmanip::mkcd("OpenScientist")) {
      out << "Lib::argZero::startMacApp : can't mkcd OpenScientist." << Lib::endl;
      return false;
    } else if(!Lib::dirmanip::mkcd(aPack)) {
      out << "Lib::argZero::startMacApp :" 
          << " can't mkcd " << aPack << " ." << Lib::endl;
      return false;
    } else if(!Lib::dirmanip::mkcd("session")) {
      out << "Lib::argZero::startMacApp : can't mkcd session." << Lib::endl;
      return false;
    } else {
      // mkcds ok.
    }
  }
  return true;
}


#include <Lib/Printer.h>

//////////////////////////////////////////////////////////////////////////////
bool Lib::argZero::checkPackEnv(
 const std::string& aArg0
,const std::string& aPack
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  return checkPackEnv(aArg0,aPack,printer);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::argZero::checkEnv(
 const std::string& aArg0
,const std::string& aEnv
,const std::string& aTo
,Slash::Core::IWriter& aPrinter
,bool aWarn
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string dir_root;
  bool is_Mac_app;
  return checkEnv(aArg0,aEnv,aTo,aPrinter,aWarn,dir_root,is_Mac_app);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::argZero::startMacApp(
 const std::string& aArg0
,const std::string& aPack
,bool& a_is_Mac_app
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  return startMacApp(aArg0,aPack,printer,a_is_Mac_app);
}
