
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Processor.h>
#include <Lib/Debug.h>
#include <Lib/smanip.h>
#include <Lib/Time.h>
#include <Lib/valuemanip.h>

#include <math.h>

int main(int,char**) {

  Lib::Printer printer;
  Lib::Out out(printer);

 {Lib::Processor proc
    ("((x>0.1)&&(y<=2))&&((name==\"Hel*o\")&&(z!=2))",printer);
  std::vector<std::string> vars;
  proc.findVariables(vars);
  Lib::smanip::write(vars,printer);
  proc.compile(vars);
  std::vector<Lib::Value> vs;
  vs.push_back(Lib::Value(1.));
  vs.push_back(Lib::Value(0.));
  vs.push_back(Lib::Value("Hello"));
  vs.push_back(Lib::Value(3));
  std::string serror;
  bool res;
  proc.cut(vs,res,serror);
  out << "result : " << res << " (true expected)" << Lib::endl;
 }

 {Lib::Processor proc("((layer>=3)&&(layer<=6))&&(E<0)",printer);
  std::vector<std::string> vars;
  proc.findVariables(vars);
  Lib::smanip::write(vars,printer);
  proc.compile(vars);
  std::vector<Lib::Value> vs;
  vs.push_back(Lib::Value(1));
  vs.push_back(Lib::Value(0.1));
  std::string serror;
  bool res;
  proc.cut(vs,res,serror);
  out << "result : " << res << " (false expected)" << Lib::endl;
  vs.clear();
  vs.push_back(Lib::Value(4));
  vs.push_back(Lib::Value(-0.1));
  proc.cut(vs,res,serror);
  out << "result : " << res << " (true expected)" << Lib::endl;}

 {Lib::Processor proc("((layer>=3)&&(layer<=6))&&(E>=0)",printer);
  std::vector<std::string> vars;
  proc.findVariables(vars);
  Lib::smanip::write(vars,printer);
  proc.compile(vars);
  std::vector<Lib::Value> vs;
  vs.push_back(Lib::Value(1));
  vs.push_back(Lib::Value(0.1));
  std::string serror;
  bool res;
  proc.cut(vs,res,serror);
  out << "result : " << res << " (false expected)" << Lib::endl;
  vs.clear();
  vs.push_back(Lib::Value(4));
  vs.push_back(Lib::Value(-0.1));
  proc.cut(vs,res,serror);
  out << "result : " << res << " (false expected)" << Lib::endl;}

 {Lib::Processor proc("!(layer>=3)",printer);
  std::vector<std::string> vars;
  proc.findVariables(vars);
  Lib::smanip::write(vars,printer);
  proc.compile(vars);
  std::vector<Lib::Value> vs;
  vs.push_back(Lib::Value(1));
  std::string serror;
  bool res;
  proc.cut(vs,res,serror);
  out << "result : (not) " << res << " (true expected)" << Lib::endl;}

 {Lib::Processor proc("!((layer>=3)&&!(layer<=6))&&(E>=0)",printer);
  std::vector<std::string> vars;
  proc.findVariables(vars);
  Lib::smanip::write(vars,printer);
  proc.compile(vars);
  std::vector<Lib::Value> vs;
  vs.push_back(Lib::Value(1));
  vs.push_back(Lib::Value(0.1));
  std::string serror;
  bool res;
  proc.cut(vs,res,serror);
  out << "result : (not) " << res << " (true expected)" << Lib::endl;}

 //  gauss(x,H,M,W)    :  H * exp(-0.5 * ((x - M)/W)**2 );
 //  pol2(x,A,B,C)     : A + B x + C x**2 
 {Lib::Processor proc("gauss(x,H,M,W)+pol2(x,A,B,C)",printer);
  std::vector<std::string> vars;
  proc.findVariables(vars);
  Lib::smanip::write(vars,printer);
  proc.compile(vars);
  std::vector<Lib::Value> vs;
  vs.push_back(Lib::Value(1.)); //x
  vs.push_back(Lib::Value(1.)); //H
  vs.push_back(Lib::Value(0.)); //M
  vs.push_back(Lib::Value(2.)); //W
  vs.push_back(Lib::Value(1.)); //A
  vs.push_back(Lib::Value(2.)); //B
  vs.push_back(Lib::Value(3.)); //C
  Lib::Value res;
  std::string serror;
  proc.execute(vs,res,serror);
  double d;
  Lib::valuemanip::to_double(res,d);

  out << "result : " <<  d << " (6.8825 expected)" << Lib::endl;}

 {Lib::Processor proc("cos(M_PI)",printer);
  std::vector<std::string> vars;
  proc.findVariables(vars);
  Lib::smanip::write(vars,printer);
  proc.compile(vars);
  std::vector<Lib::Value> vs;
  Lib::Value res;
  std::string serror;
  proc.execute(vs,res,serror);
  double d;
  Lib::valuemanip::to_double(res,d);
  out << "result : " << d << " (-1 expected)" << Lib::endl;}

 {Lib::Time begin = Lib::Time::now();
  unsigned rown = 77622;
  double e = 1;
  double vx = 1;
  double vy = 1;
  double tot = 0;
  out << " begin..." << Lib::endl;
  for(unsigned int row=0;row<rown;row++) {
    tot += log10(fabs(e)*sqrt(vx*vx+vy*vy));
    //tot += e*vx*vy;
  }
  Lib::Time end = Lib::Time::elapsed(begin);
  out << " elapsed : " << end.seconds() << " s " 
      << tot << " (11683.3 expected)" << Lib::endl;}

  out << " sizeof(Variable) = " << (unsigned int)sizeof(Lib::Value) 
      << Lib::endl;
 {Lib::Time begin = Lib::Time::now();
  Lib::Processor proc("log10(abs(e)*sqrt(vx**2+vy**2))",printer);
  //Lib::Processor proc("sqrt(vx**2+vy**2)",printer);
  //Lib::Processor proc("log10(e)*vx*vy",printer);
  //Lib::Processor proc("log10(abs(e))",printer);
  std::vector<std::string> vars;
  proc.findVariables(vars);
  Lib::smanip::write(vars,printer);
  proc.compile(vars);
  unsigned int number = vars.size();
  std::vector<Lib::Value> vals;
  vals.resize(number,Lib::Value(1.));
  unsigned rown = 77622;
  double tot = 0;
  Lib::Value res;
  double d;
  out << " begin..." << Lib::endl;
  for(unsigned int row=0;row<rown;row++) {
    //for(unsigned int count=0;count<number;count++) {
    //  vals[count].set(3.14);
    //}
    std::string serror;
    proc.execute(vals,res,serror);
    Lib::valuemanip::to_double(res,d);
    tot += d;
  }
  Lib::Time end = Lib::Time::elapsed(begin);
  out << " elapsed : " << end.seconds() << " s "
      << tot << " (11683.3 expected)" << Lib::endl;}

 Lib::Debug::balance(printer);
 
 return 0;
}
