// this :
#include <Lib/Upgrader.h>

#include <Lib/fmanip.h>
#include <Lib/smanip.h>
#include <Lib/dirmanip.h>
#include <Lib/Out.h>
#include <Lib/sout.h>

//////////////////////////////////////////////////////////////////////////////
Lib::Upgrader::Upgrader(
 Slash::Core::IWriter& aPrinter
,int aVerboseLevel
,const std::string& aDataFile
)
:fPrinter(aPrinter)
,fVerboseLevel(aVerboseLevel)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fVerboseLevel) {
    Lib::Out out(fPrinter);
    out << "Lib::Upgrader::Upgrader :"
        << " upgrade file is " << Lib::sout(aDataFile) 
        << Lib::endl;
  }
  if(!fmanip::read(aDataFile,fOrders)) {
    Lib::Out out(fPrinter);
    out << "Lib::Upgrader::Upgrader :"
        << " can't read upgrade file." 
        << Lib::endl;
  }

  // include files :
  bool hasIncs = true;
  while(hasIncs) {
    hasIncs = false;
    std::vector<std::string> orders;
    unsigned int ordern = fOrders.size();
    for(unsigned int index=0;index<ordern;index++) {
      std::vector<std::string> words = 
        smanip::words(fOrders[index],"\\n");
      if( (words.size()==2) && (words[0]=="include")) {
        const std::string& file = words[1];
        if(fVerboseLevel) {
          Lib::Out out(fPrinter);
          out << "Lib::Upgrader::upgradeFile :"
              << " include " << Lib::sout(file) 
              << Lib::endl;
        }
        std::vector<std::string> incs;
        if(!fmanip::read(file,incs)) {
          Lib::Out out(fPrinter);
          out << "Lib::Upgrader::Upgrader :"
              << " can't read include upgrade file " << Lib::sout(file) << "."
              << Lib::endl;
          fOrders.clear();
          return;
        }
        unsigned int incn = incs.size();
        for(unsigned int inci=0;inci<incn;inci++) {
          hasIncs = true;
          orders.push_back(incs[inci]);
        }
      } else {
        orders.push_back(fOrders[index]);
      }
    }
    fOrders = orders;
  }
}
//////////////////////////////////////////////////////////////////////////////
Lib::Upgrader::Upgrader(
 const Upgrader& aUpgrader
)
:fPrinter(aUpgrader.fPrinter)
,fVerboseLevel(aUpgrader.fVerboseLevel)
,fOrders(aUpgrader.fOrders)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Upgrader::~Upgrader(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Upgrader& Lib::Upgrader::operator =(
 const Lib::Upgrader& aUpgrader
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVerboseLevel = aUpgrader.fVerboseLevel;
  fOrders = aUpgrader.fOrders;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int Lib::Upgrader::verboseLevel(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Upgrader::upgrade(
 const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fOrders.size()) return true; //Nothing to do.
  bool value;
  if(!dirmanip::isDirectory(aString,value)) return false;
  if(value) return upgradeDirectory(aString);
  else return upgradeFile(aString);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Upgrader::upgradeFile(
 const std::string& aFile
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fVerboseLevel) {
    Lib::Out out(fPrinter);
    out << "Lib::Upgrader::upgradeFile :"
        << " upgrade file " << Lib::sout(aFile) << "..."
        << Lib::endl;
  }
  unsigned int ordern = fOrders.size();
  for(unsigned int index=0;index<ordern;index++) {
    std::vector<std::string> words =
      smanip::words(fOrders[index],"\\n");
    if(fOrders[index].size()<=0) {
    } else if( (fOrders[index].size()>=1) && (fOrders[index][0]=='#') ) {
    } else if( (words.size()==3) && (words[0]=="replace")) {
      if(fVerboseLevel) {
        Lib::Out out(fPrinter);
        out << "Lib::Upgrader::upgradeFile :"
            << " replace " << Lib::sout(words[1]) 
            << " by " << Lib::sout(words[2]) 
            << Lib::endl;
      }
      std::vector<std::string> changes;
      if(!fmanip::replace(aFile,words[1],words[2],changes)) {
        Lib::Out out(fPrinter);
        out << "Lib::Upgrader::upgradeFile :"
            << " upgrade of file " << Lib::sout(aFile) << " fails."
            << Lib::endl;
        return false;
      }
      Lib::Out out(fPrinter);
      unsigned int changen = changes.size();
      for(unsigned int count=0;count<changen;count+=2) {
        out << ">>>>>>>>>>>>> In file " << Lib::sout(aFile) 
            << ", line changed : " << Lib::endl;
        out << changes[count] << Lib::endl;
        out << changes[count+1] << Lib::endl;
      }
    } else if( (words.size()==2) && (words[0]=="remove_line")) {
      if(fVerboseLevel) {
        Lib::Out out(fPrinter);
        out << "Lib::Upgrader::upgradeFile :"
            << " remove_line " << Lib::sout(words[1]) 
            << Lib::endl;
      }
      if(!fmanip::removeLines(aFile,words[1])) {
        Lib::Out out(fPrinter);
        out << "Lib::Upgrader::upgradeFile :"
            << " upgrade of file " << Lib::sout(aFile) << " fails."
            << Lib::endl;
        return false;
      }
    } else if( (words.size()==3) && (words[0]=="if_rm_eol")) {
      if(fVerboseLevel) {
        Lib::Out out(fPrinter);
        out << "Lib::Upgrader::upgradeFile :"
            << " if_rm_eol " << Lib::sout(words[1]) 
            << Lib::endl;
      }
      std::vector<std::string> changes;
      if(!fmanip::if_rm_eol(aFile,words[1],words[2],changes)) {
        Lib::Out out(fPrinter);
        out << "Lib::Upgrader::upgradeFile :"
            << " upgrade of file " << Lib::sout(aFile) << " fails."
            << Lib::endl;
        return false;
      }
      Lib::Out out(fPrinter);
      unsigned int changen = changes.size();
      for(unsigned int count=0;count<changen;count+=2) {
        out << ">>>>>>>>>>>>> In file " << Lib::sout(aFile) 
            << ", line changed : " << Lib::endl;
        out << changes[count] << Lib::endl;
        out << changes[count+1] << Lib::endl;
      }
    } else if( (words.size()==3) && (words[0]=="warn")) {
      std::vector<std::string> found;
      if(!fmanip::found(aFile,words[1],found)) {
        Lib::Out out(fPrinter);
        out << "Lib::Upgrader::upgradeFile :"
            << " upgrade of file " << Lib::sout(aFile) << " fails."
            << Lib::endl;
        return false;
      }
      Lib::Out out(fPrinter);
      unsigned int foundn = found.size();
      for(unsigned int count=0;count<foundn;count++) {
        out << ">>>>>>>>>>>>> In file " << Lib::sout(aFile) 
            << ", have to upgrade by hand line :" 
            << Lib::endl;
        out << found[count] << Lib::endl;
        out << "Action being :" << Lib::endl;
        out << words[2] << Lib::endl;
      }
    } else {
      Lib::Out out(fPrinter);
      out << "Lib::Upgrader::upgradeFile :"
          << " bad formed or unknown request " << Lib::sout(fOrders[index])
          << Lib::endl;
    }
  }
  return true;
}

namespace Lib {

class UpgradeVisitor : public Slash::Core::IDirectoryVisitor {
public:
  virtual bool directory(const std::string& aString,bool& aProcess) {
    aProcess = true;
    std::string p,n,s;
    smanip::pathNameSuffix(aString,p,n,s);
    if(n=="CVS") aProcess = false;
    return true;
  }
  virtual bool file(const std::string& aString) {
    return fUpgrader.upgradeFile(aString);
  }
public:
  UpgradeVisitor(Upgrader& aUpgrader):fUpgrader(aUpgrader){}
  virtual ~UpgradeVisitor() {};
private:
  Upgrader& fUpgrader;
};

}

//////////////////////////////////////////////////////////////////////////////
bool Lib::Upgrader::upgradeDirectory(
 const std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fOrders.size()) return true; //Nothing to do.
  if(fVerboseLevel) {
    Lib::Out out(fPrinter);
    out << "Lib::Upgrader::upgradeDirectory :"
        << " upgrade directory " << Lib::sout(aPath) << "..."
        << Lib::endl;
  }
  Lib::Directory tree;
  tree.setPath(aPath);
  if(!dirmanip::content(tree)) {
    Lib::Out out(fPrinter);
    out << "Lib::Upgrader::upgradeDirectory :"
        << "Can't get files of " << Lib::sout(aPath) << Lib::endl;
    return false;
  }
  UpgradeVisitor visitor(*this);
  return tree.visit(visitor);
}
