// this :
#include <Lib/ZipReader.h>

#include <Lib/Out.h>
#include <Lib/Debug.h>

#ifdef LIB_HAS_ZIP
#include <zlib.h>
#endif

//////////////////////////////////////////////////////////////////////////////
bool Lib::ZipReader::available(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef LIB_HAS_ZIP
  return true;
#else
  return false;
#endif
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::ZipReader::ZipReader(
 const std::string& aFileName
,Slash::Core::IWriter& aPrinter
)
:fPrinter(aPrinter)
,fFileName(aFileName)
,fFile(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::ZipReader");
#ifdef LIB_HAS_ZIP
  fFile = ::gzopen(aFileName.c_str(),"rb");
#else
  Lib::Out out(fPrinter);
  out << "Lib::ZipReader::ZipReader :"
      << " Lib package not reconstructed with zip !" << Lib::endl;
#endif
}
//////////////////////////////////////////////////////////////////////////////
Lib::ZipReader::~ZipReader(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef LIB_HAS_ZIP
  if(fFile) ::gzclose((gzFile*)fFile);
#endif
  Lib::Debug::decrement("Lib::ZipReader");
}
//////////////////////////////////////////////////////////////////////////////
void Lib::ZipReader::disable(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef LIB_HAS_ZIP
  if(!fFile) return; // Already closed.
  ::gzclose((gzFile*)fFile);
  fFile = 0;
#endif
}
//////////////////////////////////////////////////////////////////////////////
void Lib::ZipReader::enable(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef LIB_HAS_ZIP
  if(fFile) return; // Already open.
  fFile = ::gzopen(fFileName.c_str(),"rb");
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::ZipReader::enabled(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFile ? true : false;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::ZipReader::read(
 void* aBuffer
,unsigned int aLength
,int& aReadLength
) 
//////////////////////////////////////////////////////////////////////////////
// aReadLength = 0 when finished (end of file).
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aReadLength = 0;
#ifdef LIB_HAS_ZIP
  if(!fFile) return false;
  int len = ::gzread(fFile, aBuffer, aLength);
  if(len==(-1)) {
    Lib::Out out(fPrinter);
    out << "Lib::ZipReader::read :"
        << " error reading in file \"" << fFileName 
        << "\"." << Lib::endl;
    disable();
    return false;
  } else {
    aReadLength = len;
    return true;
  }
#else
  return false;
#endif
}
